﻿using System;
using System.Collections.Generic;

namespace Jypeli.Assets
{
    /// <summary>
    /// Aivot, jotka seuraavat annettua polkua.
    /// </summary>
    public class PathFollowerBrain : AbstractMoverBrain
    {
        private IList<Vector> path;
        private int wayPointIndex = 0;

        /// <summary>
        /// Polku, eli lista pisteistä joita aivot seuraa.
        /// </summary>
        public IList<Vector> Path
        {
            get { return path; }
            set
            {
                path = value;
                if ( wayPointIndex >= value.Count )
                {
                    wayPointIndex = 0;
                }

                if ( Owner != null )
                    Owner.MoveTo( path[wayPointIndex], Speed );
            }
        }

        /// <summary>
        /// Jos true, palataan alkupisteeseen ja kierretään reittiä loputtomiin.
        /// </summary>
        public bool Loop { get; set; }

        /// <summary>
        /// Luo uudet polunseuraaja-aivot.
        /// </summary>
        public PathFollowerBrain()
            : base()
        {
            Path = new Vector[] { };
        }

        /// <summary>
        /// Luo uudet polunseuraaja-aivot ja asettaa niille nopeuden.
        /// </summary>
        public PathFollowerBrain( double speed )
            : base( speed )
        {
            Path = new Vector[] { };
        }

        protected override void OnAdd( GameObject newOwner )
        {
            if ( newOwner == null ) return;

            if ( Path != null && Path.Count > 0 && Speed > 0 )
            {
                newOwner.MoveTo( path[wayPointIndex], Speed );
            }

            newOwner.ArrivedAt += new Action<GameObject, Vector>( ArrivedAt );
        }

        protected override void OnRemove( GameObject prevOwner )
        {
            if ( prevOwner == null ) return;
            prevOwner.ArrivedAt -= new Action<GameObject, Vector>( ArrivedAt );
        }

        void ArrivedAt( GameObject obj, Vector location )
        {
            if ( Path.Count == 0 || wayPointIndex >= path.Count ) return;
            if ( Vector.Distance( Path[wayPointIndex], location ) > float.Epsilon ) return;
            
            if ( wayPointIndex + 1 >= path.Count )
            {
                if ( Loop ) wayPointIndex = 0;
                else return;
            }
            else
                wayPointIndex++;

            Vector nextWayPoint = Path[wayPointIndex];
            Owner.MoveTo( nextWayPoint, Speed );
        }

        /// <summary>
        /// Luo aivot, jotka seuraavat polkua <c>path</c>.
        /// </summary>
        public PathFollowerBrain( params Vector[] path )
        {
            this.Path = path;
        }
    }
}
