﻿using System;
using System.Diagnostics;
using Microsoft.Xna.Framework.Audio;

using XnaSoundEffect = Microsoft.Xna.Framework.Audio.SoundEffect;

namespace Jypeli
{
    /// <summary>
    /// Ääniefekti. Yhdestä efektistä voi luoda <c>CreateSound</c>-metodilla monta ääntä (<c>Sound</c>),
    /// jotka voivat soida yhtäaikaa. Ääntä ei tarvitse kuitenkaan luoda itse, jos vain kutsuu
    /// <c>Play</c>-metodia.
    /// </summary>
    public class SoundEffect
    {
        private string assetName;
        XnaSoundEffect xnaEffect;

        public TimeSpan Duration { get { DoLoad(); return xnaEffect.Duration; } }

        private void DoLoad()
        {
            if (xnaEffect == null)
            {
                Debug.Assert(assetName != null);
                xnaEffect = Game.Instance.Content.Load<XnaSoundEffect>(assetName);
            }
        }

        internal SoundEffect(string assetName)
        {
            this.assetName = assetName;
            this.xnaEffect = null;
        }

        internal SoundEffect( XnaSoundEffect effect )
        {
            this.assetName = null;
            xnaEffect = effect;
        }

        public Sound CreateSound()
        {
            DoLoad();
            return new Sound( xnaEffect.CreateInstance() );
        }

        public bool Play()
        {
            DoLoad();
            return xnaEffect.Play();
        }

        public bool Play( double volume, double pitch, double pan )
        {
            DoLoad();
            return xnaEffect.Play( (float)volume, (float)pitch, (float)pan );
        }
    }
}
