﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Jypeli
{
    public static class Factory
    {
        struct FactoryKey
        {
            public Type type;
            public string tag;

            public FactoryKey( Type type, string tag )
            {
                this.type = type;
                this.tag = tag;
            }
        }

        public delegate object FactoryMethod();
        static Dictionary<FactoryKey, FactoryMethod> constructors = new Dictionary<FactoryKey, FactoryMethod>();

        public static void AddFactory<T>( string tag, FactoryMethod method )
        {
            FactoryKey key = new FactoryKey( typeof( T ), tag );
            constructors.Add( key, method );
        }

        public static void RemoveFactory<T>( string tag, FactoryMethod method )
        {
            Type type = typeof( T );
            List<FactoryKey> removeKeys = new List<FactoryKey>();

            foreach ( FactoryKey key in constructors.Keys )
            {
                if ( key.type == type && key.tag == tag )
                    removeKeys.Add( key );
            }

            for ( int i = 0; i < removeKeys.Count; i++ )
            {
                constructors.Remove( removeKeys[i] );
            }
        }

        public static T FactoryCreate<T>( string tag )
        {
            return (T)FactoryCreate( typeof( T ), tag );
        }

        internal static object FactoryCreate( Type type, string tag )
        {
            foreach ( FactoryKey key in constructors.Keys )
            {
                if ( key.type == type && key.tag == tag )
                    return constructors[key].Invoke();
            }

            throw new KeyNotFoundException( "Key " + tag + " for class " + type.Name + " was not found." );
        }
    }
}
