using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.Reflection;
using System.Xml;
using System.IO;

namespace Jypeli
{
    public abstract class FileManager
    {
        protected string _currentDir;

        /// <summary>
        /// Nykyinen tyhakemisto.
        /// </summary>
        public string CurrentDirectory
        {
            get { return _currentDir; }
            set { ChDir( value ); }
        }

        public abstract bool ChDir( string path );
        public abstract void MkDir( string path );
        public abstract void RmDir( string path );

        public abstract bool Exists( string fileName );
        public abstract StorageFile Open( string fileName, bool write );
        public abstract void Delete( string fileName );

        public StorageFile Create( string fileName )
        {
            return Open( fileName, true );
        }

        internal static string SanitizeFileName( string fileName )
        {
            StringBuilder newName = new StringBuilder( fileName );
            newName.Replace( "[]", "Array" ).Replace( "()", "Func" );

            for ( int i = 0; i < newName.Length; i++ )
            {
                char c = newName[i];
                if ( !Char.IsLetterOrDigit( c ) && c != '-' && c != '_' && c != '.' )
                    newName[i] = '_';
            }

            return newName.ToString();
        }

        internal void MakeAbsolute( ref string path )
        {
            if ( !Path.IsPathRooted( path ) )
                path = Path.Combine( _currentDir, path );
        }

        public LoadState BeginLoad( string fileName )
        {
            return new LoadState( this, fileName );
        }

        internal LoadState BeginLoad( StorageFile file, string fileName )
        {
            return new LoadState( file, fileName );
        }

        public SaveState BeginSave( string tag )
        {
            return new SaveState( this, tag );
        }

#if JYPELI

        public LoadState BeginLoadContent( string assetName )
        {
            if ( Game.Instance == null )
                throw new InvalidOperationException( "Content can not be loaded here, because the game has not been initialized." );

            StorageFile contentFile = Game.Instance.Content.Load<StorageFile>( assetName );
            return new LoadState( contentFile, assetName );
        }

        public T LoadContent<T>( T obj, string assetName )
        {
            if ( Game.Instance == null )
                throw new InvalidOperationException( "Content can not be loaded here, because the game has not been initialized." );

            byte[] contentData = Game.Instance.Content.Load<byte[]>( assetName );
            MemoryStream contentStream = new MemoryStream( contentData );
            StorageFile contentFile = new StorageFile( assetName, contentStream );

            LoadState state = BeginLoad( contentFile, assetName );
            T result = state.Load<T>( obj, "default" );
            state.EndLoad();
            return result;
        }

#endif

        public T Load<T>( T obj, string fileName )
        {
            LoadState state = BeginLoad( fileName );
            T result = state.Load<T>( obj, "default" );
            state.EndLoad();
            return result;
        }

        public void Save<T>( T obj, string fileName )
        {
            SaveState state = BeginSave( fileName );
            state.Save( obj, typeof( T ), "default" );
            state.EndSave();
        }
    }
}
