#if WINDOWS_PHONE

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.IsolatedStorage;
using System.IO;

namespace Jypeli
{
    public class IsolatedStorageManager : FileManager
    {
        public IsolatedStorageManager()
        {
            _currentDir = ".";
        }

        /// <summary>
        /// Kertoo onko tiedosto olemassa.
        /// </summary>
        /// <param name="fileName">Tiedoston nimi</param>
        /// <returns>Onko olemassa</returns>
        public override bool Exists( string fileName )
        {
            MakeAbsolute( ref fileName );
            return IsolatedStorageFile.GetUserStoreForApplication().FileExists( fileName );
        }

        /// <summary>
        /// Vaihtaa tyhakemistoa.
        /// </summary>
        /// <param name="path">Hakemistopolku.</param>
        /// <returns>Onnistuiko hakemiston vaihtaminen (palauttaa false jos hakemistoa ei ole).</returns>
        public override bool ChDir( string path )
        {
            IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication();

            if ( !storage.DirectoryExists( path ) )
                return false;

            _currentDir = path;
            return true;
        }

        /// <summary>
        /// Luo uuden hakemiston.
        /// </summary>
        /// <param name="path">Hakemistopolku.</param>
        public override void MkDir( string path )
        {
            MakeAbsolute( ref path );
            IsolatedStorageFile.GetUserStoreForApplication().CreateDirectory( path );
        }

        /// <summary>
        /// Tuhoaa hakemiston. Heitt poikkeuksen jos hakemisto ei ole tyhj.
        /// </summary>
        /// <param name="path"></param>
        public override void RmDir( string path )
        {
            MakeAbsolute( ref path );
            IsolatedStorageFile.GetUserStoreForApplication().DeleteDirectory( path );
        }

        /// <summary>
        /// Avaa tiedoston.
        /// </summary>
        /// <param name="fileName">Tiedoston nimi.</param>
        /// <param name="write">Tarvitaanko tiedostoon kirjoitusoikeus.</param>
        /// <returns>StorageFile-olio</returns>
        public override StorageFile Open( string fileName, bool write )
        {
            MakeAbsolute( ref fileName );

            IsolatedStorageFile store = IsolatedStorageFile.GetUserStoreForApplication();
            long freespace = store.AvailableFreeSpace;
            FileMode mode = write ? FileMode.Create : FileMode.Open;
            FileAccess access = write ? FileAccess.ReadWrite : FileAccess.Read;

            if ( freespace > 0 )
                return new StorageFile( fileName, store.OpenFile( fileName, mode, access ) );
            else
                return null;
        }

        /// <summary>
        /// Poistaa tiedoston.
        /// </summary>
        /// <param name="fileName">Tiedoston nimi</param>
        public override void Delete( string fileName )
        {
            MakeAbsolute( ref fileName );
            IsolatedStorageFile.GetUserStoreForApplication().DeleteFile( fileName );
        }
    }
}

#endif
