using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Jypeli
{
    public abstract class TypeHelper
    {
        public static bool InheritsFrom( Type actual, Type expected )
        {
            Type t = actual;

            do
            {
                if ( t == expected ) return true;
                t = t.BaseType;
            } while ( t != null && t != typeof( object ) );

            return false;
        }

        public static string ToString( Type type )
        {
            StringBuilder sb = new StringBuilder( type.AssemblyQualifiedName );
            if ( !type.ContainsGenericParameters ) return sb.ToString();

            sb.Append( '<' );
            Type[] genargs = type.GetGenericArguments();

            for ( int i = 0; i < genargs.Length; i++ )
            {
                sb.Append( TypeHelper.ToString( genargs[i] ) );
                sb.Append( ',' );
            }

            sb[sb.Length - 1] = '>';
            return sb.ToString();
        }

        public static Type Parse( string typeStr )
        {
            int genOpen = typeStr.IndexOf( '<' );
            int genClose = typeStr.IndexOf( '>' );

            if ( genOpen < 0 && genClose < 0 ) return Type.GetType( typeStr );
            if ( genOpen >= 0 && genClose < 0 ) throw new ArgumentException( "Unterminated < in type string: " + typeStr );
            if ( genOpen < 0 && genClose >= 0 ) throw new ArgumentException( "Unexpected > in type string: " + typeStr );

            Type parsedType = Type.GetType( typeStr.Substring( 0, genOpen ) );
            string[] genargStrings = typeStr.Substring( genOpen + 1, genClose - genOpen - 1 ).Split( ',' );
            Type[] genargs = new Type[genargStrings.Length];

            for ( int i = 0; i < genargStrings.Length; i++ )
            {
                genargs[i] = TypeHelper.Parse( genargStrings[i] );
            }

            return parsedType;
        }
    }
}
