#if WINDOWS

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace Jypeli
{
    public class WindowsFileManager : FileManager
    {
        public WindowsFileManager()
        {
            try
            {
                _currentDir = Path.Combine( Path.GetDirectoryName( Application.ExecutablePath ), "Data" );
                if ( !Directory.Exists( _currentDir ) )
                    Directory.CreateDirectory( _currentDir );
            }
            catch ( IOException )
            {
                try
                {
                    _currentDir = Path.Combine( Application.UserAppDataPath, Application.ProductName );
                    _currentDir = Path.Combine( Path.GetDirectoryName( Application.ExecutablePath ), "Data" );
                    if ( !Directory.Exists( _currentDir ) )
                        Directory.CreateDirectory( _currentDir );
                }
                catch ( IOException )
                {
                    MessageBox.Show( "Could not get user data directory.\nDataStorage cannot be used." );
                }
            }
        }

        /// <summary>
        /// Kertoo onko tiedosto olemassa.
        /// </summary>
        /// <param name="fileName">Tiedoston nimi</param>
        /// <returns>Onko olemassa</returns>
        public override bool Exists( string fileName )
        {
            MakeAbsolute( ref fileName );
            return File.Exists( fileName );
        }

        /// <summary>
        /// Vaihtaa tyhakemistoa.
        /// </summary>
        /// <param name="path">Hakemistopolku.</param>
        /// <returns>Onnistuiko hakemiston vaihtaminen (palauttaa false jos hakemistoa ei ole).</returns>
        public override bool ChDir( string path )
        {
            MakeAbsolute( ref path );

            if ( !Directory.Exists( path ) )
                return false;

            _currentDir = path;
            return true;
        }

        /// <summary>
        /// Luo uuden hakemiston.
        /// </summary>
        /// <param name="path">Hakemistopolku.</param>
        public override void MkDir( string path )
        {
            MakeAbsolute( ref path );
            Directory.CreateDirectory( path );
        }

        /// <summary>
        /// Tuhoaa hakemiston. Heitt poikkeuksen jos hakemisto ei ole tyhj.
        /// </summary>
        /// <param name="path"></param>
        public override void RmDir( string path )
        {
            MakeAbsolute( ref path );
            Directory.Delete( path );
        }

        /// <summary>
        /// Avaa tiedoston.
        /// </summary>
        /// <param name="fileName">Tiedoston nimi.</param>
        /// <param name="write">Tarvitaanko tiedostoon kirjoitusoikeus.</param>
        /// <returns>StorageFile-olio</returns>
        public override StorageFile Open( string fileName, bool write )
        {
            MakeAbsolute( ref fileName );
            FileMode mode = write ? FileMode.Create : FileMode.Open;
            FileAccess access = write ? FileAccess.ReadWrite : FileAccess.Read;
            Stream stream = new FileStream( fileName, mode, access );
            return new StorageFile( fileName, stream );
        }

        /// <summary>
        /// Poistaa tiedoston.
        /// </summary>
        /// <param name="fileName">Tiedoston nimi</param>
        public override void Delete( String fileName )
        {
            File.Delete( fileName );
        }
    }
}

#endif
