using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

#if WINDOWS_PHONE
using Microsoft.Xna.Framework.Input.Touch;
#endif

namespace Jypeli.WP7
{
    /// <summary>
    /// Kosketuspaneelin kosketus.
    /// </summary>
    public class Touch
    {
        Vector2 _previousPosition;
        Vector2 _position;

        /// <summary>
        /// Id-tunnus tlle kosketukselle.
        /// </summary>
        public int Id { get; private set; }

        /// <summary>
        /// Kosketuksen paikka ruudulla.
        /// </summary>
        public Vector PositionOnScreen
        {
            get
            {
                return Game.Screen.FromXnaScreenCoordinates( _position );
            }
        }

        /// <summary>
        /// Kosketuksen paikka pelimaailmassa.
        /// </summary>
        public Vector PositionOnWorld
        {
            get
            {
                return Game.Instance.Camera.ScreenToWorld( PositionOnScreen );
            }
        }

        /// <summary>
        /// Kosketuksen liike ruudulla.
        /// </summary>
        public Vector MovementOnScreen
        {
            get
            {
                double horizontalMovement = _position.X - _previousPosition.X;
                double verticalMovement = -( _position.Y - _previousPosition.Y );
                return new Vector( horizontalMovement, verticalMovement );
            }
        }

        /// <summary>
        /// Kosketuksen liike pelimaailmassa.
        /// </summary>
        public Vector MovementOnWorld
        {
            get
            {
                return MovementOnScreen / Game.Instance.Camera.ZoomFactor;
            }
        }

#if WINDOWS_PHONE
        public TouchLocationState State { get; internal set; }

        internal Touch( int id, Vector2 initialPosition, TouchLocationState initialState )
        {
            this.Id = id;
            this._position = this._previousPosition = initialPosition;
            this.State = initialState;
        }

        internal void Update( Vector2 newPosition, TouchLocationState newState )
        {
            _previousPosition = _position;
            _position = newPosition;
            State = newState;
        }
#endif

    }
}
