﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Jypeli.Widgets
{
    /// <summary>
    /// Ikkuna, joka antaa käyttäjän valita yhden annetuista vaihtoehdoista.
    /// </summary>
    public class MultiSelectWindow : Window
    {
        static readonly Key[] keys = { Key.D1, Key.D2, Key.D3, Key.D4, Key.D5, Key.D6, Key.D7, Key.D8, Key.D9, Key.D0 };

        public PushButton[] Buttons { get; private set; }

        public override Color Color
        {
            get
            {
                return base.Color;
            }
            set
            {
                foreach ( var b in Buttons ) b.Color = Color.Darker( value, 40 );
                base.Color = value;
            }
        }

        /// <summary>
        /// Tapahtuma joka tapahtuu kun nappia painetaan.
        /// Ottaa parametrikseen painonapin indeksin (alkaen nollasta).
        /// </summary>
        public event Action<int> ItemSelected;

        /// <summary>
        /// Luo uuden monivalintaikkunan.
        /// </summary>
        /// <param name="question">Kysymys.</param>
        /// <param name="buttonTexts">Nappien tekstit merkkijonoina.</param>
        public MultiSelectWindow(string question, params string[] buttonTexts)
            : base()
        {
            VerticalScrollLayout layout = new VerticalScrollLayout();
            layout.LeftPadding = layout.RightPadding = 20;
            layout.BottomPadding = 30;
            layout.Spacing = 20;
            this.Layout = layout;

            Label kysymysLabel = new Label( question );
            Add( kysymysLabel );

            Buttons = new PushButton[buttonTexts.Length];

            for ( int i = 0; i < buttonTexts.Length; i++ )
            {
                PushButton button = new PushButton( buttonTexts[i] );
                button.Tag = i;
                button.Clicked += new Action( delegate { button_Clicked( (int)button.Tag ); } );
                Add( button );
                Buttons[i] = button;
            }

            AddedToGame += new Action( AddControls );
        }

        public void AddItemHandler( int item, Action handler )
        {
            Buttons[item].Clicked += handler;
        }

        public void RemoveItemHandler( int item, Action handler )
        {
            Buttons[item].Clicked -= handler;
        }

        public void SetButtonColor( Color color )
        {
            for ( int i = 0; i < Buttons.Length; i++ )
            {
                Buttons[i].Color = color;
            }
        }

        void AddControls()
        {
            for ( int i = 0; i < Math.Min( Buttons.Length, keys.Length ); i++ )
            {
                Game.Keyboard.Listen( keys[i], ButtonState.Pressed, button_Clicked, null, i ).InContext( this );
            }
        }

        void button_Clicked( int index )
        {
            if ( ItemSelected != null )
                ItemSelected( index );

            Close();
        }
    }
}
