﻿#region MIT License
/*
 * Copyright (c) 2009-2011 University of Jyväskylä, Department of Mathematical
 * Information Technology.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#endregion

/*
 * Authors: Tero Jäntti, Tomi Karppinen, Janne Nikkanen.
 */

using System;
using AdvanceMath;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Jypeli.Controls;

namespace Jypeli.Widgets
{
    /// <summary>
    /// Aloitusruutu, joka voidaan näyttää ennen pelin käynnistämistä.
    /// </summary>
    public class SplashScreen : Window
    {
        /// <summary>
        /// Kontrolliohje (Aloita peli painamalla Enter / Xbox A).
        /// </summary>
        public String ControlHelp { get; set; }

        /// <summary>
        /// Latausteksti.
        /// </summary>
        public String LoadingText { get; set; }

        /// <summary>
        /// Tapahtuu kun ruudusta poistutaan.
        /// Tee varsinaiset pelin alustukset tämän tapahtuman käsittelijässä.
        /// </summary>
        public event Action GameStarted;

        /// <summary>
        /// Alustaa aloitusruudun.
        /// </summary>
        public SplashScreen( string gameName, string authors, string copyright, string textBody )
            : base( 600, 600 )
        {
            Layout = new VerticalLayout() { Spacing = 50 };

            ControlHelp = "Aloita peli painamalla Enter / Xbox A.";
            LoadingText = "Ladataan...";

            Label yoteksti = InitializeTextDisplay( copyright, 550, Color.Blue );
            Label tekijat = InitializeTextDisplay( authors, 550, Color.Blue );
            Label ohje = InitializeTextDisplay( textBody, 550, Color.Black );
            Label aloitusohje = InitializeTextDisplay( ControlHelp, 550, Color.Green );

            Game.Controls.Keyboard.Listen( Key.Enter, ButtonState.Pressed, BeginLoad, null,aloitusohje ).InContext( this );
            Game.Controls.Keyboard.Listen( Key.Escape, ButtonState.Pressed, Game.Instance.Exit, null ).InContext( this ); ;
            for ( int i = 0; i < Game.Controls.GameControllers.Length; i++ )
            {
                Game.Controls.GameControllers[i].Listen( Button.A, ButtonState.Pressed, BeginLoad, null, aloitusohje ).InContext( this );
                Game.Controls.GameControllers[i].Listen( Button.B, ButtonState.Pressed, Game.Instance.Exit, null ).InContext( this ); ;
            }

            Add( yoteksti );
            Add( tekijat );
            Add( ohje );
            Add( aloitusohje );
        }

        private void BeginLoad( Label aloitusohje )
        {
            aloitusohje.TextColor = Color.Red;
            aloitusohje.Text = LoadingText;
            Timer.SingleShot( 0.1, ResumeLoad );
        }

        private void ResumeLoad()
        {
            Close();
            if ( GameStarted != null )
                GameStarted();
        }

        private Label InitializeTextDisplay( string text, double textWidth, Color textColor )
        {
            Label kentta = new Label(text);

            kentta.HorizontalAlignment = HorizontalAlignment.Left;
            kentta.VerticalAlignment = VerticalAlignment.Top;
            kentta.SizeMode = TextSizeMode.Wrapped;
            kentta.Width = textWidth;
            kentta.Text = text;
            kentta.TextColor = textColor;

            return kentta;
        }
    }
}
