﻿using System;
using Jypeli;
using Jypeli.Widgets;
using Jypeli.Controls;

namespace LevelEditor.Menus
{
    public class ColorMenu : Window
    {
        static readonly Color[] Colors =
    {
        Color.Aqua,
        Color.Beige,
        Color.Black,
        Color.Blue,
        Color.Brown,
        Color.Cyan,
        Color.DarkBlue,
        Color.DarkCyan,
        Color.DarkGray,
        Color.DarkGreen,
        Color.DarkMagenta,
        Color.DarkOrange,
        Color.DarkRed,
        Color.DarkTurquoise,
        Color.DarkViolet,
        Color.ForestGreen,
        Color.Fuchsia,
        Color.Gray,
        Color.Green,
        Color.GreenYellow,
        Color.LightBlue,
        Color.LightCyan,
        Color.LightGray,
        Color.LightGreen,
        Color.LightPink,
        Color.LightYellow,
        Color.Lime,
        Color.LimeGreen,
        Color.Magenta,
        Color.Maroon,
        Color.MediumBlue,
        Color.MediumPurple,
        Color.MediumTurquoise,
        Color.MediumVioletRed,
        Color.Orange,
        Color.OrangeRed,
        Color.Pink,
        Color.Purple,
        Color.Red,
        Color.SkyBlue,
        Color.Transparent,
        Color.Turquoise,
        Color.White,
        Color.Violet,
        Color.Wheat,
        Color.Yellow,
        Color.YellowGreen,
    };

        private Widget[] colorWidgets = new Widget[Colors.Length];

        public event Action<Color> ColorSelected;


        public ColorMenu()
        {
            Layout = new RowLayout()
            {
                TargetWidth = 500,
                LeftPadding = 25,
                RightPadding = 25,
                TopPadding = 25,
                BottomPadding = 25,
                HorizontalSpacing = 10,
                VerticalSpacing = 10
            };

            for ( int i = 0; i < Colors.Length; i++ )
            {
                Widget w = new Widget( 50, 50, Shape.Rectangle )
                {
                    Color = Colors[i],
                    BorderColor = Color.Black,
                    HorizontalSizing = Sizing.FixedSize,
                    VerticalSizing = Sizing.FixedSize,
                };
                Add( w );
                colorWidgets[i] = w;
            }

            AddedToGame += AddControls;
        }

        void AddControls()
        {
            Game.Mouse.ListenOn( this, MouseButton.Left, ButtonState.Pressed, SelectColor, null ).InContext( this );
            Game.Keyboard.Listen( Key.Escape, ButtonState.Pressed, Close, null ).InContext( this );
            Game.Mouse.Listen( MouseButton.Left, ButtonState.Pressed, Close, null ).InContext( this );
        }

        void SelectColor()
        {
            for ( int i = 0; i < colorWidgets.Length; i++ )
            {
                Widget w = colorWidgets[i];

                if ( w.IsInside( Game.Mouse.PositionOnScreen ) )
                {
                    Close();
                    if ( ColorSelected != null )
                        ColorSelected( w.Color );
                    break;
                }
            }
        }
    }
}
