﻿using System;
using Jypeli;
using Jypeli.Widgets;
using Jypeli.Controls;

namespace LevelEditor.Menus
{
    public class ShapeMenu : Window
    {
        static readonly string[] shapeNames =
        {
            "Rectangle",
            "Circle",
            "Triangle",
            "Pentagon",
            "Hexagon",
            "Octagon",
            "Heart",
            "Star",
        };

        private Widget[] shapeWidgets = new Widget[shapeNames.Length];

        public event Action<string> ShapeSelected;


        public ShapeMenu()
        {
            Layout = new RowLayout()
            {
                TargetWidth = 400,
                LeftPadding = 25,
                RightPadding = 25,
                TopPadding = 25,
                BottomPadding = 25,
                HorizontalSpacing = 10,
                VerticalSpacing = 10
            };

            for ( int i = 0; i < shapeNames.Length; i++ )
            {
                Widget w = new Widget( 80, 80, Shape.FromString( shapeNames[i] ) ) { Color = Color.Gray, Tag = shapeNames[i] };
                Add( w );
                shapeWidgets[i] = w;
            }

            AddedToGame += AddControls;
        }

        void AddControls()
        {
            Game.Mouse.ListenOn( this, MouseButton.Left, ButtonState.Pressed, SelectShape, null ).InContext( this );
            Game.Keyboard.Listen( Key.Escape, ButtonState.Pressed, Close, null ).InContext( this );
            Game.Mouse.Listen( MouseButton.Left, ButtonState.Pressed, Close, null ).InContext( this );
        }

        void SelectShape()
        {
            for ( int i = 0; i < shapeWidgets.Length; i++ )
            {
                Widget w = shapeWidgets[i];

                if ( w.IsInside( Game.Mouse.PositionOnScreen ) )
                {
                    Close();
                    if ( ShapeSelected != null )
                        ShapeSelected( w.Tag.ToString() );
                    break;
                }
            }
        }
    }
}
