﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Widgets;
using LevelEditor.Tools;

namespace LevelEditor.Menus
{
    public class ToolMenu : Widget
    {
        static readonly Color SelectedColor = Color.Green;
        static readonly Color DefaultColor = Color.Gray;

        private Tool _selectedTool = null;
        private List<PushButton> buttons = new List<PushButton>();

        public Tool SelectedTool
        {
            get { return _selectedTool; }
            set
            {
                if ( _selectedTool != null ) _selectedTool.OnDeactivated();
                _selectedTool = value;
                _selectedTool.OnActivated();
            }
        }

        public event Action<Tool> ToolSelected;

        public ToolMenu()
            : base( new HorizontalLayout() )
        {
            SelectedTool = Tool.Select;

            CreateButton( Tool.Select, "Select" );
            CreateButton( Tool.Insert, "Insert" );
            ChangeSelection( buttons[0] );
            RefreshLayout();
        }

        PushButton CreateButton( Tool tool, string text )
        {
            PushButton b = new PushButton( text ) { Tag = tool };
            b.Clicked += delegate() { ChangeSelection( b ); };
            Add( b );
            buttons.Add( b );
            return b;
        }

        void ChangeSelection( PushButton selectedButton )
        {
            foreach ( var b in buttons )
            {
                b.Color = DefaultColor;
            }
            selectedButton.Color = SelectedColor;

            SelectedTool = (Tool)selectedButton.Tag;
            if ( ToolSelected != null )
            {
                ToolSelected( SelectedTool );
            }
        }

        public void Select( Tool tool )
        {
            foreach ( var b in buttons )
            {
                if ( ( (Tool)b.Tag ) == tool )
                {
                    ChangeSelection( b );
                    break;
                }
            }
        }
    }
}
