﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;
using Jypeli.LevelEditor;

namespace LevelEditor.Tools
{
    class AngleHandle : Handle
    {
        public AngleHandle( ObjectPair obj )
            : base( obj, 30, 15, Shape.Hexagon, Color.Red )
        {
        }

        public override void OnDrag()
        {
            this.Position = Mouse.PositionOnScreen;
            Vector dragPosition = Camera.ScreenToWorld( this.Position );
            Angle dragAngle = ( dragPosition - SelectedObject.GameObject.Position ).Angle;

            if ( Keyboard.IsCtrlDown() )
            {
                dragAngle.Degrees = Math.Round( dragAngle.Degrees / 45 ) * 45;
            }

            SelectedObject.GameObject.Angle = dragAngle;
            Update();
        }

        public override void OnEndDrag()
        {
            SelectedObject.LevelObject.OverridingProperties.SetValue( "Angle", SelectedObject.GameObject.Angle );
        }

        public override void Update()
        {
            double r = GetObjectRadius( SelectedObject.GameObject );
            this.Position = Camera.WorldToScreen( SelectedObject.GameObject.Position + Vector.FromLengthAndAngle( r + 10, SelectedObject.GameObject.Angle ) );
            this.Angle = SelectedObject.GameObject.Angle;
        }

        private double GetObjectRadius( GameObject obj )
        {
            return Math.Max( obj.Width, obj.Height ) / 2;
        }
    }
}
