﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;
using Jypeli.Controls;

namespace LevelEditor.Tools
{
    public abstract class Tool : ControlContexted
    {
        public static Select Select = new Select();
        public static Insert Insert = new Insert();

        protected static Camera Camera { get { return Game.Instance.Camera; } }
        protected static ScreenView Screen { get { return Game.Screen; } }
        protected static Mouse Mouse { get { return Game.Instance.Mouse; } }
        protected static Keyboard Keyboard { get { return Game.Instance.Keyboard; } }

        ListenContext _context = new ListenContext();
        public ListenContext ControlContext { get { return _context; } }

        public event Action Activated;
        public event Action Deactivated;

        public Tool()
        {
            _context.Disable();
            Activated += delegate { _context.Enable(); };
            Deactivated += delegate { _context.Disable(); };
            Game.AssertInitialized( AddControls );
        }

        internal void OnActivated()
        {
            if ( Activated != null )
                Activated();
        }

        internal void OnDeactivated()
        {
            if ( Deactivated != null )
                Deactivated();
        }

        protected virtual void AddControls()
        {
        }

        public virtual void UpdatePositions()
        {
        }
    }
}
