﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;
using Jypeli.LevelEditor;

namespace LevelEditor.Tools
{
    class VerticalSizeHandle : Handle
    {
        public VerticalSizeHandle( ObjectPair obj )
            : base( obj, 20, 20, Shape.Rectangle, Color.Green )
        {
            SelectedObject = obj;
        }

        public override void OnDrag()
        {
            double tx = this.Position.ScalarProjection( UnitX );
            double ty = Mouse.PositionOnScreen.ScalarProjection( UnitY );
            this.Position = tx * UnitX + ty * UnitY;

            Vector dragPosition = Camera.ScreenToWorld( this.Position );

            if ( Keyboard.IsCtrlDown() )
            {
                dragPosition = LevelEditor.Grid.SnapToLines( dragPosition );
                this.Position = Camera.WorldToScreen( dragPosition );
            }

            double oldTy = SelectedObject.GameObject.Position.ScalarProjection( UnitY );
            double newTy = dragPosition.ScalarProjection( UnitY );
            SelectedObject.GameObject.Height = Math.Abs( newTy - oldTy ) * 2;
        }

        public override void OnEndDrag()
        {
            SelectedObject.LevelObject.OverridingProperties.SetValue( "Size", SelectedObject.GameObject.Size );
        }

        public override void Update()
        {
            this.Position = Camera.WorldToScreen( SelectedObject.GameObject.Position + Vector.FromLengthAndAngle( SelectedObject.GameObject.Height / 2, SelectedObject.GameObject.Angle - Angle.RightAngle ) );
        }
    }
}
