using System;
using System.IO;
using Microsoft.Xna.Framework.Graphics;
using Jypeli.Controls;

#if WINDOWS_PHONE
using Microsoft.Phone.Tasks;
using Microsoft.Xna.Framework.Media;
using System.Windows.Media.Imaging;
using System.IO.IsolatedStorage;
#endif

namespace Jypeli.WP7
{
    /// <summary>
    /// Knnykkkamera-luokka.
    /// Authors: Jaakko Kosonen, Tomi Karppinen
    /// </summary>
    public class Camera
    {

#if WINDOWS_PHONE
        CameraCaptureTask cameraCaptureTask;
        PhotoChooserTask photoChooserTask;
#endif
        /// <summary>
        /// Tapahtuma kun kameralla otettu kuva on valmis.
        /// </summary>
        public event Handler<Image> PhotoReady;

        /// <summary>
        /// Viimeisin kameralla otettu/valittu kuva.
        /// </summary>
        public Image Photo { get; private set; }

        /// <summary>
        /// Mahdollistaa kameran avaamisen kuvanvalitsemisvalikosta.
        /// Tt kautta otetut kuvat eivt viel tallennu muistiin vaikka olisi SavePhotos == true
        /// </summary>        
        public bool ShowCameraWhenChoosingPhoto
        {
            set
            {
#if WINDOWS_PHONE
                photoChooserTask.ShowCamera = value;
#endif
            }
        }

        /// <summary>
        /// Mritt tallennetaanko kameralla otettu kuva puhelimen muistiin.
        /// Kuvan tallentaminen saattaa olla hidasta.
        /// </summary>        
        public bool SavePhotos { private get;  set; }

        /// <summary>
        /// Nimi mill kuvat tallennetaan (Oletuksena JyPeli).
        /// </summary>
        public string SaveName { private get; set; }

        /// <summary>
        /// Konstruktori knnykn kameralle.
        /// </summary>
        public Camera()
        {
#if WINDOWS_PHONE
            cameraCaptureTask = new CameraCaptureTask();
            cameraCaptureTask.Completed += new EventHandler<PhotoResult>( PhotoTaken );
            photoChooserTask = new PhotoChooserTask();
            photoChooserTask.Completed += new EventHandler<PhotoResult>( PhotoChosen );
            ShowCameraWhenChoosingPhoto = false;
            SavePhotos = false;
            SaveName = "JyPeli";
#endif
        }

        /// <summary>
        /// Avaa kameran kuvan ottamista varten.
        /// </summary>
        public void TakePhoto()
        {
#if WINDOWS_PHONE
            try
            {
                cameraCaptureTask.Show();
            }
            catch ( Exception ) { }
#endif
        }

        /// <summary>
        /// Avaa kuvagallerian kuvan valitsemista varten.
        /// </summary>
        public void ChoosePhoto()
        {
#if WINDOWS_PHONE
            try
            {
                photoChooserTask.Show();
            }
            catch ( Exception ) { }
#endif
        }

        void PhotoTaken( object sender, PhotoResult e )
        {
            HandlePhoto( e, true );
        }

        void PhotoChosen( object sender, PhotoResult e )
        {
            HandlePhoto( e, false );
        }

        void HandlePhoto( PhotoResult e, bool save )
        {
#if WINDOWS_PHONE
            if ( e.TaskResult == TaskResult.OK )
            {
                Photo = new Image( Texture2D.FromStream( Game.GraphicsDevice, e.ChosenPhoto ) );
                if ( save && SavePhotos )
                {
                    SavePhoto( e );
                }
                if ( PhotoReady != null )
                {
                    PhotoReady( Photo );
                }
            }
#endif
        }

        private void SavePhoto( PhotoResult e )
        {
#if WINDOWS_PHONE
            try
            {
                string tmp = "tmp";
                var isolatedStorage = IsolatedStorageFile.GetUserStoreForApplication();
                if ( isolatedStorage.FileExists( tmp ) )
                {
                    isolatedStorage.DeleteFile( tmp );
                }

                IsolatedStorageFileStream stream = isolatedStorage.CreateFile( tmp );
                Uri uri = new Uri( SaveName + ";component/" + tmp + ".jpg", UriKind.Relative );
                BitmapImage bitmap = new BitmapImage();
                bitmap.SetSource( e.ChosenPhoto );
                WriteableBitmap writeableBMP = new WriteableBitmap( bitmap );
                Extensions.SaveJpeg( writeableBMP, stream, writeableBMP.PixelWidth, writeableBMP.PixelHeight, 0, 90 );
                stream.Close();

                stream = isolatedStorage.OpenFile( tmp, FileMode.Open, FileAccess.Read );
                MediaLibrary library = new MediaLibrary();
                library.SavePicture( SaveName, stream );
                stream.Close();
            }
            catch ( Exception ) { }
        }
#endif
    }
}
