import sys
import os
from zipdir import zipdir
from installer import create_installer, find_nsis
from os.path import join, exists
from shutil import *
from glob import *
from subprocess import *

trunk_url = "https://svn.cc.jyu.fi/srv/svn/npo/trunk"
npo = "npo_build"
version_name = "latest"
source_package = "jypeli_source.zip"
binary_package = "jypeli.zip"
network_dir = "\\\\eppu.it.jyu.fi\\kurssit\\npo\\"
material_dir = join(network_dir, "material")
release_dir = join(material_dir, version_name)
project_template_dir = join(release_dir, "project_templates")
documentation_dir = join(release_dir, "documentation")


def check_requirements():
	if not os.access(network_dir, os.F_OK):
		raise Exception("Could not find " + network_dir + ". Make sure that it has been mapped.")

	if not os.access(network_dir, os.R_OK|os.W_OK):
		raise Exception("No permission to access " + network_dir)
		
	return_code = call('svn --version', shell=True, stdout=PIPE)
	if return_code != 0:
		raise Exception("svn not found. Please install the subversion command line client.")

	return_code = call("doxygen --help", shell=True, stdout=PIPE)
	if return_code not in [0, 1]:
		raise Exception("Doxygen not found. Please install doxygen.")

	find_nsis()

def create_release_directories():
	try:
		print "Creating release directories..."
		if not exists(material_dir):
			os.mkdir(material_dir)
		if exists(release_dir):
			rmtree(release_dir)
		os.mkdir(release_dir)
		os.mkdir(project_template_dir)
	except IOError as e:
		print "Could not write to the network drive"
		raise

def go_to_temp_dir():
	os.chdir(os.getenv("TEMP"))

def delete_old_files():
	print "Deleting old files..."
	if exists(npo):
		rmtree(npo)
	if exists(binary_package):
		os.remove(binary_package)
	if exists(source_package):
		os.remove(source_package)

def fetch_code():
	return_code = call(['svn', 'export', trunk_url, npo], shell=True)
	if return_code != 0:
		raise Exception("svn export failed")

def create_source_package():
	zipdir(npo, source_package)

def generate_documentation():
	print "Generating documentation..."
	previous_dir = os.getcwd()
	os.chdir(npo)
	os.chdir("etc")
	call("doxygen doxygen.config", shell=True)
	os.chdir(previous_dir)
	
def build_project_templates():
	print "zipping project templates..."
	for template_dir in glob(join(npo, "projektimallit\\*")):
		zipdir(template_dir, template_dir + ".zip")

def build():
	print "Building..."
	previous_dir = os.getcwd()
	os.chdir(npo)
	return_code = call('build.bat')
	if return_code != 0:
		raise Exception("Build not successful!")
	os.chdir(previous_dir)
	zipdir(join(npo, "Compiled\\x86\\Jypeli"), binary_package)
	
def release_files():
	print "Copying files to release directory..."
	copy(source_package, release_dir)
	for zip in glob(join(npo, "projektimallit\\*.zip")):
		copy(zip, project_template_dir)
	copy(binary_package, release_dir)
	copy(join(npo, "installer\\Jypeli_setup.exe"), release_dir)
	copytree(join(npo, "doxygen_output"), documentation_dir)
	
def main():
	try:
		check_requirements()
		create_release_directories()
		go_to_temp_dir()
		delete_old_files()
		fetch_code()
		create_source_package()
		generate_documentation()
		build_project_templates()
		build()
		create_installer(join(npo, "installer\\jypeli.nsi"))
		release_files()
	except KeyboardInterrupt:
		print
		print "RELEASE CANCELLED"
	except Exception as e:
		print
		print "RELEASE NOT SUCCESSFUL:"
		print e
		return 1
	return 0
	

if __name__ == '__main__':
	sys.exit(main())
