﻿using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.WP7;

public class Game : PhysicsGame
{
    static String[] lines = {
                  "           ",
                  "           ",
                  "           ",
                  "   X   X   ",
                  "X          ",
                  "         * ", 
                  "     X   X ",
                  "           ",
                  "           ",
                  "           ",
                  "           ",
                  "*  X   X   ",
                  "X          ",
                  "         * ", 
                  "     X   X ",
                  "           ",
                  "           ",
                  "           ",
                  "   *       ",
                  "   X   X   ",
                  "X          ",
                  "           ", 
                  "     X   X ",
                  "           ",
                  };

    static int tileWidth = 800 / lines[0].Length;
    static int tileHeight = 480 / lines.Length;
    static Image playerImage = LoadImage("ship");
    static Image galaxyImage = LoadImage("galaxy2");
    static Image sombreroImage = LoadImage("sombrero");
    static Image explosionImage = LoadImage("bum");

    SoundEffect blopSound = LoadSoundEffect("blop");
    SoundEffect explosionSound = LoadSoundEffect("exp");
    
    ExplosionSystem explosionSystem;
    PhysicsObject player;

    public override void Begin()
    {
        Level.Background.Image = LoadImage("space");
        Gravity = new Vector(0, -1000);
        NewGame(null);
    }

    void NewGame(Touch touch)
    {
        ClearGameObjects();
        ClearControls();

        player = new PhysicsObject(50, 50, Shape.Circle);
        player.Image = playerImage;
        Add(player);

        explosionSystem = new ExplosionSystem(explosionImage, 50);
        Add(explosionSystem);

        Keyboard.Listen(Key.Up, ButtonState.Pressed, MovePlayer, "Move up", player, new Vector(0, 500));
        Keyboard.Listen(Key.Down, ButtonState.Pressed, MovePlayer, null, player, new Vector(0, -500));
        Keyboard.Listen(Key.Left, ButtonState.Pressed, MovePlayer, null, player, new Vector(-500, 0));
        Keyboard.Listen(Key.Right, ButtonState.Pressed, MovePlayer, null, player, new Vector(500, 0));
        TouchPanel.Listen(ButtonState.Pressed, NewGame, null);
        Accelerometer.Calibration = AccelerometerCalibration.ZeroAngle;
        Accelerometer.ListenAnalog(AccelerometerSensitivity.Realtime, ChangeGravity, null);

        TileMap tiles = TileMap.FromStringArray(lines);
        tiles['X'] = CreateGalaxy;
        tiles['*'] = CreateSombrero;
        tiles.Insert(tileWidth, tileHeight);
        Level.CreateBorders();
        Camera.ZoomToLevel();
    }

    public void MovePlayer(PhysicsObject player, Vector force)
    {
        player.Hit(force);
    }

    PhysicsObject CreateGalaxy()
    {
        PhysicsObject galaxy = PhysicsObject.CreateStaticObject(tileWidth, tileHeight);
        galaxy.Color = Color.LightBlue;
        AddCollisionHandler(galaxy, CollidedWithGalaxy);
        galaxy.Image = galaxyImage;
        return galaxy;
    }

    PhysicsObject CreateSombrero()
    {
        PhysicsObject sombrero = PhysicsObject.CreateStaticObject(tileWidth, tileHeight);
        sombrero.Color = Color.Yellow;
        sombrero.Image = sombreroImage;
        AddCollisionHandler(sombrero, CollidedWithSombrero);
        return sombrero;
    }

    void CollidedWithGalaxy(PhysicsObject galaxy, PhysicsObject target)
    {
        Phone.Vibrate(40);
        blopSound.Play();
    }

    void CollidedWithSombrero(PhysicsObject sombrero, PhysicsObject target)
    {
        Phone.Vibrate(400);
        explosionSound.Play();
        explosionSystem.AddEffect(target.X, target.Y, 50);
        sombrero.Destroy();
    }

    void ChangeGravity(AnalogState s)
    {
        Gravity = s.StateVector * 2000;
    }
}