using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace Pong
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Pong : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D myTexture;
        // our pong entities
        Rectangle blueBar;
        Rectangle redBar;
        Rectangle ball; // since theres no "circle" class in XNA, well simulate it with a bounding rectangle box
        // ball speed
        Vector2 ballVelocity = Vector2.Zero;
        // scores
        int blueScore = 0;
        int redScore = 0;

        SpriteFont font;

        // source rectangles of our graphics
        Rectangle blueSrcRect = new Rectangle( // blue bar src rectangle
              0, // upper left corner x-coordinate of the blue bar inside the spriteSheet
              0, // upper left corner y-coordinate
              32, // width
              128); // height
        Rectangle redSrcRect = new Rectangle( // red bar src rectangle
              32, // upper left corner x-coordinate of the red bar inside the spriteSheet
              0,
              32,
              128);
        Rectangle ballSrcRect = new Rectangle( // ball src rectangle
              64, // upper left corner x-coordinate of the ball inside the spriteSheet
              0,
              32,
              32);

        // our pong clone textures
        Texture2D grass;
        Texture2D spriteSheet;
        // our sound effects
        SoundEffect ballBounce;
        SoundEffect playerScored;
        
        public Pong()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Pre-autoscale settings.
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 480;
            graphics.IsFullScreen = true; // so the battery meter and other stuff are not displayed
            graphics.SupportedOrientations = DisplayOrientation.LandscapeLeft; // forcing our game to support only one landscape mode so the screen doesn't rotate if the two players are shaking the device

            graphics.ApplyChanges();
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // initializing our entities
            blueBar = new Rectangle(
                  32, // x coordinate of the upper left corner of our rectangle
                  GraphicsDevice.Viewport.Bounds.Height / 2 - 64, // y coordinate of the upper left corner
                  32, // its width
                  128); // and its height
            redBar = new Rectangle(
                  GraphicsDevice.Viewport.Bounds.Width - 64, // x coordinate of the upper left corner of our rectangle
                  GraphicsDevice.Viewport.Bounds.Height / 2 - 64, // y coordinate of the upper left corner
                  32, // its width
                  128); // and its height
            ball = new Rectangle(
                  GraphicsDevice.Viewport.Bounds.Width / 2 - 16, // x coordinate of the upper left corner of our rectangle
                  GraphicsDevice.Viewport.Bounds.Height / 2 - 16, // y coordinate of the upper left corner
                  32, // its width
                  32); // and its height

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // load our textures from the Content Pipeline
            grass = Content.Load<Texture2D>(@"Textures/Grass");
            spriteSheet = Content.Load<Texture2D>(@"Textures/Objects");

            // load our sound effects from the Content Pipeline
            ballBounce = Content.Load<SoundEffect>(@"Sounds/Bounce");
            playerScored = Content.Load<SoundEffect>(@"Sounds/Supporters");
            // load our score font
            font = Content.Load<SpriteFont>(@"ScoreFont");

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // for each detected touch
            TouchCollection touchCollection = TouchPanel.GetState();
            foreach (TouchLocation touchLocation in touchCollection)
            {
                // upper left corner touche detection
                if (touchLocation.Position.X < 150
                      && touchLocation.Position.Y < 150)
                    blueBar.Y -= 10; // move the blue bar up
                // lower left corner
                else if (touchLocation.Position.X < 150
                      && touchLocation.Position.Y > 240)
                    blueBar.Y += 10; // move the blue bar down

                // upper right corner touche detection
                if (touchLocation.Position.X > 400
                      && touchLocation.Position.Y < 200)
                    redBar.Y -= 10; // move the red bar up
                // lower right corner
                else if (touchLocation.Position.X > 400
                      && touchLocation.Position.Y > 240)
                    redBar.Y += 10; // move the red bar down

                // ball touch
                if (touchLocation.Position.X < 420
                      && touchLocation.Position.X > 380
                      && touchLocation.Position.Y < 260
                      && touchLocation.Position.Y > 220)
                    InitBall();
            }

            // limit the bars movement to the screen bounds
            if (redBar.Y < 0) // upper bound
                redBar.Y = 0; // limit
            if (blueBar.Y < 0)
                blueBar.Y = 0;
            if (redBar.Y + redBar.Height > GraphicsDevice.Viewport.Bounds.Height)
                redBar.Y = GraphicsDevice.Viewport.Bounds.Height - redBar.Height;
            if (blueBar.Y + blueBar.Height > GraphicsDevice.Viewport.Bounds.Height)
                blueBar.Y = GraphicsDevice.Viewport.Bounds.Height - blueBar.Height;

            // move the ball
            ball.X += (int)ballVelocity.X;
            ball.Y += (int)ballVelocity.Y;

            // collision handling
            if (ball.Y < 0 || // if the ball reach the upper bound of the screen
                  ball.Y + ball.Height > GraphicsDevice.Viewport.Bounds.Height) // or the lower one
            {
                ballVelocity.Y = -ballVelocity.Y; // make if bounce by inverting the Y velocity
                ballBounce.Play(); // bounce sound
            }

            if (ball.Intersects(redBar) || // if the ball collide with blue bar
                  ball.Intersects(blueBar)) // or the red one
            {
                ballVelocity.X = -ballVelocity.X; // make if bounce by inverting the X velocity
                ballBounce.Play(); // bounce sound
            }

            // score handling
            if (ball.X < 0) // red scores
            {
                redScore++;
                playerScored.Play(); // hurray !
                InitBall(); // re-init the ball
            }
            else if (ball.X + ball.Width > GraphicsDevice.Viewport.Bounds.Width) // blue scores
            {
                blueScore++;
                playerScored.Play(); // hurray !
                InitBall(); // re-init the ball
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue); // clear all the screen with a blue color

            // draw the grass background
            spriteBatch.Begin();
            spriteBatch.Draw(
                  grass, // use the grass texture
                  GraphicsDevice.Viewport.Bounds, // stretch the texture to the whole screen
                // GraphicsDevice.Viewport.Bounds is Rectangle corresponding to the actual viewport (meaning the entire screen no matter the resolution), only available as of XNA 4.0
                  Color.White);
            spriteBatch.End();

            // draw the entities (bars and ball)
            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend); // setup alpha blend to support transparency
            // draw the red bar
            spriteBatch.Draw(
                  spriteSheet, // use the sprites texture
                  redBar, // the rectangle where to draw the bar on the screen
                  redSrcRect, // the source rectangle of the bar inside the sprite sheet
                  Color.White);
            // draw the blue bar
            spriteBatch.Draw(
                  spriteSheet,
                  blueBar,
                  blueSrcRect,
                  Color.White);
            // draw the ball
            spriteBatch.Draw(
                  spriteSheet,
                  ball,
                  ballSrcRect,
                  Color.White);
            spriteBatch.End();

            // draw the score
            spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend);
            spriteBatch.DrawString( // draw our score string
                  font, // our score font
                  blueScore.ToString() + " - " + redScore.ToString(), // building the string
                  new Vector2( // text position
                  GraphicsDevice.Viewport.Bounds.Width / 2 - 25, // half the screen and a little to the left
                  10.0f),
                  Color.Yellow); // yellow text
            spriteBatch.End();

        }

        private void InitBall()
        {
            int speed = 5; // default velocity
            Random rand = new Random();

            // randomize the ball orientation
            switch (rand.Next(4))
            {
                case 0: ballVelocity.X = speed; ballVelocity.Y = speed; break;
                case 1: ballVelocity.X = -speed; ballVelocity.Y = speed; break;
                case 2: ballVelocity.X = speed; ballVelocity.Y = -speed; break;
                case 3: ballVelocity.X = -speed; ballVelocity.Y = -speed; break;
            }

            // initialize the ball to the center of the screen
            ball.X = GraphicsDevice.Viewport.Bounds.Width / 2 - ball.Width / 2;
            ball.Y = GraphicsDevice.Viewport.Bounds.Height / 2 - ball.Height / 2;
        }
    }
}
