﻿using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Assets;
using Jypeli.Widgets;
using Jypeli.WP7;

/*
 * Author: Panu Lappalainen, 2009
 * Windows Phone 7 -version: Antti-Jussi Lakanen, 2011
 * Tombstoning: Tomi Karppinen, 2011
 */

namespace Hyppija
{
    public class Hyppija : PhysicsGame
    {
        Widget aloitusRuutu;

        [Save] public PlatformCharacter kalle;
        [Save] public PhysicsObject vesi;
        const double liikeNopeus = 500;
        const double hyppyNopeus = 1500;
        [Save] public PhysicsObject maali;
        [Save] public bool pelaajaMaalissa = false;
        [Save] public bool pelaajaKuoli = false;

#if WINDOWS_PHONE
        Label vasemmalleLiikutusPainike;
        Label oikealleLiikutusPainike;
        Label hyppyPainike;

#endif
        [Save] public double viimeisenTalonX;
        [Save] public double viimeisenTalonY;
        [Save] public double kameranNopeus = 3;
        [Save] public const int Taloja = 10;
        [Save] public IntMeter pisteLaskuri;
        Timer ajastin;
        Timer pisteAjastin;
        Label pisteNaytto;

        //HighScoreList Top10;
        bool aloitaAlusta = false;

        Image kallenKuva = LoadImage("kallepaik");
        Image[] kallenAnimaatioKuvat = LoadImages("kalle01", "kalle02", "kalle03", "kalle04", "kalle05", "kalle06", "kalle07", "kalle08");
        Image[] talonKuvat = LoadImages("talo1", "talo2", "talo3", "talo4", "talo5", "talo6");
        Image viemarinKuva = LoadImage("viemari");

        SoundEffect hyppyAani = LoadSoundEffect( "sounds\\hyppy3" );
        SoundEffect splashAani = LoadSoundEffect( "sounds\\splash" );

        Font otsikkoFontti = LoadFont("Pericles");


        public override void Begin()
        {
            Phone.DisplayOrientation = DisplayOrientation.Landscape;
            LuoKentta();
            LisaaMaali();

            NaytaAloitusRuutu();
        }

        public override void Continue()
        {
            Phone.DisplayOrientation = DisplayOrientation.Landscape;
            LuoKentta();
            LisaaMaali();

            AloitaPeli();
        }

        void NaytaAloitusRuutu()
        {
#if WINDOWS_PHONE
            var asettelu = new VerticalLayout { Spacing = 2, TopPadding = 5, BottomPadding = 5, LeftPadding = 5, RightPadding = 5 };
#else
            var asettelu = new VerticalLayout { Spacing = 50, TopPadding = 50, BottomPadding = 50, LeftPadding = 50, RightPadding = 50 };
#endif
            aloitusRuutu = new Widget(asettelu) { Color = new Color(200, 200, 255, 220), BorderColor = Color.Black };
            Add(aloitusRuutu);

            aloitusRuutu.Add(new Label("Panu Lappalainen and "));
            aloitusRuutu.Add(new Label("University of Jyväskylä present:"));
            aloitusRuutu.Add(new Label("Hyppijä") { Font = otsikkoFontti });
            aloitusRuutu.Add(new Widget(10, 20) { Color = Color.Transparent }); // Tyhjää tilaa
#if WINDOWS_PHONE
            aloitusRuutu.Add(new Label("Tap on the screen"));
#else
            aloitusRuutu.Add(new Label("Press enter"));
#endif

            Keyboard.Listen(Key.Enter, ButtonState.Pressed, NaytaTarinaRuutu, null);
            TouchPanel.Listen(ButtonState.Pressed, NaytaTarinaRuutu, null);
        }

        void NaytaTarinaRuutu(Touch kosketus)
        {
            NaytaTarinaRuutu();
        }

        void NaytaTarinaRuutu()
        {
            aloitusRuutu.Destroy();
            ClearControls();

#if WINDOWS_PHONE
            var asettelu = new VerticalLayout { Spacing = 5, TopPadding = 5, BottomPadding = 5, LeftPadding = 5, RightPadding = 5 };
#else
            var asettelu = new VerticalLayout { Spacing = 50, TopPadding = 50, BottomPadding = 50, LeftPadding = 50, RightPadding = 50 };
#endif
            aloitusRuutu = new Widget(asettelu) { Color = new Color(200, 200, 255, 220), BorderColor = Color.Black };
            Add(aloitusRuutu);

            aloitusRuutu.Add(new Label("A flood has taken over the city."));
            aloitusRuutu.Add(new Label("You must get to the outskirt of the city,"));
            aloitusRuutu.Add(new Label("find a pump and activate it."));
            aloitusRuutu.Add(new Widget(10, 20) { Color = Color.Transparent }); // Tyhjää tilaa
#if WINDOWS_PHONE
            aloitusRuutu.Add(new Label("Tap on the screen"));
#else
            aloitusRuutu.Add(new Label("Press enter"));
#endif

            Keyboard.Listen(Key.Enter, ButtonState.Pressed, AloitaPeli, null);
            TouchPanel.Listen(ButtonState.Pressed, AloitaPeli, null);
        }

        void AloitaPeli(Touch kosketus)
        {
            AloitaPeli();
        }

        void AloitaPeli()
        {
            if ( aloitusRuutu != null ) aloitusRuutu.Destroy();

            LuoPelaaja();
            LuoPistelaskuri();
            //Top10 = HighScoreList.LoadOrCreate(this, "Top10Hyppija.txt", 10);
            AloitaUusiPeli();
        }

        void AloitaUusiPeli(Touch kosketus)
        {
            AloitaUusiPeli();
        }

        void AloitaUusiPeli()
        {
            kameranNopeus = 3;
            if (pelaajaKuoli)
            {
                LuoPelaaja(); 
            }
            PelaajaAlkuun();
            pelaajaKuoli = false;
            pelaajaMaalissa = false;
            pisteLaskuri.Value = 0;
            aloitaAlusta = true;

            ClearControls();
            LisaaKosketusKontrollit();
            LisaaNappaimet();

            Camera.ZoomFactor = Screen.Height / Level.Height;

            MessageDisplay.Add("Hurry...");
        }

        void LuoPelaaja()
        {
            kalle = new PlatformCharacter(80, 80);
            Image[] kavelyVasemmalle = Image.Mirror(kallenAnimaatioKuvat);
            kalle.RightWalkingAnimation = new Animation(kallenAnimaatioKuvat);
            kalle.LeftWalkingAnimation = new Animation(kavelyVasemmalle);
            kalle.RightIdleAnimation = new Animation(kallenKuva);
            kalle.LeftIdleAnimation = new Animation(Image.Mirror(kallenKuva));

            Add(kalle);
            AddCollisionHandler(kalle, VeteenTormays);
            AddCollisionHandler(kalle, MaaliinPaasty);
        }


        void LuoKentta()
        {
            Gravity = new Vector(0, -1000);
            int x = LuoTalot();
            //LuoTalo(0, -500);
            vesi = PhysicsObject.CreateStaticObject(x + Screen.Width * 3, 300);
            vesi.Color = Color.Blue;
            Add(vesi);
            vesi.Y = -500;
            vesi.X = x / 2;
        }

        private int LuoTalot()
        {
            int x = -100;
            int w;
            int h;
            int dx;
            Random r = new Random(1);

            for (int i = 0; i < Taloja; i++)
            {
                w = r.Next(200, 400); // talon leveys arvotaan kokonaislukuna 200 ja 400 väliltä
                h = r.Next(50, 200); // talon korkeus arvotaan kokonaislukuna 50 ja 200 väliltä
                LuoTalo(x, h - 300, w, 2 * h + 300);
                dx = 200; // Talojen välinen etäisyys?
                if (r.Next(100) > 90)
                    dx = r.Next(100, 250);
                x += w + dx;
            }
            return x;
        }

        void LuoTalo(int x, int y, int w, int h)
        {
            int ac = 255 / 2 - 10, lc = 255 / 2 + 10;
            Color talonvari = new Color((Byte)RandomGen.NextInt(ac, lc), (Byte)RandomGen.NextInt(ac, lc), (Byte)RandomGen.NextInt(ac, lc));

            PhysicsObject talo = PhysicsObject.CreateStaticObject(w, h);
            talo.Color = talonvari;
            talo.X = x;
            talo.Y = y;
            Add(talo);

            viimeisenTalonX = x;
            viimeisenTalonY = y + talo.Height / 2;

            int kuvaIndeksi = RandomGen.NextInt(0, talonKuvat.Length);
            talo.Image = talonKuvat[kuvaIndeksi];
        }

        void LisaaNappaimet()
        {
            // Pelaajan näppäimet
            Keyboard.Listen(Key.Left, ButtonState.Down, liikuta, "Moves left", kalle, new Vector(-liikeNopeus, 0));
            Keyboard.Listen(Key.Right, ButtonState.Down, liikuta, "Moves right", kalle, new Vector(liikeNopeus, 0));
            Keyboard.Listen(Key.Up, ButtonState.Pressed, hyppaa, "Jumps", hyppyNopeus);
            Keyboard.Listen(Key.F5, ButtonState.Pressed, AloitaUusiPeli, "New game");

            Keyboard.Listen( Key.F1, ButtonState.Pressed, SaveGame, "Save" );
            Keyboard.Listen( Key.F3, ButtonState.Pressed, LoadGame, "Load" );
        }

        void SaveGame()
        {
            SaveGame( "hyppija.xml" );
        }

        void LoadGame()
        {
            LoadGame( "hyppija.xml" );
        }

        void LisaaKosketusKontrollit()
        {
#if WINDOWS_PHONE
            TouchPanel.Clear();

            double marginaali = 10;
            Color vari = new Color(30, 30, 30, 30);

            vasemmalleLiikutusPainike = new Label(Screen.Width * 0.30, Screen.Height * 0.4, "<-") { Color = vari, BorderColor = Color.Gray, TextColor = Color.White };
            vasemmalleLiikutusPainike.X = Screen.Left + marginaali + vasemmalleLiikutusPainike.Width / 2;
            vasemmalleLiikutusPainike.Y = Screen.Bottom + marginaali + vasemmalleLiikutusPainike.Height / 2;
            Add(vasemmalleLiikutusPainike);

            oikealleLiikutusPainike = new Label(Screen.Width * 0.30, Screen.Height * 0.4, "->") { Color = vari, BorderColor = Color.Gray, TextColor = Color.White };
            oikealleLiikutusPainike.X = Screen.Right - marginaali - oikealleLiikutusPainike.Width / 2;
            oikealleLiikutusPainike.Y = Screen.Bottom + marginaali + oikealleLiikutusPainike.Height / 2;
            Add(oikealleLiikutusPainike);

            hyppyPainike = new Label(Screen.Width * 0.30, Screen.Height * 0.3, "JUMP") { Color = vari, BorderColor = Color.Gray, TextColor = Color.White };
            hyppyPainike.X = Screen.Center.X;
            hyppyPainike.Y = Screen.Bottom + marginaali + hyppyPainike.Height / 2;
            Add(hyppyPainike);

            TouchPanel.ListenOn(vasemmalleLiikutusPainike, ButtonState.Down, LiikutaVasemmalle, null);
            TouchPanel.ListenOn(oikealleLiikutusPainike, ButtonState.Down, LiikutaOikealle, null);
            TouchPanel.ListenOn(hyppyPainike, ButtonState.Pressed, Hyppaa, null);
#endif
        }

        void LiikutaVasemmalle(Touch kosketus)
        {
            liikuta(kalle, new Vector(-liikeNopeus, 0));
        }

        void LiikutaOikealle(Touch kosketus)
        {
            liikuta(kalle, new Vector(liikeNopeus, 0));
        }

        void Hyppaa(Touch kosketus)
        {
            hyppaa(hyppyNopeus);
        }

        void liikuta(PlatformCharacter pelaaja, Vector voima)
        {
            kalle.Walk(voima.X);
        }

        void hyppaa(double voima)
        {
            if (kalle.Jump(voima))
            {
                Phone.Vibrate(40);
                hyppyAani.Play();
            }
        }

        void VeteenTormays(PhysicsObject tormaaja, PhysicsObject kohde)
        {
            if (kohde == vesi)
            {
                splashAani.Play();
                MessageDisplay.Add("You drowned!");
                Phone.Vibrate(400);
                Kuole();
            }
        }

        void NaytaTop()
        {
            //Top10.ScreenList.X = 0;
            //Top10.ScreenList.X = -100;
            //Top10.ScreenList.ItemColor = Color.Red;
            //Top10.ScreenList.BackGroundColor = Color.Black;
            //Top10.CongratulationText = "Onneksi olkoon! Pistemääräsi oli {1}, jolla pääset sijalle {0}!";
            //Top10.Show(pisteLaskuri);
        }

        void LisaaMaali()
        {
            maali = PhysicsObject.CreateStaticObject(50, 50, Shape.Circle);
            maali.Tag = "maali";

            maali.X = viimeisenTalonX;
            maali.Y = viimeisenTalonY + maali.Height / 2;
            maali.Image = viemarinKuva;
            Add(maali);
        }

        void MaaliinPaasty(PhysicsObject tormaaja, PhysicsObject kohde)
        {
            if (kohde == maali && !pelaajaMaalissa)
            {
                pisteAjastin.Stop();
                Keyboard.Disable(Key.Up); Keyboard.Disable(Key.Left); Keyboard.Disable(Key.Right);

                pelaajaMaalissa = true;
                MessageDisplay.Add("You saved the city!!!");
                ajastin = new Timer();
                ajastin.Interval = 3;
                ajastin.Timeout += UusiPeli;
                ajastin.Start();
                kameranNopeus *= 1.5;
            }
        }

        void UusiPeli()
        {
            if ( pelaajaKuoli ) return;
            Keyboard.EnableAll();
            pisteAjastin.Start();
            MessageDisplay.Add( "Another flood reached the city!" );
            MessageDisplay.Add( "Hurry..." );
            pelaajaMaalissa = false;
            PelaajaAlkuun();

            kameranNopeus += 0.00001;
            ajastin.Stop();
        }

        void PelaajaAlkuun()
        {
            kalle.X = -150;
            kalle.Y = 50;
            Camera.Position = new Vector(0, 0);
        }

        void LuoPistelaskuri()
        {
            // Luodaan pistelaskuri
            pisteLaskuri = new IntMeter(0);

            // luodaan pistelaskunäyttö
            pisteNaytto = new Label();
#if WINDOWS_PHONE
            pisteNaytto.Font = Font.DefaultSmallBold;
#else
            pisteNaytto.Font = Font.DefaultLargeBold;
#endif
            pisteNaytto.HorizontalAlignment = HorizontalAlignment.Right;
            pisteNaytto.Title = "Score";
            pisteNaytto.X = Screen.RightSafe - 100;
            pisteNaytto.Y = Screen.TopSafe - pisteNaytto.Height / 2;
            pisteNaytto.BindTo(pisteLaskuri);
            Add(pisteNaytto);    // lisätään peliin

            // Luodaan ajastin, joka lisää pisteitä tietyin väliajoin
            pisteAjastin = new Timer();
            pisteAjastin.Interval = 0.1;
            pisteAjastin.Timeout += lisaaPiste;
            pisteAjastin.Start();
        }

        void lisaaPiste()
        {
            if ( !pelaajaKuoli )
                pisteLaskuri.Value++;
        }

        void Kuole()
        {
            //kalle.Destroy();
            kalle.IsVisible = false;
            pelaajaKuoli = true;

            TouchPanel.Clear();
#if WINDOWS_PHONE
            vasemmalleLiikutusPainike.Destroy();
            oikealleLiikutusPainike.Destroy();
            hyppyPainike.Destroy();
            TouchPanel.Listen(ButtonState.Pressed, AloitaUusiPeli, null);
#else
            Keyboard.Listen(Key.Enter, ButtonState.Pressed, AloitaUusiPeli, null);
#endif
            // NaytaTop();
        }

        protected override void Update(Time gameTime)
        {
            base.Update(gameTime);

            if (kalle == null) return;

            if (aloitaAlusta)
            {
                aloitaAlusta = false;
            }

            if (pelaajaMaalissa || pelaajaKuoli) return;

            //double muutos = gameTime.SinceStartOfGame.Ticks * kameranNopeus / 5;
            Camera.Position = new Vector(Camera.Position.X + kameranNopeus, Camera.Position.Y);

            if (Camera.Position.X > kalle.Position.X + 1000)
            {
                MessageDisplay.Add("You drowned!");
                Kuole();
                // NaytaTop();
            }
        }
    }
}