using System;
using System.Collections.Generic;
using System.Linq;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Assets;
using Jypeli.Widgets;
using Jypeli.WP7;

/*
 * Author: Antti-Jussi Lakanen
 * 
 * Todo: - Pelilogiikka kesken
 *       - Jypeli.Effects.ExplosionSystem ei toimi?
 */
namespace Myyrat
{
    public class KurkkivatMyyrat : PhysicsGame
    {
        Timer myyraAjastin;
        List<PhysicsObject> myyrat;
        const double MYYRAN_KOKO = 30;
        ExplosionSystem rajahdys;
        IntMeter pistelaskuri;
        Label pistenaytto;

        public override void Begin()
        {
            Level.Height = 800;
            Level.Width = 480;
            Camera.ZoomToLevel();
            rajahdys = new ExplosionSystem(LoadImage("explosion"), 40);
            Add(rajahdys);

            // Pistelaskuri ja nytt
            pistelaskuri = new IntMeter(0);
            pistenaytto = new Label();
            pistenaytto.BindTo(pistelaskuri);
            pistenaytto.Position = new Vector(Screen.LeftSafe + 50, Screen.TopSafe - 50);
            Add(pistenaytto);

            // Myyrajastin
            myyraAjastin = new Timer();
            myyraAjastin.Interval = 1;
            myyraAjastin.Timeout += LisaaMyyra;

            AloitaPeli();
        }


        public void LisaaMyyra()
        {
            double r = MYYRAN_KOKO / 2;
            double x = RandomGen.NextDouble(Level.Left + r, Level.Right - r);
            double y = RandomGen.NextDouble(Level.Bottom + r, Level.Top - r);
            
            // Ei list myyri pllekkin
            while (true)
            {
                for (int i = 0; i < myyrat.Count; i++)
                {
                    if (Etaisyys(myyrat[i].Position, new Vector(x, y)) < 3 * r)
                    {
                        x = RandomGen.NextDouble(Level.Left + r, Level.Right - r);
                        y = RandomGen.NextDouble(Level.Bottom + r, Level.Top - r);
                        i = -1;
                    }
                }
                break;
            }
            LisaaMyyra(x, y, r);
        }

        /// @example
        /// <pre name="test">
        /// Vector v1 = new Vector(0, 0);
        /// Vector v2 = new Vector(3, 4);         
        /// KurkkivatMyyrat.Etaisyys(v1, v2) ~~~ 5;
        /// Vector v3 = new Vector(-4, -3);
        /// KurkkivatMyyrat.Etaisyys(v1, v3) ~~~ 5;
        /// </pre>
        /// @endexample
        public static double Etaisyys(Vector v1, Vector v2)
        {
            double dx = v1.X - v2.X;
            double dy = v1.Y - v2.Y;
            return Math.Sqrt(dx * dx + dy * dy);
        }

        public void LisaaMyyra(double x, double y, double r)
        {
            PhysicsObject myyra = PhysicsObject.CreateStaticObject(r * 2, r * 2, Shape.Circle);
            myyra.Position = new Vector(x, y);
            //Timer.SingleShot(2, delegate() { MyyraMissattu(myyra); });
            Add(myyra);
            myyrat.Add(myyra);
            // MessageDisplay.Add(myyrat.Count.ToString());
        }

        public void MyyraMissattu(PhysicsObject myyra)
        {
            if (myyra == null) return;            
            myyra.Destroy();
            myyrat.Remove(myyra);

        }

        public void AloitaPeli()
        {
            pistelaskuri.Value = 0;
            
            myyrat = new List<PhysicsObject>();
            Mouse.IsCursorVisible = true;
#if WINDOWS_PHONE
            TouchPanel.Listen(ButtonState.Pressed, NayttoaPainettu, null); // Ei toimi emulaattorissa?
#endif
#if WINDOWS
            Mouse.Listen(MouseButton.Left, ButtonState.Pressed, HiirenVasenPainettu, null);
#endif
            myyraAjastin.Start();
        }

        void HiirenVasenPainettu()
        {
            // rajahdys.AddEffect(Mouse.PositionOnScreen.X, Mouse.PositionOnScreen.Y, 40);
            Explosion rajahdys = new Explosion(30);
            rajahdys.Position = Mouse.PositionOnWorld;
            rajahdys.ShockwaveColor = Color.Yellow;
            Add(rajahdys);
        }

#if WINDOWS_PHONE
        public void NayttoaPainettu(Touch kosketus)
        {
            double kx = kosketus.PositionOnWorld.X;
            double ky = kosketus.PositionOnWorld.Y;
            // rajahdys.AddEffect(kx, ky, 40);
            Explosion rajahdys = new Explosion(30);
            rajahdys.Position = new Vector(kx, ky);
            rajahdys.ShockwaveColor = Color.Yellow;
            Add(rajahdys);

            foreach (PhysicsObject myyra in myyrat)
            {
                double etaisyys = Etaisyys(myyra.Position, new Vector(kx, ky));
                MessageDisplay.Add(etaisyys.ToString());
                if (etaisyys < MYYRAN_KOKO*2)
                {
                    MessageDisplay.Add("Osuttiin");
                    myyrat.Remove(myyra);
                    myyra.Destroy();
                    pistelaskuri.Value++;
                    break;
                }
            }
        }
#endif
    }
}