﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Controls;

namespace PiirtoFysiikka
{
    public class PiirtoKuvio
    {
        public List<GameObject> Jalki { get; set; }

        public Color Vari { get; set; }

        public double VasenReuna { get; set; }
        public double OikeaReuna { get; set; }
        public double YlaReuna { get; set; }
        public double AlaReuna { get; set; }

        public Vector VasenYlakulma { get; set; }
        public Vector VasenAlakulma { get; set; }
        public Vector OikeaYlakulma { get; set; }
        public Vector OikeaAlakulma { get; set; }

        public double Leveys
        {
            get { return OikeaReuna - VasenReuna; }
        }

        public double Korkeus
        {
            get { return YlaReuna - AlaReuna; }
        }

        public Vector Origo
        {
            get { return new Vector( VasenReuna + Leveys / 2, AlaReuna + Korkeus / 2 ); }
        }

        public PiirtoKuvio( Vector alkupiste, Color vari )
        {
            Jalki = new List<GameObject>();
            VasenReuna = OikeaReuna = alkupiste.X;
            YlaReuna = AlaReuna = alkupiste.Y;
            VasenYlakulma = VasenAlakulma = OikeaYlakulma = OikeaAlakulma = alkupiste;
            Vari = vari;
        }

        public void Paivita( Vector uusiPiste )
        {
            VasenReuna = Math.Min( VasenReuna, uusiPiste.X );
            OikeaReuna = Math.Max( OikeaReuna, uusiPiste.X );
            AlaReuna = Math.Min( AlaReuna, uusiPiste.Y );
            YlaReuna = Math.Max( YlaReuna, uusiPiste.Y );

            if ( uusiPiste.X <= VasenYlakulma.X && uusiPiste.Y >= VasenYlakulma.Y )
                VasenYlakulma = uusiPiste;

            if ( uusiPiste.X >= OikeaYlakulma.X && uusiPiste.Y >= OikeaYlakulma.Y )
                OikeaYlakulma = uusiPiste;

            if ( uusiPiste.X <= VasenAlakulma.X && uusiPiste.Y <= VasenAlakulma.Y )
                VasenAlakulma = uusiPiste;

            if ( uusiPiste.X >= OikeaAlakulma.X && uusiPiste.Y <= OikeaAlakulma.Y )
                OikeaAlakulma = uusiPiste;
        }

        internal Vector find(Vector v)
        {
            if (Jalki.Count == 0) return v;
            Vector pmin = Jalki[0].Position;
            double dmin = Vector.Distance(v, pmin);

            foreach (GameObject g in Jalki)
            {
                double d = Vector.Distance(v, g.Position);
                if (d < dmin)
                {
                    dmin = d;
                    pmin = g.Position;
                }

            }
            return pmin;
        }


    /// <summary>
    /// Laskeen pisten ja janan etäisyyden, jana otetaan taulukosta dp alkaen paikasta i
    /// x1 = dp[i], y1 = [dpi+1], x2 = dp[i+2], y2 = dp[i+3] 
    /// </summary>
    /// <param name="X">pisteen x</param>
    /// <param name="Y">pisteen y</param>
    /// <param name="dp">taulukko</param>
    /// <param name="i">aloituspaikka</param>
    /// <returns></returns>
    public static double DistanceToPointSquared(double X, double Y, 
        double[] dp, int i)
    {
      double vx = dp[i]-X, vy = dp[i+1]-Y, ux = dp[i+2]-dp[i], uy = dp[i+3]-dp[i+1];
      double length = ux*ux+uy*uy;

      double det = (-vx*ux)+(-vy*uy); //if this is < 0 or > length then its outside the line segment
      if(det<0 || det>length)
      {
        ux=dp[i+2]-X;
        uy=dp[i+3]-Y;
        return Math.Min(vx*vx+vy*vy, ux*ux+uy*uy);
      }

      det = ux*vy-uy*vx;
      return (det*det)/length;
    }
    }
}