using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Controls;
using Jypeli.WP7;

namespace PiirtoFysiikka
{
    public class PiirtoPeli : PhysicsGame
    {
        List<PhysicsObject> valmiitOliot = new List<PhysicsObject>();
        PiirtoKuvio kuvio;
 

        public override void Begin()
        {
            Phone.DisplayOrientation = DisplayOrientation.Portrait;
            Level.Width = 480;
            Level.Height = 800;
            Level.CreateBorders( 0.7, true );
            Camera.ZoomFactor = 0.5;

            Gravity = new Vector( 0, -200 );

            TouchPanel.Listen( ButtonState.Pressed, AloitaKuvio, null );
            TouchPanel.Listen( ButtonState.Down, JatkaKuviota, null );
            TouchPanel.Listen( ButtonState.Released, ViimeisteleKuvio, null );
            Accelerometer.ListenAnalog(AccelerometerSensitivity.Realtime, MuutaPainoVoimanSuunta, null);
        }

        void MuutaPainoVoimanSuunta(AnalogState s)
        {
            Gravity = s.StateVector * 2000;
        }

        private void AloitaKuvio(Touch kosketus)
        {
            if ( kuvio != null ) PoistaJalki(kuvio.Jalki);

            Color uusiVari = RandomGen.NextColor();
            kuvio = new PiirtoKuvio( kosketus.PositionOnWorld, uusiVari );
        }

        private void JatkaKuviota(Touch kosketus)
        {
            JatkaJalkea( kosketus, kuvio.Vari, 0 );
            kuvio.Paivita( kosketus.PositionOnWorld );
        }

        private void ViimeisteleKuvio( Touch kosketus )
        {
            LisaaObjekti(kuvio);
           // PoistaJalki(kuvio.Jalki);
        }

        void JatkaJalkea( Touch kosketus, Color vari, int taso )
        {
            GameObject pallo = new GameObject( 2, 2, Shape.Circle );
            pallo.Color = vari;
            pallo.Position = kosketus.PositionOnWorld;
            Add( pallo, taso );
            kuvio.Jalki.Add( pallo );

            Vector movement = kosketus.MovementOnWorld;
            GameObject movementVector = new GameObject( Math.Max( movement.Magnitude, 1 ), 3, Shape.Rectangle );
            movementVector.Position = kosketus.PositionOnWorld - movement / 2;
            movementVector.Angle = movement.Angle;
            movementVector.Color = vari;
            Add( movementVector );
            kuvio.Jalki.Add( movementVector );
        }

        private void PoistaJalki( List<GameObject> jalki )
        {
            foreach ( GameObject g in jalki )
            {
                Remove( g );
            }

            //jalki.Clear();
        }

        private void LisaaObjekti( PiirtoKuvio kuvio )
        {
            if (kuvio == null) return;
            if ( kuvio.Leveys == 0 || kuvio.Korkeus == 0 )
                return;

            PhysicsObject obj = RakennaObjekti( kuvio );
            if ( obj == null ) return;

            obj.Position = kuvio.Origo;
            //obj.Color = RandomGen.NextColor();
            obj.Color = kuvio.Vari;
            Add( obj );
            valmiitOliot.Add( obj );
        }


        private void kAdd(double r, Vector p, Boolean b)
        {
            GameObject g = new GameObject(r, r, Shape.Circle);
            g.Position = p;
            if (b) g.Color = Color.Red; else g.Color = Color.White;
            kuvio.Jalki.Add(g);
            Add(g);

        }

        private void kAdd(double r, Vector p)
        {
            kAdd(r, p,false);
        }
 
        private PhysicsObject RakennaObjekti( PiirtoKuvio kuvio )
        {
            PhysicsObject obj = new PhysicsObject( kuvio.Leveys, kuvio.Korkeus );

            double lavistaja = Math.Sqrt( kuvio.Leveys * kuvio.Leveys + kuvio.Korkeus * kuvio.Korkeus );
            double eps = lavistaja / 10;     // Tarkkuus, jolla kuvio tunnistetaan
            eps = Math.Max(eps,10);
            double eps2 = 2 * eps;

            // Kuvion kulmat reunoista laskettuina
            Vector maxvy = new Vector( kuvio.VasenReuna, kuvio.YlaReuna );
            Vector maxva = new Vector( kuvio.VasenReuna, kuvio.AlaReuna );
            Vector maxoy = new Vector( kuvio.OikeaReuna, kuvio.YlaReuna );
            Vector maxoa = new Vector( kuvio.OikeaReuna, kuvio.AlaReuna );

            kuvio.VasenAlakulma = kuvio.find(maxva);
            kuvio.VasenYlakulma = kuvio.find(maxvy);
            kuvio.OikeaAlakulma = kuvio.find(maxoa);
            kuvio.OikeaYlakulma = kuvio.find(maxoy);

            // Mit kulmia kosketetaan
            bool vyk = (maxvy - kuvio.VasenYlakulma).Magnitude < eps;
            bool vak = (maxva - kuvio.VasenAlakulma).Magnitude < eps;
            bool oyk = (maxoy - kuvio.OikeaYlakulma).Magnitude < eps;
            bool oak = (maxoa - kuvio.OikeaAlakulma).Magnitude < eps;


            kAdd(eps2, maxva, vak);
            kAdd(eps2, maxvy, vyk);
            kAdd(eps2, maxoa, oak);
            kAdd(eps2, maxoy, oyk);

            double r = 10;
            kAdd(r,kuvio.VasenAlakulma);
            kAdd(r,kuvio.VasenYlakulma);
            kAdd(r,kuvio.OikeaAlakulma);
            kAdd(r,kuvio.OikeaYlakulma);


            if ( vyk && oyk && vak && oak )
                obj.Shape = Shape.Rectangle;
            else if ( (vyk&&oak&&!vak&&!oyk) || (!vyk&&!oak&&vak&&oyk) )
            {
                // Diagonaaliveto -> poista objekti(t)
                PoistaObjektit( maxvy, maxoa );
                return null;
            }
//            else if ( !vyk && !oyk && vak && oak )
            else if ( vak && oak )
            {
                obj.Shape = Shape.Triangle;
                //obj.Angle = Angle.FromDegrees(0);
            }
            else if ( vyk && !oyk && vak && !oak )
            {
                obj.Shape = Shape.Triangle;
                obj.Angle = Angle.FromDegrees( -90 );
            }
//            else if ( vyk && oyk && !vak && !oak )
            else if ( vyk && oyk  )
            {
                obj.Shape = Shape.Triangle;
                obj.Angle = Angle.FromDegrees( 180 );
            }
            else if ( !vyk && oyk && !vak && oak )
            {
                obj.Shape = Shape.Triangle;
                obj.Angle = Angle.FromDegrees( 90 );
            }
            else
                obj.Shape = Shape.Circle;

            return obj;
        }

        private void PoistaObjektit( Vector vasenYla, Vector oikeaAla )
        {
            List<PhysicsObject> poistettavat = new List<PhysicsObject>();

            for ( int i = 0; i < valmiitOliot.Count; i++ )
            {
                if ( valmiitOliot[i].AbsolutePosition.X >= vasenYla.X &&
                     valmiitOliot[i].AbsolutePosition.Y <= vasenYla.Y &&
                     valmiitOliot[i].AbsolutePosition.X <= oikeaAla.X &&
                     valmiitOliot[i].AbsolutePosition.Y >= oikeaAla.Y )
                {
                    poistettavat.Add(valmiitOliot[i]);
                }
            }

            if (poistettavat.Count > 0) PoistaJalki(kuvio.Jalki);

            for ( int i = 0; i < poistettavat.Count; i++ )
            {
                valmiitOliot.Remove( poistettavat[i] );
                Remove( poistettavat[i] );
            }
        }
    }
}
