﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Globalization;

namespace FrekvenssiLaskuri
{
    public class Program
    {
        static void Main( string[] args )
        {
            String merkit = null;

            if ( args.Length < 1 || args.Length > 7 )
            {
                Console.WriteLine( "Frekvenssilaskuri 1.0" );
                Console.WriteLine( "Laskee tiedostossa olevien merkkien esiintymistodennäköisyydet." );
                Console.WriteLine( "Käyttö: Frekvenssilaskuri [optiot] <tiedosto>" );
                Console.WriteLine();
                Console.WriteLine( "Optiot:" );
                Console.WriteLine( "  -b mjono = Aloita merkkijonon lukeminen alimerkkijonon mjono jälkeen" ) ;
                Console.WriteLine( "  -e mjono = Lopeta merkkijonon lukeminen ennen alimerkkijonoa mjono" );
                Console.WriteLine( "  -m merkkitiedosto = Laske vain tietyt merkit (tiedostosta)" );
                Console.WriteLine();
                return;
            }

            String alku = null;
            String loppu = null;

            for ( int i = 0; i < args.Length - 1; i++ )
            {
                if ( args[i] == "-b" )
                    alku = args[++i];
                else if ( args[i] == "-e" )
                    loppu = args[++i];
                else if ( args[i] == "-m" )
                {
                    Stream fileStream = new FileStream( args[++i], FileMode.Open, FileAccess.Read );
                    StreamReader reader = new StreamReader( fileStream, Encoding.UTF8 );
                    merkit = reader.ReadLine();
                }
            }

            if ( merkit == null )
            {
                byte[] merkkiKoodit = new byte[256];
                for ( int i = 0; i < 256; i++ ) merkkiKoodit[i] = (byte)i;
                char[] merkkiTaulu = UnicodeEncoding.UTF8.GetChars( merkkiKoodit );
                merkit = new String( merkkiTaulu );
            }

            Encoding oldEncoding = Console.OutputEncoding;
            Console.OutputEncoding = Encoding.UTF8;

            try
            {
                using ( StreamReader input = File.OpenText( args[args.Length - 1] ) )
                {
                    FrekvenssiTaulu generaattori = new FrekvenssiTaulu( merkit );
                    String rivi;

                    while ( ( rivi = input.ReadLine() ) != null )
                    {
                        rivi = midStr( rivi, alku, loppu );
                        if ( String.IsNullOrEmpty( rivi ) ) continue;
                        generaattori.LisaaRivi( rivi );
                    }

                    List<String> rivitUlos = generaattori.Listaksi();
                    for ( int i = 0; i < rivitUlos.Count; i++ )
                        Console.WriteLine( rivitUlos[i] );
                }
            }
            catch ( FileNotFoundException )
            {
                Console.WriteLine( "Tiedostoa ei ole olemassa!" );
                return;
            }

            Console.OutputEncoding = oldEncoding;
        }

        static String midStr( String jono, String alku, String loppu )
        {
            if ( alku != null || loppu != null )
            {
                int ia = 0;

                if ( alku != null )
                {
                    ia = jono.IndexOf( alku );
                    if ( ia < 0 ) return null;
                    ia += alku.Length;
                }

                if ( loppu != null )
                {
                    int il = jono.IndexOf( loppu );
                    if ( il < 0 ) return null;
                    jono = jono.Substring( ia, il - ia );
                }
                else
                {
                    jono = jono.Substring( ia );
                }
            }

            return jono;
        }
    }
}
