using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

public class FrekvenssiTaulu
{
    static CultureInfo culture = CultureInfo.InvariantCulture;

    public String Merkit { get; private set; }
    public double[] Data { get; private set; }

    int merkkeja = 0;
    bool normalisoitu;

    public double this[char i]
    {
        get { return Data[Merkit.IndexOf( i )]; }
        set { Data[Merkit.IndexOf( i )] = value; }
    }

    public FrekvenssiTaulu( String merkit )
    {
        this.Merkit = merkit;
        Data = new double[merkit.Length];
        normalisoitu = false;
    }

    public void LisaaRivi( String rivi )
    {
        if ( normalisoitu )
            throw new InvalidOperationException( "Taulu on jo normalisoitu, uusia rivej ei voi en list." );

        for ( int i = 0; i < rivi.Length; i++ )
        {
            int j = Merkit.IndexOf( rivi[i] );

            if ( j >= 0 )
            {
                Data[j]++;
                merkkeja++;
            }
        }
    }

    public void Normalisoi()
    {
        if ( normalisoitu || merkkeja == 0 ) return;

        double normaaliKerroin = 1.0 / merkkeja;

        for ( int i = 0; i < Data.Length; i++ )
        {
            Data[i] *= normaaliKerroin;
        }

        normalisoitu = true;
    }

    public char ArvoKirjain( Random r )
    {
        double arpa = r.NextDouble();
        double summa = 0;

        for ( int i = 0; i < Merkit.Length; i++ )
        {
            summa += Data[i];
            if ( summa >= arpa ) return Merkit[i];
        }

        throw new InvalidOperationException( "ArvoKirjain: frekvenssien summa != 1.00" );
    }

    public List<String> Listaksi()
    {
        IFormatProvider format = culture.NumberFormat;
        List<String> lista = new List<string>();
        lista.Add( Merkit );

        Normalisoi();

        for ( int i = 0; i < Data.Length; i++ )
        {
            lista.Add( Data[i].ToString( format ) );
        }

        return lista;
    }

    public static FrekvenssiTaulu Listasta( List<String> lista )
    {
        FrekvenssiTaulu taulu = new FrekvenssiTaulu( lista[0] );

        for ( int i = 1; i < lista.Count; i++ )
        {
            taulu.Data[i - 1] = double.Parse( lista[i], culture.NumberFormat );
        }

        return taulu;
    }
}
