﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.WP7;


public class Peli : Game
{
    Random rand = new Random();
    StringList sanat;
    FrekvenssiTaulu frekvenssit;

    char[] kirjaimet = "abcdefghijklmnopqrstuvwxyzäö".ToCharArray();
    Dictionary<char, Image> kuvat = new Dictionary<char, Image>();
    Dictionary<char, Image> aktiiviKuvat = new Dictionary<char, Image>();
    Dictionary<char, int> pisteetKirjaimille = new Dictionary<char, int>();
    IntMeter pisteet = new IntMeter( 0 );

    const int palikoitaRivissa = 5;
    double palikanKoko;

    LinkedList<GameObject>[] sarakkeet = new LinkedList<GameObject>[palikoitaRivissa];
    Touch viimeKosketus = null;
    List<GameObject> valittuSana = new List<GameObject>();    


    public override void Begin()
    {
        Phone.DisplayOrientation = DisplayOrientation.Portrait;
        Phone.DisplayResolution = DisplayResolution.Small;

        MessageDisplay.TextColor = Color.DarkCyan;
        Level.BackgroundColor = Color.Black;
        Level.Width = Level.Height = 500;
        palikanKoko = Level.Width / palikoitaRivissa;

        Label pisteNaytto = new Label(Screen.Width, 40);
        pisteNaytto.HorizontalAlignment = HorizontalAlignment.Right;
        pisteNaytto.Y = Screen.TopSafe - pisteNaytto.Height;
        pisteNaytto.TextColor = Color.LightGreen;
        pisteNaytto.BindTo( pisteet );
        Add( pisteNaytto );

        LataaSanat();
        LuoKuvat();
        LuoPalikat();        
    }

    private void LataaSanat()
    {
        sanat = StringList.FromAsset( "omatsanat" );
        StringList frekvenssiLista = StringList.FromAsset( "omatfreq" );
        frekvenssit = FrekvenssiTaulu.Listasta( frekvenssiLista );

        for ( int i = 0; i < frekvenssit.Data.Length; i++ )
        {
            char c = frekvenssit.Merkit[i];
            double f = frekvenssit.Data[i];

            if ( f < 0.001 ) pisteetKirjaimille.Add( c, 6 );
            else if ( f < 0.005 ) pisteetKirjaimille.Add( c, 5 );
            else if ( f < 0.015 ) pisteetKirjaimille.Add( c, 4 );
            else if ( f < 0.030 ) pisteetKirjaimille.Add( c, 3 );
            else if ( f < 0.060 ) pisteetKirjaimille.Add( c, 2 );
            else pisteetKirjaimille.Add( c, 1 );
        }
    }

    void LuoKuvat()
    {
        Font kirjainFontti = Font.DefaultLarge;
        Font pisteFontti = Font.DefaultSmallBold;
        Image passiiviKuva = LoadImage( "palikka" );
        Image aktiiviKuva = LoadImage( "palikkaAlhaalla" );

        for ( int i = 0; i < kirjaimet.Length; i++ )
        {
            char k = kirjaimet[i];
            Vector oikeaYlareuna = new Vector( passiiviKuva.Width / 2 - 10, passiiviKuva.Height / 2 - 13 );
            Image kuva = Image.DrawTextOnImage( passiiviKuva, k.ToString(), kirjainFontti, Color.White );
            kuva = Image.DrawTextOnImage( kuva, pisteetKirjaimille[k].ToString(), oikeaYlareuna, pisteFontti, Color.Fuchsia, Color.Transparent );
            kuvat.Add( kirjaimet[i], kuva );

            Image kuvaAlhaalla = Image.DrawTextOnImage( aktiiviKuva, k.ToString(), kirjainFontti, Color.White );
            aktiiviKuvat.Add( kirjaimet[i], kuvaAlhaalla );
        }
    }

    void LuoPalikat()
    {
        char kirjain;

        for ( int i = 0; i < palikoitaRivissa; i++ )
        {
            sarakkeet[i] = new LinkedList<GameObject>();

            for ( int j = 0; j < palikoitaRivissa; j++ )
            {
                kirjain = frekvenssit.ArvoKirjain( rand );
                GameObject palikka = LuoPalikka( kirjain );
                palikka.Left = Level.Left + i * palikanKoko;
                palikka.Top = Level.Top - j * palikanKoko;
                sarakkeet[i].AddLast( palikka );
            }
        }

        Camera.ZoomToLevel();
        Camera.Y = Level.Top - Level.Height / 2;
    }

    GameObject LuoPalikka( char kirjain )
    {
        GameObject palikka = new GameObject( palikanKoko, palikanKoko, Shape.Rectangle );
        palikka.Tag = kirjain;
        palikka.Image = kuvat[kirjain];
        Add( palikka );

        TouchPanel.ListenOn( palikka, ButtonState.Down, AktivoiPalikka, null, palikka );
        TouchPanel.Listen( ButtonState.Released, KokeileSanaa, null );
        return palikka;
    }

    LinkedList<GameObject> LaskeSarake( GameObject palikka )
    {
        int i = (int)Math.Round( ( palikka.Left - Level.Left ) / palikanKoko );
        return sarakkeet[i];
    }

    char AktivoiPalikka( GameObject palikka, bool aktivoitu )
    {
        char kirjain = (char)palikka.Tag;

        if ( aktivoitu )
            palikka.Image = aktiiviKuvat[kirjain];
        else
            palikka.Image = kuvat[kirjain];

        return kirjain;
    }

    void AktivoiPalikka( Touch kosketus, GameObject palikka )
    {
        viimeKosketus = kosketus;
        int viimeIndeksi = valittuSana.Count - 1;

        if ( valittuSana.Count > 0 && palikka == valittuSana[viimeIndeksi] )
        {
            // Ei olla liikuttu mihinkään, ei tehdä mitään
            return;
        }

        if ( valittuSana.Count > 1 && palikka == valittuSana[viimeIndeksi - 1] )
        {
            // Siirryttiin edelliseen kirjaimeen -> poistetaan viimeisin
            AktivoiPalikka(valittuSana[viimeIndeksi], false);
            valittuSana.RemoveAt( viimeIndeksi );
            return;
        }

        int palikkaIndeksi = valittuSana.IndexOf( palikka );

        if ( palikkaIndeksi > 0 )
        {
            // Siirryttiin aikaisempaan kirjaimeen -> jatketaan siitä

            for ( int i = viimeIndeksi; i >= palikkaIndeksi; i-- )
            {
                AktivoiPalikka( valittuSana[i], false );
                valittuSana.RemoveAt( i );
            }

            return;
        }

        if ( valittuSana.Count == 0 ||
            Vector.Distance( valittuSana[viimeIndeksi].AbsolutePosition, palikka.AbsolutePosition ) < 2 * palikanKoko )
        {
            // Uusi kirjain
            AktivoiPalikka( palikka, true );
            valittuSana.Add( palikka );
        }
    }

    void PoistaPalikka( GameObject poistettava )
    {
        LinkedList<GameObject> sarake = LaskeSarake( poistettava );
        LinkedListNode<GameObject> palikkaSolmu = sarake.Last;

        // Tee uusi palikka poistettavan tilalle
        char kirjain = frekvenssit.ArvoKirjain( rand );
        GameObject uusiPalikka = LuoPalikka( kirjain );
        uusiPalikka.Left = poistettava.Left;
        uusiPalikka.Top = sarake.First.Value.Top + palikanKoko;
        sarake.AddFirst( uusiPalikka );

        while ( palikkaSolmu != null )
        {
            if ( palikkaSolmu.Value == poistettava )
            {
                LinkedListNode<GameObject> ylempi = palikkaSolmu.Previous;
                sarake.Remove( palikkaSolmu );
                poistettava.Destroy();
                palikkaSolmu = ylempi;

                Timer pudotusAjastin = new Timer();
                pudotusAjastin.Interval = 0.05;
                pudotusAjastin.Tag = palikkaSolmu;
                pudotusAjastin.Trigger += new Timer.TriggerHandler( PudotaSarake );
                pudotusAjastin.Start();

                return;
            }

            palikkaSolmu = palikkaSolmu.Previous;
        }
    }

    void PudotaPalikka( Timer sender )
    {
        LinkedListNode<GameObject> palikkaSolmu = (LinkedListNode<GameObject>)sender.Tag;
        GameObject palikka = palikkaSolmu.Value;
        double alareuna = ( palikkaSolmu.Next != null ) ? palikkaSolmu.Next.Value.Top : Level.Bottom;

        if ( palikka.Bottom - alareuna > 0 )
            palikka.Y -= palikka.Height / 20;
        else
            sender.Stop();
    }

    void PudotaSarake( Timer sender )
    {
        LinkedListNode<GameObject> palikkaSolmu = sender.Tag as LinkedListNode<GameObject>;
        if ( palikkaSolmu == null ) return;

        double alareuna = ( palikkaSolmu.Next != null ) ? palikkaSolmu.Next.Value.Top : Level.Bottom;

        if ( palikkaSolmu.Value.Bottom - alareuna > 0 )
        {
            double dy = palikkaSolmu.Value.Height / 10;

            while ( palikkaSolmu != null )
            {
                palikkaSolmu.Value.Y -= dy;
                palikkaSolmu = palikkaSolmu.Previous;
            }
        }
        else
            sender.Stop();
    }

    void KokeileSanaa( Touch kosketus )
    {
        if ( kosketus != viimeKosketus ) return;

        StringBuilder sana = new StringBuilder();

        foreach ( GameObject palikka in valittuSana )
        {
            char kirjain = AktivoiPalikka( palikka, false );
            sana.Append( kirjain );
        }

        String sanaString = sana.ToString();

        MessageDisplay.Add( sanaString );

        if ( sanat.Contains( sanaString ) )
        {
            foreach ( GameObject palikka in valittuSana )
            {
                PoistaPalikka( palikka );
                pisteet.Value += pisteetKirjaimille[(char)palikka.Tag];
            }
        }

        valittuSana.Clear();
        viimeKosketus = null;
    }    
}