﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;
using Jypeli.WP7;

public class Peli : PhysicsGame
{
    Timer pallojenluontiAjastin = new Timer();
    Timer pallojenluomisenNopeutin = new Timer();

    SoundEffect osumaAani = LoadSoundEffect( "osui" );
    SoundEffect eiOsuAani = LoadSoundEffect( "eiosunut" );

    List<PhysicsObject> pallot = new List<PhysicsObject>();
    List<PhysicsObject> powerupit = new List<PhysicsObject>();

    const double PIKKUPALLON_SADE = 20;
    const double POWERUP_SADE = 40;
    ExplosionSystem explosionSystem, explosionSystem2;
    IntMeter pistelaskuri = new IntMeter(0);
    int montakoPalloaTehty = 0;

    bool juuriTehtyPowerup = false;

    public override void Begin()
    {
        explosionSystem = new ExplosionSystem(LoadImage("bum"), 100);
        explosionSystem2 = new ExplosionSystem(LoadImage("bum2"), 50);
        explosionSystem.MinScale = 0.3;
        explosionSystem.MaxScale = 0.4;
        explosionSystem.MinLifetime = 0.2;
        explosionSystem.MaxLifetime = 0.6;
        explosionSystem2.MinScale = 0.1;
        explosionSystem2.MaxScale = 0.2;
        explosionSystem2.MinLifetime = 0.1;
        explosionSystem2.MaxLifetime = 0.2;
        Add(explosionSystem); Add(explosionSystem2);
        Level.Width = 800;
        Level.Height = 480;
        Camera.ZoomToLevel();
        Level.Background.Image = LoadImage("tausta");
        //Phone.DisplayOrientation = DisplayOrientation.Portait;

        pallojenluontiAjastin.Interval = 1;
        pallojenluontiAjastin.Timeout += PallojenLuonti;
        pallojenluontiAjastin.Start();
        pallojenluomisenNopeutin.Interval = 5;
        pallojenluomisenNopeutin.Timeout += PallojenNopeutus;
        pallojenluomisenNopeutin.Start();

        Level.CreateBorders();
        MessageDisplay.TextColor = Color.White;
        MessageDisplay.Add("Ammu pallot ja varo, ettei palloja tule liikaa!");
        Label pisteNaytto = new Label();
        pisteNaytto.BindTo(pistelaskuri);
        pisteNaytto.TextColor = Color.White;
        pisteNaytto.Color = Color.Black;
        Add(pisteNaytto);
        pisteNaytto.Position = new Vector(Screen.Right - 50, Screen.Top - 50);
        TouchPanel.Listen(ButtonState.Pressed, PainettuNayttoa, null);
    }

    void PainettuNayttoa(Touch kosketus)
    {
        int i = 0;
        while (i < pallot.Count)
        {
            if (Etaisyys(kosketus.PositionOnWorld, pallot[i].Position) < 50)
            {
                RajaytaPallo(pallot[i]); 
                pistelaskuri.Value++;
                return;
            }
            i++;
        }
        i = 0;

        while (i < powerupit.Count)
        {
            if (Etaisyys(kosketus.PositionOnWorld, powerupit[i].Position) < 50)
            {
                RajaytaKaikkiPallot();
                // PlaySound("osui");
                powerupit[i].Destroy();
                powerupit.RemoveAt(i);
                MessageDisplay.Add("POWERUP");
                return;
            }
            i++;
        }

        eiOsuAani.Play();
        explosionSystem2.AddEffect(kosketus.PositionOnWorld.X, kosketus.PositionOnWorld.Y, 10);
    }

    void RajaytaKaikkiPallot()
    {
        int i = 0;
        while (pallot.Count>0)
        {
            RajaytaPallo(pallot[i]);
            pallot.RemoveAt(i);
            pistelaskuri.Value++;
        }
    }

    void RajaytaPallo(PhysicsObject pallo)
    {
        explosionSystem.AddEffect(pallo.X, pallo.Y, 5);
        osumaAani.Play();
        pallo.Destroy();
    }

    void PallojenLuonti()
    {
        //
        if (Time.SinceStartOfGame.Seconds % 5 - 4 == 0 && !juuriTehtyPowerup)
        {
            PhysicsObject powerup = TeePowerup();
            juuriTehtyPowerup = true; // tämä siksi, että pallojenluontiajastimen nopeutuksen takia poweruppeja saattaa "yllättäen" ilmestyä pari kpl putkeen
            Timer.SingleShot(1.5, delegate() { TuhoaPowerup(powerup); });
            montakoPalloaTehty++;
            return;
        }
        TeePalloja(1);
        montakoPalloaTehty++;
        if (pallot.Count > 30) PeliPaattyy();
    }

    void TuhoaPowerup(PhysicsObject powerup)
    {
        juuriTehtyPowerup = false; // ks. PallojenLuontiTrigger(Timer)
        if (powerup != null)
        {
            powerupit.Remove(powerup);
            powerup.Destroy();
        }
    }

    PhysicsObject TeePowerup()
    {
        Vector paikka = SopivaPaikkaRuudulla(POWERUP_SADE);
        PhysicsObject powerup = TeePallo(POWERUP_SADE, Color.White);
        Add(powerup);
        powerupit.Add(powerup);
        Potkaise(powerup);
        return powerup;
    }

    void PeliPaattyy()
    {
        TouchPanel.Clear();
        pallojenluomisenNopeutin.Stop();
        pallojenluontiAjastin.Stop();
        int pisteet = pallot.Count;
        foreach (PhysicsObject pallo in pallot)
        {
            pallo.Destroy();
        }
        pallot.Clear();
        Label pisteNaytto = new Label("Pisteesi: \n" + pistelaskuri.Value.ToString());
        pisteNaytto.HorizontalAlignment = HorizontalAlignment.Center;
        pisteNaytto.TextColor = Color.White;
        pisteNaytto.Font = Font.DefaultLarge;
        Add(pisteNaytto);
    }

    void PallojenNopeutus()
    {
        pallojenluontiAjastin.Interval *= 0.75;
    }

    /// <summary>
    /// Ei synnytetä vehkeitä päällekkäin.
    /// </summary>
    /// <param name="sade"></param>
    /// <returns></returns>
    Vector SopivaPaikkaRuudulla(double sade)
    {
        Vector paikka = SatunnainenPaikka(sade);
        int j = 0;

        while (j < pallot.Count)
        {
            if (Etaisyys(pallot[j].Position, paikka) < sade * 2)
            {
                j = 0;
                paikka = SatunnainenPaikka(sade);
            }
            else j++;
        }
        return paikka;
    }

    Vector Potkaise(PhysicsObject p)
    {
        Vector voima = SatunnainenPaikka(PIKKUPALLON_SADE).Normalize() * 500;
        p.Hit(voima);
        return voima;
    }

    void TeePalloja(int montako)
    {
        for (int i = 0; i < montako; i++)
        {
            Vector paikka = SopivaPaikkaRuudulla(PIKKUPALLON_SADE);
            Color vari;
            do
            {
                vari = RandomGen.NextColor();
            } while (vari.RedComponent < 120 || vari.GreenComponent < 120 || vari.BlueComponent < 120);

            PhysicsObject pallo = TeePallo(PIKKUPALLON_SADE, vari);
            pallo.Position = paikka;
            pallot.Add(pallo);
            Add(pallo);
            Potkaise(pallo);
        }
    }

    PhysicsObject TeePallo(double r, Color vari)
    {
        PhysicsObject pallo = new PhysicsObject(r * 2, r * 2, Shape.Circle);
        pallo.Color = vari;
        return pallo;
    }

    Vector SatunnainenPaikka(double sade)
    {
        return new Vector(
                RandomGen.NextDouble(Level.Left + sade, Level.Right - sade),
                RandomGen.NextDouble(Level.Bottom + sade, Level.Top - sade));
    }

    static double Etaisyys(Vector a, Vector b)
    {
        return Vector.Distance(a, b);
    }
}