﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.WP7;

public class Peli : PhysicsGame
{
    List<TileMap> kentat;
    int kentanNumero;
    bool peliKeskeytetty;
    Label keskeytysTeksti;
    PhysicsObject pallo;

    public override void Begin()
    {
        kentat = new List<TileMap>();
        kentanNumero = 0;
        peliKeskeytetty = false;        

        Level.Width = 800;
        Level.Height = 480;

        Phone.DisplayOrientation = DisplayOrientation.LandscapeLeft;
        Accelerometer.SetDefaultSensitivity(AccelerometerSensitivity.Realtime);

        LisaaKentat();        
        AlustaKentat();                
        AloitaKentta();
    }    

    void LisaaKentat()
    {
        kentat.Add(TileMap.FromLevelAsset("kentta1"));
        kentat.Add(TileMap.FromLevelAsset("kentta2"));
        //kentat.Add(TileMap.FromLevelAsset("kentta3"));
    }

    void AlustaKentat()
    {
        foreach (TileMap kentta in kentat)
        {
            kentta['='] = LuoVaakaSeina;
            kentta['|'] = LuoPystySeina;
            kentta['o'] = LuoPallo;
            kentta['x'] = LuoReika;
            kentta['w'] = LuoMaali;
        }
    }

    void LuoKontrollit()
    {
        TouchPanel.Listen(ButtonState.Pressed, KeskeytaPeli, "Keskeyttää pelin");
        Accelerometer.ListenAnalog(MuutaPainoVoima, "Muuttaa painovoiman suuntaa");
    }

    void LuoKeskeytysTeksti()
    {
        keskeytysTeksti = new Label("Peli keskeytetty!");
        keskeytysTeksti.Position = Level.Center;
    }

    void AloitaKentta()
    {
        ClearAll();
        LuoKontrollit();
        if (kentanNumero >= kentat.Count)
            kentanNumero = 0;
        kentat.ElementAt(kentanNumero).Insert();
        Level.CreateBorders();
        Camera.ZoomToLevel();
        LuoKeskeytysTeksti();
    }

    PhysicsObject LuoPystySeina()
    {
        return PhysicsObject.CreateStaticObject(20, 60);
    }
    PhysicsObject LuoVaakaSeina()
    {
        return PhysicsObject.CreateStaticObject(20, 20);
    }
    PhysicsObject LuoPallo()
    {
        pallo = new PhysicsObject(30, 30, Shape.Circle);
        pallo.Restitution = 0.99;
        pallo.LinearDamping = 0.99;
        pallo.AngularDamping = 0.99;
        pallo.Color = Color.Green;
        return pallo;
    }
    PhysicsObject LuoReika()
    {
        var reika = PhysicsObject.CreateStaticObject(20, 20, Shape.Circle);        
        reika.Color = Color.Black;
        AddCollisionHandler(reika, PalloOsuiReikaan);        
        return reika;
    }
    PhysicsObject LuoMaali()
    {
        var maali = PhysicsObject.CreateStaticObject(40, 40, Shape.Circle);
        maali.Color = Color.Red;
        AddCollisionHandler(maali, PalloMaalissa);
        return maali;
    }

    void KeskeytaPeli(Touch t)
    {
        peliKeskeytetty = !peliKeskeytetty;
        if ( peliKeskeytetty )
        {
            Add( keskeytysTeksti );
            pallo.Stop();
        }
        else
        {
            Remove( keskeytysTeksti );
        }
    }

    void MuutaPainoVoima( AnalogState tila )
    {
        Gravity = peliKeskeytetty ? Vector.Zero : tila.StateVector * 1000;        
    }

    void PalloOsuiReikaan(PhysicsObject reika, PhysicsObject pallo)
    {
        Phone.Vibrate(50);
        AloitaKentta();
    }

    void PalloMaalissa(PhysicsObject maali, PhysicsObject pallo)
    {
        Phone.Vibrate(100);
        kentanNumero++;
        AloitaKentta();
    }
}