using System;

/*
 * Authors: Jaakko Kosonen 
 */

namespace ParticleComponent
{
    /// <summary>
    /// A closed interval of float values.
    /// </summary>
    public class ClosedFloatInterval
    {

        private float min;        
        /// <summary>
        /// Mininum value of interval.
        /// </summary>
        public float MinValue
        {
            get { return min; }
            set { min = value; }
        }

        private float max;
        /// <summary>
        /// Maximum value of interval.
        /// </summary>
        public float MaxValue
        {
            get { return max; }
            set { max = value; }
        }

        /// <summary>
        /// Minimum value of interval as 32-bit integer.
        /// </summary>
        public int MinIntValue
        {
            get { return (Int32)min; }
        }

        /// <summary>
        /// Maximum value of interval as 32-bit integer.
        /// </summary>
        public int MaxIntValue
        {
            get { return (Int32)max; }
        }

        /// <summary>
        /// Constructs an interval with all possible values of System.Single
        /// </summary>
        public ClosedFloatInterval() : this(float.MinValue, float.MaxValue)
        { }

        /// <summary>
        /// Contructs a new interval.
        /// </summary>
        /// <param name="min">Minimum value.</param>
        /// <param name="max">Maximum value.</param>
        public ClosedFloatInterval(float min, float max)
        {
            this.min = Math.Min(min, max);
            this.max = Math.Max(min, max);
        }

        /// <summary>
        /// Returns a random value inside the interval.
        /// </summary>
        /// <param name="random">Random number generator</param>
        /// <returns>A random value</returns>
        public float GetRandomValue(Random random)
        {
            return min + (float)random.NextDouble() * (max - min);
        }
    }
}