using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;

/*
 * Authors: Jaakko Kosonen 
 */

namespace ParticleComponent
{
    /// <summary>
    /// Settings for a particle-effect.
    /// </summary>
    public class EffectSettings
    {        
        private ClosedFloatInterval particlesPerEffect = new ClosedFloatInterval(5, 10);
        /// <summary>
        /// Number of particles for single effect.
        /// </summary>
        [ContentSerializer(Optional = true)]
        public ClosedFloatInterval ParticlesPerEffect
        {
            get { return particlesPerEffect; }
            set { particlesPerEffect = value; }           
        }
        
        private ClosedFloatInterval initialParticleSpeed = new ClosedFloatInterval(20, 120);
        /// <summary>
        /// Initial speed for single particle.
        /// </summary>
        [ContentSerializer(Optional = true)]
        public ClosedFloatInterval InitialParticleSpeed 
        {
            get { return initialParticleSpeed; }
            set { initialParticleSpeed = value; }
        }
       
        private ClosedFloatInterval particleAccelerationScale = new ClosedFloatInterval(-50, -10);
        /// <summary>
        /// Acceleration for single particle.
        /// </summary>
        [ContentSerializer(Optional = true)]
        public ClosedFloatInterval ParticleAccelerationScale 
        {
            get { return particleAccelerationScale; }
            set { particleAccelerationScale = value; }
        }
        
        private ClosedFloatInterval particleRotationSpeed = new ClosedFloatInterval(-10, 10);
        /// <summary>
        /// Rotation speed for single particle.
        /// </summary>
        [ContentSerializer(Optional = true)]
        public ClosedFloatInterval ParticleRotationSpeed
        {
            get { return particleRotationSpeed; }
            set { particleRotationSpeed = value; }
        }
        
        private ClosedFloatInterval particleLifeTime = new ClosedFloatInterval(.5f, 1.5f);
        /// <summary>
        /// Lifetime of single particle.
        /// </summary>
        [ContentSerializer(Optional = true)]
        public ClosedFloatInterval ParticleLifeTime
        {
            get { return particleLifeTime; }
            set { particleLifeTime = value; }
        }
        
        private ClosedFloatInterval particleSize = new ClosedFloatInterval(.5f, 1);
        /// <summary>
        /// Scale of single particle.
        /// </summary>
        [ContentSerializer(Optional = true)]
        public ClosedFloatInterval ParticleSize
            {
            get { return particleSize; }
            set { particleSize = value; }
        }
        
        private ClosedFloatInterval particleDirection = new ClosedFloatInterval(0, 360);
        /// <summary>
        /// Direction for single particle in agnles (0-360).
        /// </summary>
        [ContentSerializer(Optional = true)]
        public ClosedFloatInterval ParticleDirection
            {
            get { return particleDirection; }
            set { particleDirection = value; }
        }
        
        private float maxOpacity = 50;
        /// <summary>
        /// Maximum opacity of single particle (0-100).
        /// </summary>
        [ContentSerializer(Optional = true)]
        public float MaxOpacity
        {
            get { return maxOpacity; }
            set
            {
                maxOpacity = MathHelper.Clamp(value, 0, 100);
            }
        }
        
        private Color colorization = Color.White;
        /// <summary>
        /// Colorization of single particle.
        /// </summary>
        [ContentSerializer(Optional = true)]
        public Color Colorization
        {
            get { return colorization; }
            set { colorization = value; }
        }
    }
}
