using System.Collections.Generic;
using Microsoft.Xna.Framework;

/*
 * Authors: Jaakko Kosonen 
 */

namespace ParticleComponent
{
    /// <summary>
    /// Collection for handling multiple effects at the same time.
    /// </summary>
    public class ParticleEffectCollection
    {
        private List<ParticleEffect> effects;

        /// <summary>
        /// Constructor.
        /// </summary>
        public ParticleEffectCollection()
        {
            effects = new List<ParticleEffect>();
        }

        /// <summary>
        /// Adds an effect to the collection.
        /// </summary>
        /// <param name="effect">Effect to add.</param>
        public void AddEffect(ParticleEffect effect)
        {
            effects.Add(effect);
        }

        /// <summary>
        /// Creates a new effect for each effect currently in collection.
        /// </summary>
        /// <param name="position">Position for new effect</param>
        public void CreateNewEffect(Vector2 position)
        {
            foreach (ParticleEffect e in effects)
            {
                e.CreateNewEffect(position);
            }
        }
    }
}