using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Audio;
using ParticleComponent;

/*
 * Authors: Jaakko Kosonen 
 */

namespace WindowsPhoneGame
{
    /// <summary>
    /// Pong-ball.
    /// </summary>
    class PongBall : PongObject
    {                        
        private Random random;
        
        /// <summary>
        /// Particle-effect for creating a trail for the ball.
        /// </summary>
        public ParticleEffect TrailEffect { get; set; }
        
        private float speed;
        /// <summary>
        /// Speed of ball.
        /// </summary>
        public float Speed
        {
            get { return speed; }
            set { speed = value; }
        }

        private int rotationDirection;
        /// <summary>
        /// Rotation direction of ball;
        /// </summary>
        public int RotationDirection
        {
            get { return rotationDirection; }
            set { rotationDirection = value; }
        }

        private Vector2 direction;
        /// <summary>
        /// Direction of ball.
        /// </summary>
        public Vector2 Direction
        {
            get { return direction; }
            set { direction = value; }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="initialPosition">Inital coordinates</param>
        /// <param name="width">Width</param>
        /// <param name="height">Height</param>
        /// <param name="initialSpeed">Initial speed</param>
        public PongBall(Vector2 initialPosition, int width, int height, float initialSpeed)
            : base((Int32)initialPosition.X, (Int32)initialPosition.Y, width, height)
        {
            random = new Random();                     
            Direction = new Vector2(0, 0);            
            Rotation = 0;
            Speed = initialSpeed;
            RotationDirection = 1;
        }

        /// <summary>
        /// Updates
        /// - position
        /// - rotation
        /// and creates a new traileffect.
        /// </summary>
        public void Update()
        {            
            Rotation += speed / 2 * rotationDirection;
            if (Rotation >= 360)
            {
                Rotation = Rotation - 360;
            }            
            Position += direction * speed;
            
            if (TrailEffect != null)
            {
                Vector2 positionOffset = direction * speed * 2.5f + Origin;
                TrailEffect.CreateNewEffect(Position + positionOffset);
            }
        }

        /// <summary>
        /// Changes the direction of ball according to given force.
        /// </summary>
        /// <param name="force">Force</param>
        public void Hit(Vector2 force)
        {
            direction = direction * force;            
        }       
    }
}
