using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;

/*
 * Authors: Jaakko Kosonen 
 */

namespace WindowsPhoneGame
{   
    /// <summary>
    /// Base-class for common pong-objects.
    /// </summary>
    class PongObject
    {
        private Texture2D img;

        /// <summary>
        /// Rotation of object.
        /// </summary>
        public float Rotation { get; set; }

        /// <summary>
        /// Height of object.
        /// </summary>
        public int Height
        {
            get { return rectangle.Height; }
        }

        /// <summary>
        /// Width of object.
        /// </summary>
        public int Width
        {
            get { return rectangle.Width; }
        }

        /// <summary>
        /// Texture of object.
        /// </summary>
        public Texture2D Image
        {
            set { img = value; }
        }        

        private Vector2 position;
        /// <summary>
        /// Position of object.
        /// </summary>
        public Vector2 Position
        {
            get
            {                
                return position;
            }
            set
            {                                
                position = value;
                rectangle.X = (Int32)value.X;
                rectangle.Y = (Int32)value.Y;                
            }
        }

        /// <summary>
        /// Origin of object.
        /// </summary>
        public Vector2 Origin
        {
            get
            {
                return new Vector2(rectangle.Width / 2, rectangle.Height / 2);                                            
            }
        }

        private Rectangle rectangle;
        /// <summary>
        /// Rectangle of object.
        /// </summary>
        public Rectangle Rectanlge
        {            
            get { return rectangle; }
        }

        private Vector2 force;
        /// <summary>
        /// Force of object.
        /// </summary>
        public Vector2 Force
        {
            set { force = value; }
            get
            {
                if (force == null)
                {
                    return new Vector2(1, 1);
                }
                else
                {
                    return force;
                }
            }
        }

        private CollisionType collisionType;
        /// <summary>
        /// Collision type of object.
        /// </summary>
        public CollisionType CollisionType
        {
            set { collisionType = value; }
            get { return collisionType; }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="x">X-coordinate</param>
        /// <param name="y">Y-coordinate</param>
        /// <param name="width">Width</param>
        /// <param name="height">Height</param>
        public PongObject(int x, int y, int width, int height)
        {
            rectangle = new Rectangle(x, y, width, height);
            Position = new Vector2(x, y);
            Rotation = 0;
        }

        /// <summary>
        /// Draws object to given spriteBatch.
        /// </summary>
        /// <param name="spriteBatch">SpriteBatch</param>
        public void DrawObject(SpriteBatch spriteBatch)
        {
            if (this.img == null) return;
            Vector2 tmp = Position + Origin;
            spriteBatch.Draw(this.img, tmp, null, Color.White, MathHelper.ToRadians(Rotation), Origin, 1, SpriteEffects.None, 0);            
        }

        /// <summary>
        /// Checks if object intersects with given rectangle.
        /// </summary>
        /// <param name="r">Other rectangle</param>
        /// <returns>True if rectangles intersect</returns>
        public bool CheckForCollision(Rectangle r)
        {
            return rectangle.Intersects(r);                     
        }

        /// <summary>
        /// Resizes objects.
        /// </summary>
        /// <param name="width">New width</param>
        /// <param name="height">New Height</param>
        public void Resize(int width, int height)
        {            
            rectangle.Height = height;
            rectangle.Width = width;
        }
    }
}
