using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;

/*
 * Authors: Jaakko Kosonen 
 */

namespace WindowsPhoneGame
{
    /// <summary>
    /// Pong-paddle
    /// </summary>
    class PongPaddle : PongObject
    {        
        private int direction;
        private Vector2 limits;

        private float speed;
        /// <summary>
        /// Speed of paddle.
        /// </summary>
        public float Speed
        {
            set { speed = value; }
            get { return speed; }
        }                

        /// <summary>
        /// Y-coordinate goal for paddle.
        /// </summary>
        public float YcoordinateGoal { get; set; }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="x">X-coordinate</param>
        /// <param name="y">Y-coordinate</param>
        /// <param name="width">Width</param>
        /// <param name="height">Height</param>
        /// <param name="initialSpeed">Initial speed</param>
        /// <param name="limits">Upper and lower limits for Y-coordinate</param>
        public PongPaddle(int x, int y, int width, int height, float initialSpeed, Vector2 limits) 
            : base(x,y,width,height)
        {                        
            CollisionType = CollisionType.BallAndPaddle;
            Force = new Vector2(-1, 1);
            YcoordinateGoal = Position.Y;
            Speed = initialSpeed;
            Rotation = 0;
            this.limits = limits;
        }

        /// <summary>
        /// Updates position.
        /// </summary>
        public void Update()
        {            
            if (YcoordinateGoal != Position.Y)
            {
                if (Math.Abs(YcoordinateGoal - Position.Y) <= speed)
                {
                    float y = MathHelper.Clamp(YcoordinateGoal, limits.X, limits.Y - Height);
                    Position = new Vector2(Position.X, y);
                    YcoordinateGoal = Position.Y;                    
                }
                else
                {
                    direction = YcoordinateGoal > Position.Y ? 1 : -1;
                    float y = Position.Y + direction * speed;
                    y = MathHelper.Clamp(y, limits.X, limits.Y - Height);                    
                    Position = new Vector2(Position.X, y);
                }
            }                      
        }
    }
}
