using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/*
 * Authors: Jaakko Kosonen 
 */

namespace WindowsPhoneGame
{
    /// <summary>
    /// Enumeration for player states.
    /// </summary>
    public enum PongPlayerState
    {
        Active,
        Defeated
    }

    /// <summary>
    /// Pong-player.
    /// </summary>
    class PongPlayer : CollisionEventHandler
    {
        private int health;

        /// <summary>
        /// Health of player.
        /// </summary>
        public int Health { get; set; }

        /// <summary>
        /// State of player.
        /// </summary>
        public PongPlayerState State { get; set; }
        
        /// <summary>
        /// Constructor.
        /// </summary>
        public PongPlayer()
        {
            Health = 100;
            health = 100;
            State = PongPlayerState.Active;
        }

        /// <summary>
        /// Heals the player to given health.
        /// </summary>
        /// <param name="health">New health</param>
        public void Heal(int health)
        {
            Health = health;
            this.health = health;
            State = PongPlayerState.Active;
        }

        /// <summary>
        /// Handles a collision.
        /// </summary>
        /// <param name="sender">Sender</param>
        /// <param name="e">Collision event arguments</param>
        public override void HandleCollision(object sender, CollisionEventArgs e)
        {
            if (e.Type == CollisionType.BallAndLeftLevelBorder || e.Type == CollisionType.BallAndRightLevelBorder)
            {
                health -= 7;
            }
        }

        /// <summary>
        /// Updates the health to real health.
        /// </summary>
        public void Update()
        {
            if (Health > health)
                Health -= 1;
            if (Health <= 0)
            {
                Health = 0;
                State = PongPlayerState.Defeated;
            }
        }
    }
}
