﻿using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.Assets;


/// <summary>
/// Sivusta päin kuvattu auto, jossa on kaksi pyörää.
/// </summary>
class Auto : PhysicsObject
{
    public PhysicsObject vasenPyora;
    public PhysicsObject oikeaPyora;

    public AxleJoint vasenLiitos;
    public AxleJoint oikeaLiitos;

    public Auto( double leveys, double korkeus )
        : base( leveys, korkeus )
    {
        CollisionIgnoreGroup = 100;
        Mass = 1000;
        AddedToGame += LisaaPyorat;
    }

    /// <summary>
    /// Kiihdyttää.
    /// </summary>
    /// <remarks>
    /// Vääntöä ei anneta suoraan parametrina, jotta kiihdytys toimii
    /// helposti eri massoilla.
    /// </remarks>
    /// <param name="kiihtyvyys">Kiihtyvyys autolle</param>
    public void Kiihdyta( double kiihtyvyys )
    {
        double kulmakiiht = kiihtyvyys * vasenPyora.Width / 2;
        double hitaus = MomentOfInertia + vasenPyora.MomentOfInertia + oikeaPyora.MomentOfInertia;
        double vaanto = kulmakiiht * hitaus;
        double vaantoPerPyora = vaanto / 2;

        vasenPyora.ApplyTorque( vaantoPerPyora );
        oikeaPyora.ApplyTorque( vaantoPerPyora );
    }

    void LisaaPyorat()
    {
        PhysicsGame fysiikkaPeli = Game as PhysicsGame;

        vasenPyora = LuoPyora();
        vasenPyora.Position = this.Position + new Vector( -Width / 4, -Height / 2 ); ;

        oikeaPyora = LuoPyora();
        oikeaPyora.Position = this.Position + new Vector( Width / 4, -Height / 2 );

        vasenLiitos = new AxleJoint( this, vasenPyora );
        vasenLiitos.Softness = 0.01;
        fysiikkaPeli.Add( vasenLiitos );

        oikeaLiitos = new AxleJoint( this, oikeaPyora );
        oikeaLiitos.Softness = 0.01;
        fysiikkaPeli.Add( oikeaLiitos );
    }

    public override void Destroy()
    {
        vasenLiitos.Destroy();
        oikeaLiitos.Destroy();
        vasenPyora.Destroy();
        oikeaPyora.Destroy();
        base.Destroy();
    }

    PhysicsObject LuoPyora()
    {
        double r = Width / 10;
        PhysicsObject pyora = new PhysicsObject( 2 * r, 2 * r, Shape.Circle );
        pyora.Mass = this.Mass / 20;
        pyora.Color = Color.Gray;
        pyora.CollisionIgnoreGroup = 100;
        pyora.AngularDamping = 0.95;
        pyora.KineticFriction = 1.0;
        Game.Add( pyora );
        return pyora;
    }
}


public class Peli : PhysicsGame
{
    Auto auto;

    public override void Begin()
    {
        LuoKentta();
        AsetaOhjaimet();
    }

    void LuoKentta()
    {
        Gravity = new Vector( 0, -400 );
        Level.CreateBorders();
        Camera.ZoomToLevel();

        auto = new Auto( 100, 40 );
        auto.Color = Color.Red;
        auto.X = 0;
        auto.Y = Level.Bottom + 40;
        Add( auto );
    }

    void AsetaOhjaimet()
    {
        Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, "Poistu" );
        Keyboard.Listen( Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet" );

        Keyboard.Listen( Key.Left, ButtonState.Down, auto.Kiihdyta, "Aja vasemmalle", 0.5 );
        Keyboard.Listen( Key.Right, ButtonState.Down, auto.Kiihdyta, "Aja oikealle", -0.5 );
    }
}
