using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Assets;
using Jypeli.Widgets;

/*
 * Authors: Tero Jäntti, Tomi Karppinen, Janne Nikkanen.
 */

namespace Autopeli
{
    public class Peli : TopDownPhysicsGame
    {
        const int autojenMaara = 3;
        const int kierrostenMaara = 1;

        Automobile[] autot = new Automobile[autojenMaara];
        IntMeter[] checkpointLaskurit = new IntMeter[autojenMaara];
        IntMeter[] kierrosLaskurit = new IntMeter[autojenMaara];
        PhysicsObject[] seuraavaCheckPoint = new PhysicsObject[autojenMaara];

        /// <summary>
        /// Lista tarkastuspisteistä, joiden kautta pitää ajaa.
        /// </summary>
        List<PhysicsObject> checkpoints = new List<PhysicsObject>();

        /// <summary>
        /// Lista näkymättömistä tarkastuspisteistä, joita tekoäly seuraa.
        /// </summary>
        List<PhysicsObject> tekoAlyCheckpointit = new List<PhysicsObject>();

        Widget lahtoValoSysteemi;
        Widget[] lahtoValot = new Widget[5];
        int valojaSytytetty = 0;

        PhysicsObject maaliviiva;
        List<PhysicsObject> maaliinTulleet = new List<PhysicsObject>();

        Image checkPointKuva = LoadImage( "checkpoint" );
        Image maaliviivanKuva = LoadImage( "maaliviiva" );
        Image numero1 = LoadImage( "numero_1" );
        Image numero2 = LoadImage( "numero_2" );
        Image numero3 = LoadImage( "numero_3" );


        public override void Begin()
        {
            SplashScreen aloitusruutu = new SplashScreen(
                "Autopeli",
                "Tekijät: Tero Jäntti, Tomi Karppinen, Janne Nikkanen",
                "Jyväskylän yliopisto, Tietotekniikan laitos 2009",
                "Aja kilpaa toisen pelaajan ja tietokoneen kanssa."
            );
            aloitusruutu.GameStarted += AloitaUusiPeli;
            Add( aloitusruutu );
        }

        void AloitaUusiPeli()
        {
            ClearAll();

            Gravity = 50;
            KineticFriction = 2.0;

            AlustaMaasto();
            LisaaRata();
            LisaaAutot();
            LuoNaytot();

            NaytaLahtoValot();
            //AloitaKisa();
            }

        void NaytaLahtoValot()
        {
            LuoLahtoValot();

            Timer valoAjastin = new Timer();
            valoAjastin.Interval = 1.0;
            valoAjastin.Timeout += SytytaValo;
            valoAjastin.Start( 5 );

            Timer.SingleShot( 6.0, AloitaKisa );
        }

        void SytytaValo()
        {
            lahtoValot[valojaSytytetty].Color = Color.Red;
            valojaSytytetty++;
        }

        void SammutaValot()
        {
            foreach ( var valo in lahtoValot )
            {
                valo.Color = Color.DarkRed;
            }

            // Tämä aika on alkaen siitä kun lähtövalot luotiin.
            lahtoValoSysteemi.MaximumLifetime = TimeSpan.FromSeconds( 7 );
        }

        void AloitaKisa()
        {
            SammutaValot();
            AsetaOhjaimet();

            foreach ( PhysicsObject auto in autot )
            {
                if ( auto.Brain != null )
                {
                    auto.Brain.Active = true;
                }
            }
        }

        void LuoNaytot()
        {
            Widget p0 = LuoPelaajaNaytto( 0, Color.Red );
            p0.X = Screen.LeftSafe + p0.Width * 0.6;
            p0.Y = Screen.TopSafe - p0.Height * 0.6;

            Widget p1 = LuoPelaajaNaytto( 1, Color.Green );
            p1.X = Screen.Center.X - 20;
            p1.Y = Screen.TopSafe - p1.Height * 0.6;

            Widget p2 = LuoPelaajaNaytto( 2, Color.Blue );
            p2.X = Screen.RightSafe - p2.Width * 0.6;
            p2.Y = Screen.TopSafe - p2.Height * 0.6;
        }

        void LuoLahtoValot()
        {
            lahtoValoSysteemi = new Widget( 300, 50 );
            lahtoValoSysteemi.Color = Color.Black;
            HorizontalLayout asettelu = new HorizontalLayout();
            asettelu.Spacing = 10.0;
            asettelu.LeftPadding = 5;
            asettelu.RightPadding = 5;
            asettelu.TopPadding = 5;
            asettelu.BottomPadding = 5;
            lahtoValoSysteemi.Layout = asettelu;
            Add( lahtoValoSysteemi );

            for ( int i = 0; i < 5; i++ )
            {
                Widget valo = new Widget( 50, 50 );
                lahtoValot[i] = valo;
                valo.Shape = Shape.Circle;
                valo.Color = Color.DarkRed;
                lahtoValoSysteemi.Add( valo );
            }
        }

        void CheckPointtiinOsutaan( PhysicsObject checkPoint, PhysicsObject toinen )
        {
            for ( int i = 0; i < autot.Length; i++ )
            {
                if ( ( toinen == autot[i] ) && ( checkPoint == seuraavaCheckPoint[i] ) )
                {
                    LaskeKierroksia( i );
                    break;
                }
            }
        }

        void LaskeKierroksia( int i )
        {
            if ( checkpointLaskurit[i].Value >= checkpoints.Count - 1 )
            {
                kierrosLaskurit[i].Value++;
                checkpointLaskurit[i].Value = 0;
                seuraavaCheckPoint[i] = checkpoints[0];

                if ( kierrosLaskurit[i].Value == kierrostenMaara )
                {
                    TullaanMaaliin( i );
        }
            }
            else
            {
                checkpointLaskurit[i].Value++;
                int seuraavanIndeksi = checkpointLaskurit[i].Value;
                seuraavaCheckPoint[i] = checkpoints[seuraavanIndeksi];
            }
        }

        void TullaanMaaliin( int i )
        {
            maaliinTulleet.Add( autot[i] );

            if ( autot[i].Brain != null )
            {
                autot[i].Brain.Active = false;
            }

            Image kuva;

            if ( maaliinTulleet.Count == 1 )
            {
                kuva = numero1;
            }
            else if ( maaliinTulleet.Count == 2 )
            {
                kuva = numero2;
            }
            else if ( maaliinTulleet.Count == 3 )
            {
                kuva = numero3;
            }
            else
            {
                // kyynel...
                return;
            }

            Widget numero = new Widget( 100, 100 );
            numero.Image = kuva;
            numero.MaximumLifetime = TimeSpan.FromSeconds( 3 );

            Timer ajastin = new Timer();
            ajastin.Interval = 0.05;
            ajastin.Timeout += delegate { numero.Size *= 1.01; };
            ajastin.Start( (int)( 3 / ajastin.Interval ) );

            Add( numero );
        }

        void AlustaMaasto()
        {
            Level.Width = 800;
            Level.Height = 600;
            Level.CreateBorders();

            //PhysicsObject maasto = PhysicsObject.CreateStaticObject( Level.Width, Level.Height, Shape.Rectangle );
            //maasto.Color = Color.Green;
            //maasto.IgnoresCollisionResponse = true;
            //maasto.KineticFriction = 3;
            //AddSurface( maasto );

            // Temporary solution until layers are implemented.
            Level.BackgroundColor = Color.Green;
        }

        Vector[] wayPoints = new Vector[]
        {
             new Vector( -255.56859,-124.27938  ),
             new Vector( -252.53814,-131.35045  ),
             new Vector( -201.02035,-214.18295  ),
             new Vector( 40.406102,-224.28448  ),
             new Vector( 276.7818,-219.23372  ),
             new Vector( 341.43157,-124.27938  ),
             new Vector( 285.87318,-49.528091  ),
             new Vector( 149.50258,-91.954501  ),
             new Vector( 53.538082,-82.863121  ),
             new Vector( 24.243662,-62.660071  ),
             new Vector( 2.7546483e-06,1.9896889  ),
             new Vector( 47.477172,114.11662  ),
             new Vector( 111.11679,143.41105  ),
             new Vector( 202.03052,122.19784  ),
             new Vector( 300.01531,123.208  ),
             new Vector( 331.33004,194.92883  ),
             new Vector( 286.88333,278.77149  ),
             new Vector( 161.62441,280.79179  ),
             new Vector( -210.11172,270.69027  ),
             new Vector( -293.95439,203.01005  ),
             new Vector( -313.14729,2.9998489  ),
             new Vector( -284.86301,-60.639771 ),
             new Vector( -255.56859,-124.27938  ),
    };

        void LisaaRata()
            {
            RoadMap tie = new RoadMap( wayPoints );
            tie.DefaultWidth = 100.0;
            tie.Insert();

            for ( int i = 2; i < wayPoints.Length - 1; i += 1 )
            {
                if ( i % 5 == 0 )
                {
                    // Joka viidennellä pisteellä luodaan "oikea" checkpoint.
                    PhysicsObject cp = LuoCheckPoint( wayPoints[i], tie.DefaultWidth * 0.8, checkPointKuva );
                    cp.Angle = tie.GetAngle( i ) + Angle.FromRadians( Math.PI / 2 );
                    checkpoints.Add( cp );
            }

                // jokaisella pisteellä luodaan checkpoint, jota tekoäly seuraa.
                // Tällä tavoin se ei yritä oikaista.
                PhysicsObject tekoAlyCP = LuoCheckPoint( wayPoints[i], tie.DefaultWidth * 0.8, null );
                tekoAlyCP.IsVisible = false;
                tekoAlyCP.Angle = tie.GetAngle( i ) + Angle.FromRadians( Math.PI / 2 );
                tekoAlyCheckpointit.Add( tekoAlyCP );
            }

            maaliviiva = LuoCheckPoint( wayPoints[wayPoints.Length - 1], tie.DefaultWidth, maaliviivanKuva );
            maaliviiva.Angle = tie.GetAngle( wayPoints.Length - 1 ) + Angle.FromRadians( Math.PI / 2 );
            checkpoints.Add( maaliviiva );

            Camera.ZoomToLevel();
        }

        PhysicsObject LuoCheckPoint( Vector paikka, double leveys, Image kuva )
            {
            PhysicsObject cp = new PhysicsObject( leveys, 20 );
            cp.Image = kuva;
            cp.Position = paikka;
            cp.IgnoresCollisionResponse = true;
            cp.IgnoresGravity = true;
            Add( cp );
            AddCollisionHandler( cp, CheckPointtiinOsutaan );
            return cp;
            }

        void LisaaAutot()
            {
            Vector takaisinPain = Vector.FromLengthAndAngle( 30, maaliviiva.Angle + Angle.RightAngle );

            for ( int i = 0; i < autojenMaara; i++ )
            {
                autot[i] = new Automobile( 16, 8 );
                autot[i].Mass = 2000;

                autot[i].TopSpeed = 80;
                autot[i].Acceleration = 50;
                autot[i].BrakeDeceleration = 25;
                autot[i].Maneuverability = Angle.FromDegrees( 120 );

                autot[i].KineticFriction = 0.5;
                autot[i].AngularDamping = 0.5;

                autot[i].Angle = maaliviiva.Angle - Angle.RightAngle;
                Add( autot[i] );

                checkpointLaskurit[i] = new IntMeter( 0 );
                checkpointLaskurit[i].MaxValue = checkpoints.Count;
                kierrosLaskurit[i] = new IntMeter( 0 );
                kierrosLaskurit[i].MaxValue = kierrostenMaara;
                seuraavaCheckPoint[i] = checkpoints[0];
            }

            autot[0].Position = maaliviiva.Position + Vector.FromLengthAndAngle( 20, maaliviiva.Angle ) + takaisinPain;
            autot[0].Color = Color.Red;

            autot[1].Position = maaliviiva.Position + takaisinPain;
            autot[1].Color = Color.Green;
            autot[1].Brain = new ComputerDriverBrain( tekoAlyCheckpointit, kierrostenMaara );
            autot[1].Brain.Active = false;

            autot[2].Position = maaliviiva.Position - Vector.FromLengthAndAngle( 20, maaliviiva.Angle ) + takaisinPain;
            autot[2].Color = Color.Blue;
        }

        Widget LuoPelaajaNaytto( int autonIndeksi, Color vari )
        {
            Widget naytto = new Widget( 200, 100 );
            naytto.Color = new Color( 100, 100, 200, 150 );
            naytto.Color = new Color( vari.RedComponent, vari.GreenComponent, vari.BlueComponent, 180 );
            naytto.Layout = new VerticalLayout();
            Add( naytto );

            Label nopeusNaytto = new Label();
            nopeusNaytto.Title = "Nopeus";
            nopeusNaytto.DecimalPlaces = 0;
            nopeusNaytto.BindTo( autot[autonIndeksi].SpeedMeter );
            naytto.Add( nopeusNaytto );

            Label checkpointNaytto = new Label();
            checkpointNaytto.Title = "Checkpoint";
            checkpointNaytto.BindTo( checkpointLaskurit[autonIndeksi] );
            naytto.Add( checkpointNaytto );

            Label kierrosNaytto = new Label();
            kierrosNaytto.Title = "Kierros";
            kierrosNaytto.BindTo( kierrosLaskurit[autonIndeksi] );
            naytto.Add( kierrosNaytto );

            return naytto;
        }

        void AsetaOhjaimet()
        {
            Keyboard.Listen( Key.Enter, ButtonState.Pressed, AloitaUusiPeli, "Aloita uusi peli" );
            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, null );
            Keyboard.Listen( Key.Up, ButtonState.Down, autot[0].Accelerate, "Kiihdytä (pelaaja 1)" );
            Keyboard.Listen( Key.Down, ButtonState.Down, autot[0].Brake, "Jarruta (pelaaja 1)" );
            Keyboard.Listen( Key.End, ButtonState.Down, autot[0].Reverse, "Pakita (pelaaja 1)" );
            Keyboard.Listen( Key.Left, ButtonState.Down, Kaanny, "Käänny vasemmalle (pelaaja 1)", autot[0], Angle.FromDegrees( 90 ) );
            Keyboard.Listen( Key.Right, ButtonState.Down, Kaanny, "Käänny oikealle (pelaaja 1)", autot[0], Angle.FromDegrees( -90 ) );

            Keyboard.Listen( Key.W, ButtonState.Down, autot[2].Accelerate, "Kiihdytä (pelaaja 2)" );
            Keyboard.Listen( Key.E, ButtonState.Down, autot[2].Reverse, "Pakita (pelaaja 2)" );
            Keyboard.Listen( Key.S, ButtonState.Down, autot[2].Brake, "Jarruta (pelaaja 2)" );
            Keyboard.Listen( Key.A, ButtonState.Down, Kaanny, "Käänny vasemmalle (pelaaja 2)", autot[2], Angle.FromDegrees( 90 ) );
            Keyboard.Listen( Key.D, ButtonState.Down, Kaanny, "Käänny oikealle (pelaaja 2)", autot[2], Angle.FromDegrees( -90 ) );

            //AlustaXboxKontrollit( PlayerIndex.One, autot[0] );
            //AlustaXboxKontrollit( PlayerIndex.Two, autot[2] );

            ShowControlHelp();
        }

        void AlustaXboxKontrollit( GamePad controller, Automobile auto )
        {
            //Controls.Listen( pelaaja, Buttons.A, ButtonState.Down, Kiihdyta, "Kiihdytä", auto );
            //Controls.Listen( pelaaja, Buttons.B, ButtonState.Down, Jarruta, "Jarruta", auto );
            //Controls.Listen( pelaaja, Buttons.LeftTrigger, ButtonState.Down, Kaanny, "Käänny vasemmalle", auto, Angle.FromDegrees( 90 ) );
            //Controls.Listen( pelaaja, Buttons.RightTrigger, ButtonState.Down, Kaanny, "Käänny oikealle", auto, Angle.FromDegrees( -90 ) );

            //Controls.Listen( pelaaja, Buttons.Start, ButtonState.Pressed, UusiPeli, "Aloita uusi peli", auto );
            //Controls.Listen( pelaaja, Buttons.LeftThumbstickUp, ButtonState.Down, Kiihdyta, "Kiihdytä", auto );
            //Controls.Listen( pelaaja, Buttons.LeftThumbstickDown, ButtonState.Down, Jarruta, "Jarruta", auto );
            ////Controls.Listen( pelaaja, Buttons.LeftThumbstickLeft, ButtonState.Down, Kaanny, "Käänny vasemmalle", auto, Angle.FromDegrees( 90 ) );
            ////Controls.Listen( pelaaja, Buttons.LeftThumbstickRight, ButtonState.Down, Kaanny, "Käänny oikealle", auto, Angle.FromDegrees( -90 ) );

            //Controls.Listen( pelaaja, Buttons.Start, ButtonState.Pressed, UusiPeli, "Aloita uusi peli", auto );
            ////Controls.Listen( pelaaja, Buttons.RightThumbstickUp, ButtonState.Down, Kiihdyta, "Kiihdytä", auto );
            ////Controls.Listen( pelaaja, Buttons.RightThumbstickDown, ButtonState.Down, Jarruta, "Jarruta", auto );
            //Controls.Listen( pelaaja, Buttons.RightThumbstickLeft, ButtonState.Down, Kaanny, "Käänny vasemmalle", auto, Angle.FromDegrees( 90 ) );
            //Controls.Listen( pelaaja, Buttons.RightThumbstickRight, ButtonState.Down, Kaanny, "Käänny oikealle", auto, Angle.FromDegrees( -90 ) );
        }

        void Kiihdyta( Automobile auto )
        {
            auto.Accelerate();
        }

        void Jarruta( Automobile auto )
        {
            auto.Brake();
        }

        void Kaanny( Automobile auto, Angle kulma )
        {
            //Angle kaannos = kulma * Math.Abs( e.AnalogState );
            Angle kaannos = kulma * 0.1;
            auto.Turn( kaannos, Time.SinceLastUpdate.TotalSeconds );
        }
    }
}
