﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.Assets;
using Jypeli.WP7;

/// <summary>
/// Author: Antti-Jussi Lakanen
/// Klooni klassisesta FallDown-pelistä.
/// Versiotietoa:
/// 31.5.2010: Melkein Jypeli2-yhteensopiva. Hiiren kuuntelu ei toimi vielä,
/// joten valikko skipataan tässä vaiheessa. Kenttää pitää paarntaa, on nyt aika vaikea.
/// 19.5.2010: Käyttää Jypeli-versiota 1.2.5
/// </summary>
public class FallDown : PhysicsGame
{
    # region Vakiot
    const double TASOJEN_VALIMATKA = 70;
    const double LEVEYS = 25;
    const double KORKEUS = 10;
    Color TASOJEN_VARI = Color.Blue;
    Color REUNOJEN_VARI = Color.Blue;
    const double kameranNopeus = 0.000004;
    const double PainoVoima = 200;
    const double PallonMassa = 2;
    const double LiikeVoima = 500;
    const double MaksimiLiikeVoimaKosketusNaytolla = 700;
    #endregion

    #region Muut attribuutit
    List<PhysicsObject> tasot = new List<PhysicsObject>();
    List<PhysicsObject> esineet = new List<PhysicsObject>();
    int kenttanro = 1;
    PhysicsObject pallo;
    bool peliKaynnissa = false;

    Timer pistelaskuriAjastin;

    IntMeter pistelaskuri = new IntMeter(0); // Pitää yllä pisteitä
    Label pisteNaytto; // Nayttaa pisteet
    ScoreList topten;

    Image vauhtiKuva = LoadImage( "speed" );
    Image maalinKuva = LoadImage( "maali" );
    Image plus5Kuva = LoadImage("plus5");
    Image pallonKuva = LoadImage( "pallo" );
    Image aloitusNappulanKuva = LoadImage( "aloita" );
    Image aloitusNappulanHoverKuva = LoadImage( "aloita_" );
    Image lopetusNappulanKuva = LoadImage( "lopeta" );
    Image lopeutusNappulanHoverKuva = LoadImage( "lopeta_" );
    Image Tausta = LoadImage( "avaruus" );
    Image[] litistysKuvat = LoadImages(
        "squished0","squished1","squished2","squished3","squished4",
        "squished5","squished6","squished7","squished8","squished9");


    /// <summary>
    /// Näyttää valikon
    /// </summary>
    void NaytaValikko()
    {
        ClearAll();

#if !WINDOWS_PHONE
        MediaPlayer.Play( "musa1" );
#endif

        Level.Background.Image = Tausta;
        Level.Background.Size = new Vector(Screen.Width, Screen.Height);

        PushButton aloitusNappula = new PushButton( 200, 60, aloitusNappulanKuva );
        aloitusNappula.ImageHover = aloitusNappulanHoverKuva;
        aloitusNappula.X = 0;
        aloitusNappula.Y = 50;
        aloitusNappula.Clicked += AloitaPeliAlusta;
        TouchPanel.ListenOn(aloitusNappula, ButtonState.Pressed, AloitaPeliAlusta, null, aloitusNappula);
        Add( aloitusNappula );

        PushButton lopetusNappula = new PushButton( 200, 60, lopetusNappulanKuva );
        lopetusNappula.ImageHover = lopeutusNappulanHoverKuva;
        lopetusNappula.X = 0;
        lopetusNappula.Y = -50;
        lopetusNappula.Clicked += Exit;
        TouchPanel.ListenOn(lopetusNappula, ButtonState.Pressed, Lopeta, null, lopetusNappula);
        Add( lopetusNappula );

        Label tekijat = new Label();
        tekijat.TextColor = Color.Green;
        tekijat.Text = "Tekijät:\n"+
            "Ohjelmointi ja grafiikka: Antti-Jussi Lakanen\n"+
            "Musiikki: Ville-Matti Pasanen";
        tekijat.X = Screen.Center.X + 250;
        tekijat.Y = Screen.Center.Y - 300;
        Add( tekijat );

        Mouse.IsCursorVisible = true;
    }

    void AloitaPeliAlusta(Touch kosketus, PushButton aloitusNappula)
    {
        aloitusNappula.Image = aloitusNappulanHoverKuva;
        Timer.SingleShot(0.2, AloitaPeliAlusta); // Viive jotta kuva kerkeää vaihtua
    }

    void Lopeta(Touch kosketus, PushButton lopetusNappula)
    {
        lopetusNappula.Image = lopeutusNappulanHoverKuva;
        Timer.SingleShot(0.5, Exit);
    }

    #endregion


    /// <summary>
    /// Tuo parhaiden pisteiden listan ruudulle.
    /// Mikali pelaajan pisteet yltavat topten-listalle,
    /// kysytaan kayttajalta nimimerkki, ja laitetaan se 
    /// topten-listalle.
    /// Topten-lista tallennetaan tiedostoon, joka on maaritelty
    /// LoadContentissa.
    /// </summary>
    private void NaytaTopTen()
    {
        HighScoreWindow toptenWindow = new HighScoreWindow( "Parhaat pisteet", "Kymmenen parasta pelaajaa", "Pistemääräsi oli %p, jolla pääset parhaiden pelaajien joukkoon!", topten, pistelaskuri.Value );
        //topten.ItemColor = Color.Red;
        //topten.Color = Color.Black;
        toptenWindow.Closed += new Jypeli.Widgets.Window.WindowHandler( ToptenSuljettu );
        Add( toptenWindow );
    }

    void ToptenSuljettu( Window sender )
    {
        DataStorage.Save<ScoreList>( topten, "topten" );
        NaytaValikko();
    }

    public override void Begin()
    {
        Phone.DisplayOrientation = DisplayOrientation.Portrait;

        topten = DataStorage.Load<ScoreList>( new ScoreList( 10, false, 0 ), "topten" );
#if !WINDOWS_PHONE
        MediaPlayer.Volume = 0.2;
        MediaPlayer.IsRepeating = true;
#endif
        NaytaValikko();
    }

    void AloitaPeliAlusta()
    {
        ClearAll();
#if !WINDOWS_PHONE
        MediaPlayer.Play( "musa2" );
#endif
        pistelaskuri.Reset();
        TeePisteNaytto();
        kenttanro = 1;
        AlustaSeuraavaKentta();
        AloitaKenttaAlusta();
    }

    void AloitaKenttaAlusta(Touch kosketus)
    {
        AloitaKenttaAlusta();
    }

    void AloitaKenttaAlusta()
    {
        pallo.Stop();
        pallo.Y = Level.Top - TASOJEN_VALIMATKA * 3.0 / 4.0;
        pallo.X = Level.Left + pallo.Width;
        pallo.Image = pallonKuva;
        pallo.Mass = PallonMassa;

        LueKentta(false, true);

        Camera.Y = Level.Top;
        Camera.ZoomFactor = Screen.Width / Level.Width * 0.95;

        AsetaPeliKontrollit();

        peliKaynnissa = true;
    }

    string[] NykyinenKentta()
    {
        if (kenttanro == 1)
        {
            return Kentat.Kentta1;
        }
        else
        {
            return Kentat.Kentta2;
        }
    }

    /// <summary>
    /// Kenttä alkaa (kentän palikoiden luonti, kameran asettaminen, pallon luonti, 
    /// </summary>
    void AlustaSeuraavaKentta()
    {
        ClearAll();
        MessageDisplay.TextColor = Color.White; // ClearAll()-metodi aliohjelmassa AloitaPeliAlusta resetoi myös MessageDisplayn tekstin värin            

        string[] ruutukentta = NykyinenKentta();
        Level.Width = ruutukentta[0].Length * LEVEYS;
        Level.Height = ruutukentta.Length * TASOJEN_VALIMATKA;

        LueKentta(true, false);

        PhysicsObject vasenReuna = Level.CreateLeftBorder();
        vasenReuna.Color = REUNOJEN_VARI;
        PhysicsObject oikeaReuna = Level.CreateRightBorder();
        oikeaReuna.Color = REUNOJEN_VARI;
        PhysicsObject alaReuna = Level.CreateBottomBorder();
        alaReuna.Color = REUNOJEN_VARI;
        PhysicsObject ylaReuna = Level.CreateTopBorder();
        ylaReuna.Color = REUNOJEN_VARI;

        Level.Background.Image = Tausta;
        Level.Background.Size = new Vector(Level.Width, Level.Height);
        Level.Background.MovesWithCamera = false;

        TeePeliPallo();
        Gravity = new Vector(0, -PainoVoima);
    }

    void PisteitaLisaa(int montako)
    {
        pistelaskuri.Value += montako;
    }

    private void TeePisteNaytto()
    {
        pisteNaytto = new Label();
        pisteNaytto.TextColor = Color.White;
        pisteNaytto.BindTo(pistelaskuri);
        pisteNaytto.X = Screen.RightSafe - 200;
        pisteNaytto.Y = Screen.TopSafe - 2 * pisteNaytto.Height;
        Add(pisteNaytto);

        //pisteNaytto.FormatDouble(0, true); // asetetaan nayttämaan siten, ettei nay desimaaleja

        // Alustetaan ajastin joka lisaa pisteita pelaajalle sita mukaa kun peli etenee
        pistelaskuriAjastin = new Timer();
        pistelaskuriAjastin.Interval = 0.5;
        pistelaskuriAjastin.Timeout += delegate { PisteitaLisaa(1); };
        pistelaskuriAjastin.Start();
    }

    private void LueKentta(bool luoTasot, bool luoEsineet)
    {
        if (luoTasot)
        {
            foreach (PhysicsObject taso in tasot)
                taso.Destroy();
            tasot.Clear();
        }

        if (luoEsineet)
        {
            foreach (PhysicsObject esine in esineet)
                esine.Destroy();
            esineet.Clear();
        }

        // Kaydaan rivit ja sarakkeet lapi.
        // "Muutetaan" merkkitaulukko kentaksi siten
        // etta tietyt merkit vastaavat tietyn kentalle lisattavan olion
        // aliohjelmakutsua. Vierekkäiset tasomerkit kootaan yhdeksi olioksi.
        // Myos koordinaatit saadaan matriisista:
        // Rivi toimii x-koordinaattina, ja sarake y-koordinaattina.

        string[] ruutuKentta = NykyinenKentta();
        int leveys = ruutuKentta[0].Length;

        for (int r = 0; r < ruutuKentta.Length; r++)
        {
            int tasonAlku = -1;

            for (int s = 0; s < leveys; s++)
            {
                char t = ruutuKentta[r][s];

                if (!char.IsWhiteSpace(t))
                {
                    if (tasonAlku < 0)
                    {
                        tasonAlku = s;
                    }
                    
                    switch (t)
                    {
                        case 'x':
                            break;
                        case 'o':
                            if (luoEsineet)
                            {
                                PhysicsObject n = TeeNopeudenkasvattaja();
                                n.X = s * LEVEYS + Level.Left + LEVEYS / 2;
                                n.Y = Level.Top - r * TASOJEN_VALIMATKA + KORKEUS;
                            }
                            break;
                        case '5':
                            if (luoEsineet)
                            {
                                PhysicsObject p = TeePlusPiste();
                                p.X = s * LEVEYS + Level.Left + LEVEYS / 2;
                                p.Y = Level.Top - r * TASOJEN_VALIMATKA + KORKEUS;
                            }
                            break;
                        case 'z':
                            PhysicsObject m = TeeMaali(10);
                            m.X = s * LEVEYS + Level.Left + LEVEYS / 2;
                            m.Y = Level.Top - r * TASOJEN_VALIMATKA + KORKEUS;
                            break;
                        default:
                            break;
                    }
                }
                else
                {
                    if (tasonAlku >= 0 && luoTasot)
                    {
                        TeeTaso(r, tasonAlku, s);
                        tasonAlku = -1;
                    }
                }
            }

            if (tasonAlku >= 0 && luoTasot)
            {
                TeeTaso(r, tasonAlku, ruutuKentta[r].Length);
            }

        }
    }

    void TeeTaso(int rivi, int alkuSarake, int loppuSarake)
    {
        PhysicsObject taso = TeeTaso((loppuSarake - alkuSarake) * LEVEYS, KORKEUS);
        taso.X = Level.Left + (alkuSarake + (loppuSarake - alkuSarake) / 2.0) * LEVEYS;
        taso.Y = Level.Top - rivi * TASOJEN_VALIMATKA;
        tasot.Add(taso);
    }

    PhysicsObject TeeTaso(double leveys, double korkeus)
    {
        PhysicsObject p = PhysicsObject.CreateStaticObject(leveys, korkeus, Shape.Rectangle);
        p.Color = TASOJEN_VARI;
        p.Restitution = 0.1;
        Add(p);
        p.Tag = "Taso";
        return p;
    }

    PhysicsObject TeeNopeudenkasvattaja()
    {
        PhysicsObject p = TeePallo(10, KORKEUS / 2);
        p.Image = vauhtiKuva;
        p.Tag = "Nopeudenkasvattaja";
        Add(p);
        esineet.Add(p);
        return p;
    }

    /// <summary>
    /// Staattinen pallo-olio kerättäviä objekteja varten, jotka eivät aiheuta törmäystä
    /// </summary>
    /// <param name="massa"></param>
    /// <param name="sade"></param>
    /// <returns></returns>
    PhysicsObject TeePallo(double massa, double sade)
    {
        PhysicsObject p = PhysicsObject.CreateStaticObject(2 * sade, 2 * sade);
        p.Shape = Shape.Circle;
        p.IgnoresCollisionResponse = true;
        return p;
    }

    PhysicsObject TeePlusPiste()
    {
        PhysicsObject p = TeePallo(10, KORKEUS / 2);
        p.Tag = 5;
        p.Image = plus5Kuva;
        Add(p);
        esineet.Add(p);
        return p;
    }

    PhysicsObject TeeMaali(double sade)
    {
        PhysicsObject p = PhysicsObject.CreateStaticObject(2 * sade, 2 * sade);
        p.Shape = Shape.Circle;
        p.Tag = "maali";
        p.Image = maalinKuva;
        Add(p);
        return p;
    }

    void PalloTormasi(PhysicsObject pallo, PhysicsObject kohde)
    {
        switch (kohde.Tag.ToString())
        {
            case "Nopeudenkasvattaja":
                MessageDisplay.Add("Nopeus kasvoi");
                pallo.Mass /= 2;
                kohde.Destroy();
                Phone.Vibrate(150);
                Timer.SingleShot(5, PalautaMassaJaVari);
                break;
            case "maali":
                MessageDisplay.Add("Pääsit tämän kentän loppuun, hyvä!");
                kenttanro++;
                AlustaSeuraavaKentta();
                break;
            default:
                break;
        }

        if (kohde.Tag is int) // Pallo voi törmätä olioihin jolla ei ole tägiä, ja siksi tarkistus tarvitaan
        {
            MessageDisplay.Add("Sait " + kohde.Tag.ToString() + " lisäpistettä");
            kohde.Destroy();
            Phone.Vibrate(100);
            PisteitaLisaa((int)kohde.Tag);
        }
    }

    void PalautaMassaJaVari()
    {
        MessageDisplay.Add("Nopeus palautettu");
        pallo.Mass *= 2;
    }

    private void TeePeliPallo()
    {
        pallo = new PhysicsObject(10, 10, Shape.Circle);
        pallo.Mass = PallonMassa;
        pallo.Image = pallonKuva;
        Add(pallo);
        pallo.KineticFriction = 0.1;
        pallo.LinearDamping = 0.99;
        AddCollisionHandler(pallo, PalloTormasi);

    }

    void Kuolema()
    {
        pallo.Angle = Angle.Zero;
        pallo.Stop();
        pallo.Animation = new Animation(litistysKuvat);
        pallo.Animation.FPS = 15;
        pallo.Animation.StopOnLastFrame = true;
        pallo.Animation.Start(1);
        pistelaskuriAjastin.Stop();
        MessageDisplay.Add("Pisteesi olivat: " + pistelaskuri.Value);
        peliKaynnissa = false;
        ClearControls();
        NaytaTopTen();
    }

    protected override void Update(Time gameTime)
    {
        if (peliKaynnissa)
        {
            Camera.Position.Y -= gameTime.SinceLastUpdate.Ticks * kameranNopeus;

            if (Camera.ScreenToWorld(new Vector(0, Screen.Top)).Y < pallo.Y + pallo.Height / 2)
            {
                Kuolema();
            }
        }

        base.Update(gameTime);
    }


    void AsetaPeliKontrollit()
    {
        ClearControls();
        Keyboard.Listen(Key.Left, ButtonState.Down, AsetaNopeus, "Liikkuu vasemmalle", new Vector(-LiikeVoima, 0));
        Keyboard.Listen(Key.Left, ButtonState.Released, AsetaNopeus, null, Vector.Zero);
        Keyboard.Listen(Key.Right, ButtonState.Down, AsetaNopeus, "Liikkuu oikealle", new Vector(LiikeVoima, 0));
        Keyboard.Listen(Key.Right, ButtonState.Released, AsetaNopeus, null, Vector.Zero);

        Accelerometer.Listen(AccelerometerDirection.Any, AccelerometerSensitivity.High, AsetaNopeus, null);
    }

    void AsetaPainovoima()
    {
        Gravity = Accelerometer.Reading * PainoVoima;
    }

    void AsetaNopeus()
    {
        double nopeus = Accelerometer.Reading.X * MaksimiLiikeVoimaKosketusNaytolla;
        AsetaNopeus(new Vector(nopeus, 0));
    }

    void AsetaNopeus(Vector voima)
    {
        if ((voima.X < 0) && (pallo.X < Level.Left))
        {
            pallo.Velocity = Vector.Zero;
        }
        if ((voima.X > 0) && (pallo.X > Level.Right))
        {
            pallo.Velocity = Vector.Zero;
        }

        pallo.Push(voima);
    }
}
