﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;

public class HingeJointRengas
{
    private List<PhysicsObject> rengas;
    private PhysicsObject keskikappale;

    public List<PhysicsObject> Rengas
    {
        get { return rengas; }
        set { }
    }

    public PhysicsObject Keskikappale
    {
        get { return keskikappale; }
        set { }
    }


    public HingeJointRengas(PhysicsGame currentGame, Vector position)
    {
        this.rengas = LisaaRengas(currentGame, position);
    }

    private List<PhysicsObject> LisaaRengas(PhysicsGame game, Vector position)
    {
        int sade = 200;
        int sivu = 50;
        int siirtyma = 15;
        double pehmeys = 0.01;

        List<PhysicsObject> rengas = new List<PhysicsObject>();

        this.keskikappale = new PhysicsObject(100, 100, Shape.Circle);
        this.keskikappale.MomentOfInertia = 0.1;
        this.keskikappale.Mass = 1000;
        this.keskikappale.Position = position;
        game.Add(keskikappale);

        PhysicsObject edellinenKappale = null;
        PhysicsObject ensimmainenKappale = null;

        for (int i = 0; i < 24; i++)
        {
            PhysicsObject kappale = new PhysicsObject(sivu, sivu, Shape.Rectangle, CollisionShapeQuality.Best);
            rengas.Add(kappale);
            kappale.Mass = 40;
            kappale.StaticFriction = 1;
            kappale.KineticFriction = 1;
            Angle kulma = Angle.FromDegrees(i * siirtyma);
            kappale.Position = position + Vector.FromLengthAndAngle(sade, Angle.FromDegrees(i * siirtyma));
            kappale.Angle = kulma;
            kappale.CollisionIgnoreGroup = 1;
            game.Add(kappale);

            Vector iv = Vector.FromLengthAndAngle(sivu, kulma);
            Vector jv = Vector.FromLengthAndAngle(sivu, kulma + Angle.RightAngle);
            Vector kulmaPiste = kappale.Position - iv / 2 - jv / 2;

            if (edellinenKappale != null)
            {
                AxleJoint topHinge = new AxleJoint( kappale, edellinenKappale, kulmaPiste );
                topHinge.Softness = pehmeys;
                game.Add(topHinge);

            }
            else
            {
                ensimmainenKappale = kappale;
            }

            AxleJoint hj = new AxleJoint( keskikappale, kappale, position );
            hj.Softness = pehmeys;
            game.Add(hj);

            edellinenKappale = kappale;

        }

        Vector kv = Vector.FromLengthAndAngle(sivu, edellinenKappale.Angle);
        Vector lv = Vector.FromLengthAndAngle(sivu, edellinenKappale.Angle + Angle.RightAngle);
        Vector viimeinenPiste = edellinenKappale.Position - kv / 2 + lv / 2;

        AxleJoint finalHinge = new AxleJoint( edellinenKappale, ensimmainenKappale, viimeinenPiste );
        finalHinge.Softness = pehmeys;
        game.Add(finalHinge);

        return rengas;
    }

    public void Accelerate(double vaantoVoima)
    {
        foreach (PhysicsObject renkaanOsa in Rengas)
        {
            Vector impulssi = Vector.FromLengthAndAngle(vaantoVoima, renkaanOsa.Angle + Angle.RightAngle);
            renkaanOsa.Hit(impulssi);
        }
    }
}


public class Peli : PhysicsGame
{
    HingeJointRengas oikeaRengas;
    const double nopeus = 900;

    public override void Begin()
    {
        Level.Height = 2000;
        Level.Width = 10000;
        Level.CreateBorders();
        Gravity = new Vector(0, -1000);
        IsMouseVisible = true;

        //TeeSaranaSilta();

        //PhysicsObject mopo = new PhysicsObject(1000, 300);
        //mopo.Mass = 5000;
        //mopo.Color = Color.Fuchsia;
        //mopo.Position = new Vector(Level.Left + 600, 0);
        //Add(mopo);

        oikeaRengas = new HingeJointRengas(this, new Vector(0, -350));

        LisaaKontrollit();

        // Camera.ZoomFactor = 0.15;
        //Camera.Follow(oikeaRengas);
        Camera.ZoomToLevel();

        //Level.CreateGround(10, 200, 20);
    }

    void TeeSaranaSilta()
    {
        int kierroksia = 15;
        int siirtyma = 50;
        double jointSiirtyma = siirtyma / 2;

        PhysicsObject alku = PhysicsObject.CreateStaticObject(50, 50);
        alku.X = -450;
        Add(alku);

        PhysicsObject viimeksiLisatty = alku;

        for (int i = 0; i < kierroksia; i++)
        {
            PhysicsObject p = new PhysicsObject(50, 50, Shape.Rectangle);
            p.Mass = 20;
            p.X = viimeksiLisatty.X + siirtyma;
            AxleJoint topHinge = new AxleJoint(p, viimeksiLisatty, new Vector(viimeksiLisatty.X + jointSiirtyma, viimeksiLisatty.Top));
            topHinge.Softness = 0.01;
            AxleJoint bottomHinge = new AxleJoint(p, viimeksiLisatty, new Vector(viimeksiLisatty.X + jointSiirtyma, viimeksiLisatty.Bottom));
            bottomHinge.Softness = 0.01;

            Add(p);
            Add(topHinge);
            Add(bottomHinge);
            viimeksiLisatty = p;
        }

        PhysicsObject loppu = PhysicsObject.CreateStaticObject(50, 50);
        loppu.X = viimeksiLisatty.X + siirtyma;
        AxleJoint endTopHinge = new AxleJoint(loppu, viimeksiLisatty, new Vector(viimeksiLisatty.X + jointSiirtyma, viimeksiLisatty.Top));
        endTopHinge.Softness = 0.01;
        AxleJoint endBottomHinge = new AxleJoint(loppu, viimeksiLisatty, new Vector(viimeksiLisatty.X + jointSiirtyma, viimeksiLisatty.Bottom));
        endBottomHinge.Softness = 0.01;
        
        Add(loppu);
        Add(endTopHinge);
    }

    void LisaaKontrollit()
    {
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, LuoPallura, null);
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, null);

        Keyboard.Listen(Key.Right, ButtonState.Down, Accelerate, null, oikeaRengas, -nopeus);
        Keyboard.Listen(Key.Left, ButtonState.Down, Accelerate, null, oikeaRengas, nopeus);
    }

    void Accelerate(HingeJointRengas rengas, double voima)
    {
        rengas.Accelerate(voima); 
    }

    void LuoPallura()
    {
        PhysicsObject palikka = new PhysicsObject(60, 60, Shape.Circle);
        //palikka.Mass = RandomGen.NextDouble(30.0, 100.0);
        palikka.Mass = 4000.0;
        palikka.Color = RandomGen.NextColor();
        palikka.Position = Mouse.PositionOnWorld;
        Add(palikka);
    }
}
