using System;
using System.Collections.Generic;
using System.Linq;
using Jypeli;
using Jypeli.Controls;

namespace Muistipeli
{
    public class Kortti : GameObject
    {
        private enum LiikkeenTila { Pysahtynyt, Pienentyy, Suurentuu }
        
        protected Vector alkuperainenKoko;
        private LiikkeenTila liikkeenTila;
        
        PuoliEsilla puoliEsilla;
        private Image kuvapuoliTekstuuri;
        public Image KuvaPuoliTekstuuri
        {
            set { kuvapuoliTekstuuri = value; }
            get { return kuvapuoliTekstuuri; }
        }
        private Image takapuoliTekstuuri;

        public Paikka Paikka
        {
            set { paikka = value; }
            get { return paikka; }
        }
        private Paikka paikka;

        public enum PuoliEsilla { Kuva, Taka }

        private bool poistettuPelista;
        public bool PoistettuPelista 
        {
            get { return poistettuPelista; }
            set { poistettuPelista = value; }
        }

        private int tunniste;

        /// <summary>
        /// Kortin tunnus, jolla parit tunnistetaan pariksi.
        /// </summary>
        public int Tunniste
        {
            get { return tunniste; }
        }


        public Kortti(double leveys, double korkeus, bool kuvapuoliEsilla, Image kuvapuoliTekstuuri, Image takapuoliTekstuuri, int tunnus)
            : base(leveys, korkeus, Shape.Rectangle)
        {
            this.kuvapuoliTekstuuri = kuvapuoliTekstuuri;
            this.takapuoliTekstuuri = takapuoliTekstuuri;
            this.liikkeenTila = LiikkeenTila.Pysahtynyt;
            this.poistettuPelista = false;

            if (kuvapuoliEsilla)
                AsetaPuoli(PuoliEsilla.Kuva);
            else
                AsetaPuoli(PuoliEsilla.Taka);

            IsUpdated = true;
            this.alkuperainenKoko = new Vector(leveys, korkeus);
            this.tunniste = tunnus;
        }

        public Kortti(double leveys, double korkeus, bool kuvapuoliEsilla, Image takapuoliTekstuuri, int tunnus)
            : base(leveys, korkeus, Shape.Rectangle)
        {
            this.kuvapuoliTekstuuri = Image.FromColor((int)Math.Floor(leveys), (int)Math.Floor(korkeus), Color.White);
            this.takapuoliTekstuuri = takapuoliTekstuuri;
            this.liikkeenTila = LiikkeenTila.Pysahtynyt;
            this.poistettuPelista = false;

            if (kuvapuoliEsilla)
                AsetaPuoli(PuoliEsilla.Kuva);
            else
                AsetaPuoli(PuoliEsilla.Taka);

            IsUpdated = true;
            this.alkuperainenKoko = new Vector(leveys, korkeus);
            this.tunniste = tunnus;
        }

        public void Kaanna()
        {
            liikkeenTila = LiikkeenTila.Pienentyy;
        }

        /// <summary>
        /// Aliohjelma palauttaa tiedon siit, onko kortin animaatio kynniss vai ei.
        /// </summary>
        /// <returns>Animaatio kynniss.</returns>
        public bool OnkoLiikkeessa()
        {
            if (liikkeenTila == LiikkeenTila.Pysahtynyt)
                return false;
            return true;
        }

        public PuoliEsilla KumpiPuoliEsilla()
        {
            return this.puoliEsilla;
        }

        private void AsetaPuoli(PuoliEsilla puoli)
        {
            this.puoliEsilla = puoli;
            switch (puoli)
            {
                case PuoliEsilla.Kuva:
                    this.Image = kuvapuoliTekstuuri;
                    break;
                case PuoliEsilla.Taka:
                    this.Image = takapuoliTekstuuri;
                    break;
            }
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(Time time)
        {
            if (liikkeenTila == LiikkeenTila.Pienentyy)
            {
                if (this.Width > 3)
                {
                    this.Size = new Vector(this.Width - 3, this.Height);
                }
                if (this.Width <= 3)
                {

                    if (puoliEsilla == PuoliEsilla.Kuva)
                    {
                        AsetaPuoli(PuoliEsilla.Taka);
                    }
                    else
                    {
                        AsetaPuoli(PuoliEsilla.Kuva);
                    }
                    liikkeenTila = LiikkeenTila.Suurentuu;
                }
            }

            if (liikkeenTila == LiikkeenTila.Suurentuu)
            {
                if (this.Width < this.alkuperainenKoko.X)
                {
                    this.Size = new Vector(this.Width + 3, this.Height);
                }
                if (this.Width >= this.alkuperainenKoko.X)
                {
                    liikkeenTila = LiikkeenTila.Pysahtynyt;
                }

            }

            base.Update(time);
        }
    }
}