﻿using System;
using System.IO;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.Assets;
using Jypeli.Effects;
using Jypeli.WP7;
/*
 * Muistipeli.
 * Author: Antti-Jussi Lakanen, Jyväskylän yliopisto, 2010.
 * Nuorten peliohjelmointi: https://trac.cc.jyu.fi/projects/npo
 */

namespace Muistipeli
{
    public struct Paikka
    {
        public int Rivi;
        public int Sarake;

        public Paikka(int rivi, int sarake)
        {
            Rivi = rivi;
            Sarake = sarake;
        }
    }

    public struct Pelaaja
    {
        public int Numero;
        public String Nimi;
        public int Pisteet;
        public Color Vari;

        public Pelaaja(String nimi, int numero, Color vari)
        {
            Numero = numero;
            Nimi = nimi;
            Pisteet = 0;
            Vari = vari;
        }
    }
    
    public class Muistipeli : Game
    {
        List<GameObject> valikonKohdat;

        int montakoRivia;
        int montakoSaraketta;

        double korttienVali = 20.0;
        double kortinLeveys = 50;
        double kortinKorkeus = 70;

        Kortti[,] kortit; // rivit, sarakkeet
        Paikka korostettuKortti;
        int montakoKorttia;
        int[] montakoKutakinKorttitunnusta;

        public enum Suunta { Vasen, Oikea, Ylos, Alas }
        Suunta mihinSuuntaanMenossa;

        GameObject highlighter;
        List<Kortti> kaannetyt; // Yhdellä vuorolla käännetyt kortit. Tyhjennetään vuoron päätteeksi. Listassa voi olla siis 0, 1 tai 2 alkiota.
        Timer piilotusAjastin;
        const double aikaaEnnenPiilottamista = 2;

        String korttisarja;

        Pelaaja pelaaja1;
#if !WINDOWS_PHONE
        Pelaaja pelaaja2;
        IntMeter pelaajan1Pisteet;
        IntMeter pelaajan2Pisteet;
        Pelaaja vuorossaOlevaPelaaja;
        Label vuoronaytto;
#else
        IntMeter kaannettyjaKortteja;
#endif
        IntMeter korttejaLoydetty;

        SoundEffect pariLoytyiAani = LoadSoundEffect("blim");
        SoundEffect kortinKaantoAani = LoadSoundEffect("blop");
        SoundEffect liikkumisAani = LoadSoundEffect("brr");
        SoundEffect vuoroVaihtuuAani = LoadSoundEffect("kolmisointu");

        Image elainKuva = LoadImage( "elaimet" );
        Image ihmisKuva = LoadImage( "ihmiset" );
        Image highlight1Kuva = LoadImage( "highlight1" );
        Image hightlight2Kuva = LoadImage( "highlight2" );
        Image takaTekstuuri = LoadImage( "taka" );
        Image jypeliLogo = LoadImage("jypeli");
        Image peliLogo = LoadImage("muistipeli2");
        Image vahanImage = LoadImage("4");
        Image sopivastiImage = LoadImage("2");
        Image paljonImage = LoadImage("1");
#if WINDOWS_PHONE
        List<Image> elainKuvat;
#endif

        public override void Begin()
        {
#if WINDOWS_PHONE
            Phone.DisplayOrientation = DisplayOrientation.LandscapeLeft;
            Phone.DisplayResolution = DisplayResolution.Large;
            NaytaJypeli();
            
            elainKuvat = new List<Image>();
            for (int i = 0; i < 24; i++)
            {
                elainKuvat.Add(LoadImage("elaimet/" + i.ToString()));
            }
            

            LisaaPelaajat(1);
#else
            LisaaPelaajat(2);
            Valikko();
#endif

        }

        public void NaytaJypeli()
        {
            Level.Width = 800;
            Level.Height = 480;
            Level.BackgroundColor = Color.White;
            ExplosionSystem logo = new ExplosionSystem(jypeliLogo, 1);
            logo.ParticleImage = jypeliLogo;
            //logo.Width = 800;
            //logo.Height = 480;
            //logo.FadeIn(2);
            //logo.FadeOut(2);
            logo.MaxAcceleration = 0;
            logo.MaxLifetime = 5;
            logo.MaxRotation = 0;
            logo.MaxRotationSpeed = 0;
            logo.MaxScale = 400;
            logo.MaxVelocity = 0;
            logo.MinAcceleration = 0;
            logo.MinLifetime = 5;
            logo.MinRotation = 0;
            logo.MinRotationSpeed = 0;
            logo.MinScale = 400;
            logo.MinVelocity = 0;
            Add(logo);
            logo.AddEffect(new Vector(0, 0), 1);
            Timer.SingleShot(5.5, Valikko);
            //jLogo = new GameObject(jypeliLogo.Width, jypeliLogo.Height);
            //jLogo.Image = elainKuva;
            //jLogo.Color = new Color(Color.Wheat, (byte)128);
            //Add(jLogo);
        }

        //protected override void Update(Time time)
        //{
        //    if (logoNakyvissa && jLogo != null)
        //    {
        //        jLogo.Image = Image.Color(jypeliLogo, (byte)logoAlpha);
        //        logoAlpha++;
        //    }
        //    base.Update(time);
        //}

        #region Valikko

        public void Valikko()
        {
            ClearAll();
#if WINDOWS_PHONE
            Level.Width = 800;
            Level.Height = 480;
            Level.Background.Image = peliLogo;
#else
            Level.Width = 2000;
            Level.Height = 1500;
#endif
            valikonKohdat = new List<GameObject>();
            Level.BackgroundColor = Color.White;
#if !WINDOWS_PHONE
            GameObject elaimet = new GameObject(538, 256);
            elaimet.Image = elainKuva;
            elaimet.Tag = "elaimet";
            elaimet.Y = elaimet.Height / 2 + 10;
            valikonKohdat.Add(elaimet);

            GameObject ihmiset = new GameObject(538, 256);
            ihmiset.Image = ihmisKuva;
            ihmiset.Tag = "ihmiset";
            ihmiset.Y = -ihmiset.Height / 2 - 10;
            valikonKohdat.Add(ihmiset);
            foreach (var valikonKohta in valikonKohdat)
            {
                Add(valikonKohta);
                Mouse.ListenOn(valikonKohta, MouseButton.Left, ButtonState.Pressed, KorttisarjaValittu, null, valikonKohta);
            }
#else
            TouchPanel.Listen(ButtonState.Pressed, delegate { KorttisarjaValittu(null); }, null);
#endif

            ValikonOhjaimet();

            Camera.ZoomToLevel();
        }

        public void KorttisarjaValittu(GameObject mitaPainettiin)
        {
#if WINDOWS_PHONE
            korttisarja = "elaimet";
#else
            korttisarja = mitaPainettiin.Tag.ToString();
#endif
            PelinKorttienMaara();
        }

        public void ValikonOhjaimet()
        {
            
#if WINDOWS_PHONE
            //TouchPanel.Listen(ButtonState.Down, ValikossaLiikkuminen, null);
            PhoneBackButton.Listen(Lopeta, "");
#else
            Mouse.IsCursorVisible = true;
            Mouse.ListenMovement(1.0, ValikossaLiikkuminen, null);
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, Lopeta, "");
#endif
        }
#if WINDOWS_PHONE
        public void ValikossaLiikkuminen(Touch kosketus)
#else
        public void ValikossaLiikkuminen(AnalogState hiirenTila)
#endif
        {
            foreach (var kohta in valikonKohdat)
            {
#if WINDOWS_PHONE
                if (kosketus.PositionOnWorld.X < kohta.Right && kosketus.PositionOnWorld.X > kohta.Left
                    && kosketus.PositionOnWorld.Y < kohta.Top && kosketus.PositionOnWorld.Y > kohta.Bottom)
#else
                if (Mouse.IsCursorOn(kohta))
#endif
                {
                    kohta.Image = LoadImage(kohta.Tag.ToString() + "_");
                }
                else
                {
                    kohta.Image = LoadImage(kohta.Tag.ToString());
                }
            }
        }

        public void PelinKorttienMaara()
        {
            int montakoKuvaaSarjassa = LataaKuvat();

            ClearAll();
            valikonKohdat.Clear();
            Level.Width = 800;
            Level.Height = 480;
            Level.BackgroundColor = Color.White;

            Label montako = new Label("Kuinka monta korttia?");
            //montako.Font = Font.DefaultSmall;
            montako.Position = new Vector(0, 0);
            Add(montako);
#if WINDOWS_PHONE
            montako.Position = new Vector(0, Screen.Top - 20);
            GameObject vahan = new GameObject(468, 96);
            vahan.Image = vahanImage;
            vahan.Tag = "4";
            valikonKohdat.Add(vahan);

            GameObject sopivasti = new GameObject(468, 96);
            sopivasti.Image = sopivastiImage;
            sopivasti.Tag = "2";
            valikonKohdat.Add(sopivasti);

            GameObject paljon = new GameObject(468, 96);
            paljon.Image = paljonImage;
            paljon.Tag = "1";
            valikonKohdat.Add(paljon);
#else

            GameObject vahan = new GameObject(512, 128);
            vahan.Tag = "4";
            valikonKohdat.Add(vahan);

            GameObject sopivasti = new GameObject(512, 128);
            sopivasti.Tag = "2";
            valikonKohdat.Add(sopivasti);

            GameObject paljon = new GameObject(512, 128);
            paljon.Tag = "1";
            valikonKohdat.Add(paljon);
#endif
            int i = 0;
            foreach (var valikonKohta in valikonKohdat)
            {
                Add(valikonKohta);
                valikonKohta.Position = new Vector(0, valikonKohdat.Count * valikonKohta.Height / 2 - ((valikonKohta.Height + 20) * i) - 10);

#if !WINDOWS_PHONE
                Mouse.ListenOn(valikonKohta, MouseButton.Left, ButtonState.Pressed, MaaraValittu, null, valikonKohta, montakoKuvaaSarjassa);
#endif
                i++;
            }
#if WINDOWS_PHONE
                //TouchPanel.ListenOn(valikonKohta, ButtonState.Pressed, delegate { MaaraValittu(valikonKohta, montakoKuvaaSarjassa); }, null);
                TouchPanel.Listen(ButtonState.Pressed, MaaraValittu, null, montakoKuvaaSarjassa);
#endif
            ValikonOhjaimet();
            Camera.ZoomToLevel();
        }

        public void MaaraValittu(Touch kosketus, int montakoKuvaaSarjassa)
        {
            foreach (var valikonKohta in valikonKohdat)
            {
                if (kosketus.PositionOnWorld.X < valikonKohta.Right && kosketus.PositionOnWorld.X > valikonKohta.Left
                    && kosketus.PositionOnWorld.Y < valikonKohta.Top && kosketus.PositionOnWorld.Y > valikonKohta.Bottom)
                {
                    MaaraValittu(valikonKohta, montakoKuvaaSarjassa);
                }
            }
        }

        public void MaaraValittu(GameObject mitaPainettiin, int montakoKuvaaSarjassa)
        {
            // jos painettiin "vähän" niin kortteja tulee 1/4 maksimista
            // esim. jos kortteja on 13, niin pareja tulee floor(1/4 * 13) = 3 ja siten 6 korttia
            montakoKorttia = 2 * (int)Math.Floor((1.0/(int.Parse(mitaPainettiin.Tag.ToString())) * montakoKuvaaSarjassa));
            AloitaUusiPeli();
        }


        #endregion
        public void AloitaUusiPeli()
        {
            ClearAll();
            Level.BackgroundColor = Color.LightCyan;
            LisaaPiilotusajastin();
            //montakoKorttia = montakoKuvaa * 2;

            // Laitetaan ensin rivejä ja sarakkeita yhtä paljon
            montakoSaraketta = (int)Math.Sqrt(montakoKorttia);
            // Kortit vähän mukavempaan muotoon, koska näytöt on yleensä leveämpiä kuin korkeita
            montakoSaraketta = (int)Math.Floor((3.0 / 2.0) * montakoSaraketta);

            // Tarvittavien rivien määrä saadaan korttien määrän ja saadun sarakkeiden määrän suhteesta
            montakoRivia = (int)Math.Ceiling((double)montakoKorttia / (double)montakoSaraketta);
#if WINDOWS_PHONE
            if (montakoRivia * (korttienVali + kortinKorkeus) > Screen.Height)
            {
                montakoSaraketta = (int)Math.Floor(Screen.Width / (kortinLeveys + korttienVali));
                montakoRivia = (int)Math.Ceiling((double)montakoKorttia / (double)montakoSaraketta);
            }
#endif
            
            
            kortit = new Kortti[montakoRivia, montakoSaraketta];

            // kortin tunnusNro indeksistä, alkiot tulkitaan siten että "montako tämän tunnuksen alkiota on lisätty peliin"
            // Tämä pitää olla ennen kuin kortit lisätään ruudulle
            montakoKutakinKorttitunnusta = new int[montakoKorttia / 2];

            KortitRuudulle();

#if WINDOWS_PHONE
            LisaaLaskuri();
#else
            LisaaLaskurit();
            LisaaVuoronaytto();
#endif
            LisaaHighLighter();

            // Laitetaan kenttä sopivan kokoiseksi että kortit täyttävät aina mahollisimman paljon ruutukapasiteetista
            Level.Width = montakoSaraketta * (korttienVali + kortinLeveys);// -korttienVali;
            Level.Height = montakoRivia * (korttienVali + kortinKorkeus);// -korttienVali * 2;
            Camera.ZoomToLevel();

            PeliKayntiin();
            AsetaNappaimet();
        }

        public void LisaaPiilotusajastin()
        {
            piilotusAjastin = new Timer();
            piilotusAjastin.Interval = aikaaEnnenPiilottamista;
            piilotusAjastin.Timeout += PiilotaKaannetyt;
        }

#if !WINDOWS_PHONE
        public void LisaaLaskurit()
        {
            pelaajan1Pisteet = new IntMeter(0);
            Label pisteNaytto1 = new Label(200, 30);
            pisteNaytto1.Title = "Pelaaja 1 pisteet";
            pisteNaytto1.HorizontalAlignment = HorizontalAlignment.Left;
            Label p1Voitot = new Label(200, 30, pelaaja1.Pisteet.ToString());
            p1Voitot.Title = "Pelaaja 1 voitot";
            p1Voitot.HorizontalAlignment = HorizontalAlignment.Left;

            pisteNaytto1.X = Screen.Left + 140;
            pisteNaytto1.Y = Screen.Bottom + 70;
            pisteNaytto1.TextColor = Color.Blue;
            pisteNaytto1.BindTo(pelaajan1Pisteet);
            p1Voitot.X = Screen.Left + 140;
            p1Voitot.Y = Screen.Bottom + 40;

            Add(pisteNaytto1);
            Add(p1Voitot);

            pelaajan2Pisteet = new IntMeter(0);
            Label pisteNaytto2 = new Label(200, 30);
            pisteNaytto2.Title = "Pelaaja 2 pisteet";
            pisteNaytto2.HorizontalAlignment = HorizontalAlignment.Right;
            Label p2Voitot = new Label(200, 30, pelaaja2.Pisteet.ToString());
            p2Voitot.Title = "Pelaaja 2 voitot";
            p2Voitot.HorizontalAlignment = HorizontalAlignment.Right;

            pisteNaytto2.X = Screen.Right - 140;
            pisteNaytto2.Y = Screen.Bottom + 70;
            pisteNaytto2.TextColor = Color.Red;
            pisteNaytto2.BindTo(pelaajan2Pisteet);
            p2Voitot.X = Screen.Right - 140;
            p2Voitot.Y = Screen.Bottom + 40;

            Add(pisteNaytto2);
            Add(p2Voitot);

            korttejaLoydetty = new IntMeter(0);
            korttejaLoydetty.MaxValue = montakoKorttia;
            korttejaLoydetty.UpperLimit += KaikkiLoydetty;
        }
#else
        public void LisaaLaskuri()
        {
            kaannettyjaKortteja = new IntMeter(0);
            Label kaannettyjaNaytto2 = new Label(200, 30);
            kaannettyjaNaytto2.Text = "Käännettyjä pareja:";
            kaannettyjaNaytto2.HorizontalAlignment = HorizontalAlignment.Left;
            kaannettyjaNaytto2.X = -100;
            kaannettyjaNaytto2.Y = Screen.Bottom + 20;


            Label kaannettyjaNaytto = new Label(200, 30);
            kaannettyjaNaytto.HorizontalAlignment = HorizontalAlignment.Left;
            kaannettyjaNaytto.X = 100;
            kaannettyjaNaytto.Y = Screen.Bottom + 20;
            kaannettyjaNaytto.BindTo(kaannettyjaKortteja);

            //Add(kaannettyjaNaytto);
            //Add(kaannettyjaNaytto2);

            korttejaLoydetty = new IntMeter(0);
            korttejaLoydetty.MaxValue = montakoKorttia;
            korttejaLoydetty.UpperLimit += KaikkiLoydetty;
        }
#endif
        /// <summary>
        /// Korostetaan ensimmäinen kortti.
        /// </summary>
        public void PeliKayntiin()
        {
            kaannetyt = new List<Kortti>();
            korostettuKortti = new Paikka(0, 0);
            KorostaKortti(new Paikka(0, 0));
            
#if !WINDOWS_PHONE
            MessageDisplay.Font = Font.DefaultSmall;

            vuorossaOlevaPelaaja = pelaaja1;
            vuoronaytto.Text = "Pelaaja 1";
            MessageDisplay.Add("Pelaaja 1 aloittaa pelin!");
#endif
        }


        public void KaikkiLoydetty(int arvo)
        {
            Mouse.IsCursorVisible = true;
            String voittoteksti;
#if WINDOWS_PHONE
            voittoteksti = "Löysit kaikki parit! Onneksi olkoon!";
#else
            if (pelaajan1Pisteet > pelaajan2Pisteet)
            {
                voittoteksti = "Pelaaja 1 voitti! ";
                pelaaja1.Pisteet++;
            }
            else if (pelaajan2Pisteet > pelaajan1Pisteet)
            {
                voittoteksti = "Pelaaja 2 voitti! ";
                pelaaja2.Pisteet++;
            }
            else
            {
                voittoteksti = "Tasapeli! ";
            }
#endif
            MessageWindow mb = new MessageWindow("Peli päättyi\n" + voittoteksti);

            //MessageWindow mb = new MessageWindow(
            //    "Peli päättyi",
            //    "" + voittoteksti + "Aloita uusi peli painamalla OK.");
#if !WINDOWS_PHONE
            //mb.OKButton.Clicked += OKPainettu;
            mb.Closed += delegate { AloitaUusiPeli(); };
#else
            mb.Closed += delegate { AloitaUusiPeli(); };
#endif
            Add(mb);
        }

        public void OKPainettu()
        {
            AloitaUusiPeli();
        }
#if !WINDOWS_PHONE
        public void LisaaVuoronaytto()
        {
            Label vuorossaNyt = new Label("Vuorossa nyt");
            vuorossaNyt.X = 0;
            vuorossaNyt.Y = Screen.Bottom + 70;
            vuoronaytto = new Label();
            vuoronaytto.X = 0;
            vuoronaytto.Y = vuorossaNyt.Y - 30;
//#if WINDOWS_PHONE
            vuorossaNyt.X = -40;
            vuorossaNyt.Y = Screen.Bottom + 20;
            vuoronaytto.X = 60;
            vuoronaytto.Y = vuorossaNyt.Y;
            vuoronaytto.Font = Font.DefaultSmall;
            vuorossaNyt.Font = Font.DefaultSmall;
//#endif
            Add(vuorossaNyt);
            Add(vuoronaytto);
        }
#endif
        void LisaaHighLighter()
        {
            highlighter = new GameObject(kortinLeveys, kortinKorkeus);
            highlighter.Image = highlight1Kuva;
            Add(highlighter, 1);
        }

        private int LataaKuvat()
        {
            
#if WINDOWS_PHONE
            return elainKuvat.Count;
#else
            List<String> kuvat = new List<string>();
            String[] kuvatiedostot = Directory.GetFiles(korttisarja);
            return kuvatiedostot.Length;
#endif
        }

        void LisaaPelaajat(int montakoPelaajaa)
        {
            // Toistaiseksi tehdään "manuaalisesti" kaksi pelaajaa
            pelaaja1 = new Pelaaja("", 1, Color.Cyan);
#if !WINDOWS_PHONE
            pelaaja2 = new Pelaaja("", 2, Color.MediumPurple);
#endif
        }

        void AsetaNappaimet()
        {
            Keyboard.Listen(Key.Left, ButtonState.Pressed, KortinValinta, null, Suunta.Vasen);
            Keyboard.Listen(Key.Right, ButtonState.Pressed, KortinValinta, null, Suunta.Oikea);
            Keyboard.Listen(Key.Up, ButtonState.Pressed, KortinValinta, null, Suunta.Ylos);
            Keyboard.Listen(Key.Down, ButtonState.Pressed, KortinValinta, null, Suunta.Alas);
            Keyboard.Listen(Key.Space, ButtonState.Pressed, KaannaKortti, null);
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, Valikko, null);
#if WINDOWS_PHONE
            TouchPanel.Listen(ButtonState.Pressed, KortinValinta, null);
            PhoneBackButton.Listen(PelinKorttienMaara, null);
#endif
        }

        void Lopeta()
        {
            Exit();
        }

        #region Liikkuminen

        void KorostaKortti(Paikka korostettavaKortti)
        {
#if !WINDOWS_PHONE
            korostettavaKortti = TarkistaKortinNro(korostettavaKortti);
#endif
            highlighter.Position = new Vector(KortinX(korostettavaKortti.Sarake), KortinY(korostettavaKortti.Rivi));
            Kortti.PuoliEsilla puoliesilla = kortit[korostettavaKortti.Rivi, korostettavaKortti.Sarake].KumpiPuoliEsilla();
            korostettuKortti = korostettavaKortti;
            liikkumisAani.Play();
        }

        /// <summary>
        /// Aliohjelma pitää huolen, että valittu kortti on aina
        /// rivi- ja sarakeindeksien sisäpuolella. Toisin sanoen 
        /// rivistä/sarakkeesta "yli" hypättäessä kursori palautuu 
        /// oikeaan paikkaan.
        /// </summary>
        /// <param name="kortinNro">Kortin rivi, sarake</param>
        /// <returns>"Korjatut" koordinaatit</returns>
        private Paikka TarkistaKortinNro(Paikka paikka)
        {
            // Viimeisen rivin jälkeen palataan ensimmäiselle riville
            if (paikka.Rivi >= montakoRivia)
                paikka.Rivi = 0;

            // Viimeisen sarakkeen jälkeen palataan ensimmäiseen sarakkeeseen
            if (paikka.Sarake >= montakoSaraketta)
                paikka.Sarake = 0;

            // Ensimmäisestä rivistä "taaksepäin" liikuttaessa
            // hypätään viimeiselle riville
            if (paikka.Rivi < 0)
                paikka.Rivi = montakoRivia - 1;

            // Ensimmäisestä sarakkeesta "taaksepäin" liikuttaessa
            // hypätään viimeiseen sarakkeeseen
            if (paikka.Sarake < 0)
                paikka.Sarake = montakoSaraketta - 1;

            // Mentiin pelipakan "tyhjälle alueelle"
            if (kortit[paikka.Rivi, paikka.Sarake] == null)
                paikka = TultiinNullViitteeseen(paikka);

            return paikka;
        }

        private Paikka TultiinNullViitteeseen(Paikka kortinPaikka)
        {
            if (mihinSuuntaanMenossa == Suunta.Vasen)
                return new Paikka(kortinPaikka.Rivi, (montakoKorttia % montakoSaraketta) - 1);
            if (mihinSuuntaanMenossa == Suunta.Oikea)
                return new Paikka(kortinPaikka.Rivi, 0);
            if (mihinSuuntaanMenossa == Suunta.Ylos)
                return new Paikka(montakoRivia - 2, kortinPaikka.Sarake);

            // alas
            return new Paikka(0, kortinPaikka.Sarake);
        }

        void KortinValinta(Touch kosketus)
        {
            //double korttienVali = 20.0;
            //double kortinLeveys = 50;
            //double kortinKorkeus = 70;
            for (int x = 0; x < kortit.GetLength(0); x++)
			    for (int y = 0; y < kortit.GetLength(1); y++)
			    {
                    if (kortit[x, y] == null)
                        continue;
                    if (kosketus.PositionOnWorld.X < kortit[x, y].Right && kosketus.PositionOnWorld.X > kortit[x, y].Left
                    && kosketus.PositionOnWorld.Y < kortit[x, y].Top && kosketus.PositionOnWorld.Y > kortit[x, y].Bottom)
                    {
                        //if (korostettuKortti.Rivi == kortit[x, y].Paikka.Rivi && korostettuKortti.Sarake == kortit[x, y].Paikka.Sarake)
                            
                        //else
                            KorostaKortti(kortit[x, y].Paikka);
                            KaannaKortti();
                    }
			    }
                
	        
            
        }

        void KortinValinta(Suunta suunta)
        {
            mihinSuuntaanMenossa = suunta;
            if (mihinSuuntaanMenossa == Suunta.Vasen)
                KorostaKortti(new Paikka(korostettuKortti.Rivi, korostettuKortti.Sarake - 1));

            if (mihinSuuntaanMenossa == Suunta.Oikea)
                KorostaKortti(new Paikka(korostettuKortti.Rivi, korostettuKortti.Sarake + 1));

            if (mihinSuuntaanMenossa == Suunta.Ylos)
                KorostaKortti(new Paikka(korostettuKortti.Rivi - 1, korostettuKortti.Sarake));

            if (mihinSuuntaanMenossa == Suunta.Alas)
                KorostaKortti(new Paikka(korostettuKortti.Rivi + 1, korostettuKortti.Sarake));
        }
        #endregion

        #region Kääntäminen ja parin löytyminen

        public bool VoikoKorttiaKaantaa(Kortti kortti)
        {
            bool onkoKaantymassa = kortti.OnkoLiikkeessa();

            // Jos kortti "on jo löydetty", niin sitä ei voi missään tapauksessa kääntää
            bool onkoPoistettuPelista = kortit[korostettuKortti.Rivi, korostettuKortti.Sarake].PoistettuPelista;

            // Ei käännetä kahta kertaa samaa korttia
            if (kaannetyt.Count == 0 || kortit[korostettuKortti.Rivi, korostettuKortti.Sarake] != kaannetyt[0])
            {
                if (!onkoPoistettuPelista && !onkoKaantymassa)
                {
                    return true;
                }
            }
            return false;
        }

        void KaannaKortti()
        {
            if (VoikoKorttiaKaantaa(kortit[korostettuKortti.Rivi, korostettuKortti.Sarake]))
            {
                kortit[korostettuKortti.Rivi, korostettuKortti.Sarake].Kaanna();
                kaannetyt.Add(kortit[korostettuKortti.Rivi, korostettuKortti.Sarake]);
                kortinKaantoAani.Play();

                if (kaannetyt.Count == 2) // Tasan kaksi korttia käännetään yhdellä vuorolla
                {
#if WINDOWS_PHONE
                    kaannettyjaKortteja.Value++;
#endif
                    if (kaannetyt[0].Tunniste == kaannetyt[1].Tunniste) // Käännetyt kortit muodostavat parin
                    {
                        foreach (Kortti kaannetty in kaannetyt)
                        {
                            RajaytaKortti(kaannetty);
                            kaannetty.PoistettuPelista = true;
                        }
                        PariLoytyi();
                    }
                    else
                    {
                        // Jos kaksi korttia on käännetty niin ajastin lähtee käyntiin eikä sinä aikana voi kääntää kortteja
                        Keyboard.Disable(Key.Space);
                        //TouchPanel.Clear();
                        piilotusAjastin.Start(1);
                        TouchPanel.Disable(ButtonState.Pressed);
                        //TouchPanel.Listen(ButtonState.Pressed, delegate { PiilotaKaannetyt(); }, null);
                    }
                }
            }
        }

        void PiilotaKaannetyt()
        {
            //piilotusAjastin.Stop();
            foreach (var item in kaannetyt)
            {
                item.Kaanna();
            }
            kaannetyt.Clear();
            Keyboard.Enable(Key.Space);
            TouchPanel.Enable(ButtonState.Pressed);
            //TouchPanel.Clear();
            //TouchPanel.Listen(ButtonState.Pressed, KortinValinta, null);
#if !WINDOWS_PHONE
            VuoroVaihtuu();
#endif
        }

        public void PariLoytyi()
        {
            pariLoytyiAani.Play();
            kaannetyt.Clear();
#if !WINDOWS_PHONE
            if (vuorossaOlevaPelaaja.Equals(pelaaja1))
            {
                pelaajan1Pisteet.Value++;
            }
            else if (vuorossaOlevaPelaaja.Equals(pelaaja2))
            {
                pelaajan2Pisteet.Value++;
            }
#endif
            korttejaLoydetty.Value += 2;
        }

        /// <summary>
        /// Näpsäkkä korttiin kohdistettu "räjähdys" tai mikä efekti lie.
        /// </summary>
        /// <param name="kaannetty">Kortti jonka päälle räjähdys tulee.</param>
        public void RajaytaKortti(Kortti kortti)
        {
            Image i = kortti.KuvaPuoliTekstuuri;
            ExplosionSystem e = new ExplosionSystem(i, 10);
            e.MinLifetime = 0.3;
            e.MaxLifetime = 1.5;
            e.MinVelocity = 1;
            e.MaxVelocity = 300;
            e.MinScale = 6;
            e.MaxScale = 30;
            Add(e);
            e.AddEffect(kortti.Position, 10);
        }
#if !WINDOWS_PHONE
        public void VuoroVaihtuu()
        {
            vuoroVaihtuuAani.Play();

            if (vuorossaOlevaPelaaja.Equals(pelaaja1))
            {
                vuorossaOlevaPelaaja = pelaaja2;
                highlighter.Image = hightlight2Kuva;
                vuoronaytto.Text = "Pelaaja 2";
                VuoronVaihtumisEfekti("pelaaja2");
                return;
            }

            if (vuorossaOlevaPelaaja.Equals(pelaaja2))
            {
                vuorossaOlevaPelaaja = pelaaja1;
                highlighter.Image = highlight1Kuva;
                vuoronaytto.Text = "Pelaaja 1";
                VuoronVaihtumisEfekti("pelaaja1");
                return;
            }
        }
#endif
        public void VuoronVaihtumisEfekti(String kuvanNimi)
        {
            ExplosionSystem e = new ExplosionSystem(LoadImage(kuvanNimi), 1);
            Add(e);
            e.MinAcceleration = 0.0;
            e.MaxAcceleration = 0.0;
            e.MinLifetime = 1;
            e.MaxLifetime = 1;
            e.MinVelocity = 0.0;
            e.MaxVelocity = 0.0;
            e.MinRotationSpeed = 0.0;
            e.MaxRotationSpeed = 0.0;
            e.MinRotation = 0.0;
            e.MaxRotation = 0.0;
            e.AddEffect(0, 0, 1);
        }

        #endregion

        #region Kortit ruudulle

        /// <summary>
        /// Aliohjelma lisää halutun määrän kortteja
        /// taulukkoon ja edelleen ruudulle.
        /// </summary>
        /// <param name="montakoKorttiaLisataan">Kuinka monta korttia taulukkoon (attribuutti) lisätään.</param>
        void KortitRuudulle()
        {
            int montakoKorttiaLisatty = 0;

            for (int r = 0; r < kortit.GetLength(0); r++) // Rivit
            {
                for (int s = 0; s < kortit.GetLength(1); s++) // Sarakkeet
                {
                    // Jos tarvittava määrä kortteja on lisätty, 
                    // laitetaan loppuihin taulukon alkioihin null-viite
                    if (montakoKorttia == montakoKorttiaLisatty)
                    {
                        kortit[r, s] = null;
                        continue;
                    }

                    kortit[r, s] = TeeKortti();
                    kortit[r, s].Paikka = new Paikka(r, s);
#if WINDOWS_PHONE
                    kortit[r,s].KuvaPuoliTekstuuri = elainKuvat[kortit[r,s].Tunniste];
#else
                    kortit[r, s].KuvaPuoliTekstuuri = Image.FromFile(korttisarja + "/" + kortit[r, s].Tunniste.ToString() + ".jpg");
#endif
                    Add(kortit[r, s]);

                    kortit[r, s].X = KortinX(s);
                    kortit[r, s].Y = KortinY(r);
                    montakoKorttiaLisatty++;
                }
            }
        }

        public double KortinX(int sarake)
        {
            return (-(montakoSaraketta * (kortinLeveys + korttienVali)) / 2) + (sarake * (kortinLeveys + korttienVali)) + korttienVali*2;
        }

        public double KortinY(int rivi)
        {
            // Viimeinen termi tulee siitä että viimeisen kortin perässä ei tarvita välistystä
            return ((montakoRivia) * (kortinKorkeus + korttienVali)) / 2 - rivi * (kortinKorkeus + korttienVali) - korttienVali*2;
        }

        public Kortti TeeKortti()
        {
            int tunniste = AnnaTunniste();
            return new Kortti(kortinLeveys, kortinKorkeus, false, takaTekstuuri, tunniste);
        }

        private int AnnaTunniste()
        {
            int tunniste;
            while (true)
            {
                tunniste = RandomGen.NextInt(0, montakoKutakinKorttitunnusta.Length);
                if (montakoKutakinKorttitunnusta[tunniste] < 2)
                {
                    montakoKutakinKorttitunnusta[tunniste]++;
                    break;
                }
            }
            return tunniste;
        }
        #endregion
    }
}