﻿using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.WP7;

/// <summary>
/// Virtuaalinen piano.
/// 
/// Tekijät: Tero Jäntti ja Antti-Jussi Lakanen.
/// Pianon äänet: Ville-Matti Pasanen.
/// 
/// TODO: Kirjastoon tarvitaan hiiren kuuntelu siten,
/// että mustat koskettimet tulevat ylemmäksi kuin valkoiset
/// ja että niiden alapuolella olevia koskettimia ei paineta.
/// Tai sitten tehdään valkoisille sellainen muoto, että ei
/// mene päällekäin.
/// </summary>
public class Peli : Game
{

    class Kosketin : GameObject
    {
        public Color AlkuperainenVari;

        /// <summary>
        /// Edeltävä C-nuotti.
        /// </summary>
        public SoundEffect PerusAani;

        /// <summary>
        /// Äänenkorkeus suhteessa edelliseen C-nuottiin.
        /// </summary>
        public double AanenKorkeus;

        public Kosketin(double leveys, double korkeus)
            : base(leveys, korkeus, Shape.Rectangle)
        {
        }
    }


#if WINDOWS_PHONE
    const int Oktaaveja = 1;
#else
    const int Oktaaveja = 3;
#endif

    const double ValkoisenLeveys = 20.0;
    const double ValkoisenKorkeus = 100.0;

    const double MustanLeveys = 15.0;
    const double MustanKorkeus = 60;

    const double ValkoistenVali = 1.0;


    public override void Begin()
    {
        Phone.DisplayOrientation = DisplayOrientation.Landscape;

        Level.Width = Oktaaveja * (7 * ValkoisenLeveys + 6 * ValkoistenVali);
        Level.Height = ValkoisenKorkeus;
        Camera.ZoomToLevel();
        Level.BackgroundColor = Color.Black;

        Mouse.IsCursorVisible = true;

        LisaaOktaavi1();
        if (Oktaaveja >= 2)
            LisaaOktaavi2();
        if (Oktaaveja >= 3)
            LisaaOktaavi3();

        Keyboard.Listen( Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet" );
        Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, "Poistu pelistä" );
    }

    void LisaaOktaavi1()
    {
        SoundEffect aani = LoadSoundEffect( "piano2" );

        Kosketin c = LuoValkoinenKosketin( 0, 0.0 / 12, aani, Key.Z );
        Kosketin d = LuoValkoinenKosketin( 1, 2.0 / 12, aani, Key.X );
        Kosketin e = LuoValkoinenKosketin( 2, 4.0 / 12, aani, Key.C );
        Kosketin f = LuoValkoinenKosketin( 3, 5.0 / 12, aani, Key.V );
        Kosketin g = LuoValkoinenKosketin( 4, 7.0 / 12, aani, Key.B );
        Kosketin a = LuoValkoinenKosketin( 5, 9.0 / 12, aani, Key.N );
        Kosketin b = LuoValkoinenKosketin( 6, 11.0 / 12, aani, Key.M );

        Kosketin cs = LuoMustaKosketin( 0, 1.0 / 12.0, aani, Key.S );
        Kosketin ds = LuoMustaKosketin( 1, 3.0 / 12.0, aani, Key.D );
        Kosketin fs = LuoMustaKosketin( 3, 6.0 / 12.0, aani, Key.G );
        Kosketin gs = LuoMustaKosketin( 4, 8.0 / 12.0, aani, Key.H );
        Kosketin ash = LuoMustaKosketin( 5, 10.0 / 12.0, aani, Key.J );
    }

    void LisaaOktaavi2()
    {
        SoundEffect aani = LoadSoundEffect( "piano3" );

        Kosketin c = LuoValkoinenKosketin( 7, 0.0 / 12, aani, Key.Q );
        Kosketin d = LuoValkoinenKosketin( 8, 2.0 / 12, aani, Key.W );
        Kosketin e = LuoValkoinenKosketin( 9, 4.0 / 12, aani, Key.E );
        Kosketin f = LuoValkoinenKosketin( 10, 5.0 / 12, aani, Key.R );
        Kosketin g = LuoValkoinenKosketin( 11, 7.0 / 12, aani, Key.T );
        Kosketin a = LuoValkoinenKosketin( 12, 9.0 / 12, aani, Key.Y );
        Kosketin b = LuoValkoinenKosketin( 13, 11.0 / 12, aani, Key.U );

        Kosketin cs = LuoMustaKosketin( 7, 1.0 / 12.0, aani, Key.D2 );
        Kosketin ds = LuoMustaKosketin( 8, 3.0 / 12.0, aani, Key.D3 );
        Kosketin fs = LuoMustaKosketin( 10, 6.0 / 12.0, aani, Key.D5 );
        Kosketin gs = LuoMustaKosketin( 11, 8.0 / 12.0, aani, Key.D6 );
        Kosketin ash = LuoMustaKosketin( 12, 10.0 / 12.0, aani, Key.D7 );
    }

    void LisaaOktaavi3()
    {
        SoundEffect aani = LoadSoundEffect( "piano4" );

        Kosketin c = LuoValkoinenKosketin( 14, 0.0 / 12, aani, Key.F12 );
        Kosketin d = LuoValkoinenKosketin( 15, 2.0 / 12, aani, Key.F12 );
        Kosketin e = LuoValkoinenKosketin( 16, 4.0 / 12, aani, Key.F12 );
        Kosketin f = LuoValkoinenKosketin( 17, 5.0 / 12, aani, Key.F12 );
        Kosketin g = LuoValkoinenKosketin( 18, 7.0 / 12, aani, Key.F12 );
        Kosketin a = LuoValkoinenKosketin( 19, 9.0 / 12, aani, Key.F12 );
        Kosketin b = LuoValkoinenKosketin( 20, 11.0 / 12, aani, Key.F12 );

        Kosketin cs = LuoMustaKosketin( 14, 1.0 / 12.0, aani, Key.F12 );
        Kosketin ds = LuoMustaKosketin( 15, 3.0 / 12.0, aani, Key.F12 );
        Kosketin fs = LuoMustaKosketin( 17, 6.0 / 12.0, aani, Key.F12 );
        Kosketin gs = LuoMustaKosketin( 18, 8.0 / 12.0, aani, Key.F12 );
        Kosketin ash = LuoMustaKosketin( 19, 10.0 / 12.0, aani, Key.F12 );
    }

    Kosketin LuoValkoinenKosketin( int numero, double aanenKorkeus, SoundEffect perusAani, Key nappula )
    {
        Kosketin kosketin = new Kosketin( ValkoisenLeveys, ValkoisenKorkeus );
        kosketin.AlkuperainenVari = Color.White;
        kosketin.Color = Color.White;
        kosketin.PerusAani = perusAani;
        kosketin.AanenKorkeus = aanenKorkeus;
        kosketin.X = Level.Left + ValkoisenLeveys / 2 + numero * ( ValkoisenLeveys + ValkoistenVali );
        Add( kosketin );

        LisaaKuuntelijat(kosketin, nappula);

        return kosketin;
    }

    Kosketin LuoMustaKosketin( int numero, double aanenKorkeus, SoundEffect perusAani, Key nappula )
    {
        Kosketin kosketin = new Kosketin( MustanLeveys, MustanKorkeus );
        kosketin.AlkuperainenVari = Color.Black;
        kosketin.Color = Color.Black;
        kosketin.PerusAani = perusAani;
        kosketin.AanenKorkeus = aanenKorkeus;
        kosketin.X = Level.Left + ValkoisenLeveys + ValkoistenVali / 2 + numero * ( ValkoisenLeveys + ValkoistenVali );
        kosketin.Y = Level.Center.Y + ValkoisenKorkeus / 2 - MustanKorkeus / 2;
        Add( kosketin );

        LisaaKuuntelijat(kosketin, nappula);

        return kosketin;
    }

    void LisaaKuuntelijat(Kosketin kosketin, Key nappula)
    {
        Keyboard.Listen(nappula, ButtonState.Pressed, SoitaNuotti, null, kosketin);
        Mouse.ListenOn(kosketin, MouseButton.Left, ButtonState.Pressed, SoitaNuottiHiirella, null, kosketin);
        TouchPanel.ListenOn(kosketin, ButtonState.Pressed, SoitaNuottiKosketuksella, null, kosketin);
    }

    void SoitaNuottiKosketuksella(Touch kosketus, Kosketin kosketin)
    {
        if (kosketin.AlkuperainenVari == Color.White && OnMustanKorkeudella(kosketus.PositionOnWorld))
            return;
        
        SoitaNuotti(kosketin);
    }

    void SoitaNuottiHiirella(Kosketin kosketin)
    {
        if (kosketin.AlkuperainenVari == Color.White && OnMustanKorkeudella(Mouse.PositionOnWorld))
            return;

        SoitaNuotti(kosketin);
    }

    // Tällä kierretään se, että hiiren kuuntelu Jypelissä voi ottaa
    // päällekäisiä olioita yhtäaikaa.
    bool OnMustanKorkeudella(Vector paikka)
    {
        return paikka.Y > (ValkoisenKorkeus / 2 - MustanKorkeus);
    }

    void SoitaNuotti( Kosketin kosketin )
    {
        kosketin.Color = Color.Gray;

        Sound aani = kosketin.PerusAani.CreateSound();
        aani.Pitch = kosketin.AanenKorkeus;
        aani.Play();

        Timer.SingleShot(0.3, delegate() { VapautaNuotti(kosketin); });
    }

    void VapautaNuotti( Kosketin kosketin )
    {
        kosketin.Color = kosketin.AlkuperainenVari;
    }
}
