﻿using System;
using Jypeli;
using Jypeli.Assets;

class Elephant : PlatformCharacter
{
    static Image[] bodyImagesRight;
    static Image[] bodyImagesLeft;

    static Image[] headImagesRight;
    static Image[] headImagesLeft;
    static Image[] closeEyesImagesRight;
    static Image[] closeEyesImagesLeft;
    static Image[] attackRight;
    static Image[] attackLeft;

    GameObject head;
    Animation lookingAnimation;
    Animation lookingAnimationLeft;
    Animation closeEyesAnimation;
    Animation closeEyesLeft;
    Animation attackAnimation;
    Animation attackAnimationLeft;
    Timer headAnimationTimer = new Timer();

    bool attacking = false;

    public Elephant(double width, double height)
        : base(width, height)
    {
        if (bodyImagesRight == null)
        {
            bodyImagesRight = Game.LoadImages("elephant\\body_1", "elephant\\body_2", "elephant\\body_3");
            bodyImagesLeft = Image.Mirror(bodyImagesRight);
            headImagesRight = Game.LoadImages("elephant\\head_1", "elephant\\head_2", "elephant\\head_3");
            headImagesLeft = Image.Mirror(headImagesRight);
            closeEyesImagesRight = Game.LoadImages("elephant\\head_1", "elephant\\head_eyes_closed");
            closeEyesImagesLeft = Image.Mirror(closeEyesImagesRight);
            attackRight = Game.LoadImages( 
                "elephant\\head_attack1", "elephant\\head_attack2", "elephant\\head_attack3",
                "elephant\\head_attack4", "elephant\\head_attack5", "elephant\\head_attack6",
                "elephant\\head_attack7", "elephant\\head_attack8", "elephant\\head_attack9" );
            attackLeft = Image.Mirror(attackRight);
        }

        const double WalkFps = 15;

        RightWalkingAnimation = new Animation(bodyImagesRight) { FPS = WalkFps };
        LeftWalkingAnimation = new Animation(bodyImagesLeft) { FPS = WalkFps };

        lookingAnimation = new Animation(headImagesRight) { FPS = 3 };
        lookingAnimationLeft = new Animation(headImagesLeft) { FPS = 3 };
        closeEyesAnimation = new Animation(closeEyesImagesRight) { FPS = 1 };
        closeEyesLeft = new Animation(closeEyesImagesLeft) { FPS = 1 };
        attackAnimation = new Animation(attackRight) { FPS = 15 };
        attackAnimationLeft = new Animation(attackLeft) { FPS = 15 };
        attackAnimation.Played += new Action(AttackAnimation_Played);
        attackAnimationLeft.Played += new Action(AttackAnimationLeft_Played);

        head = new GameObject(width * 0.9, height * 0.9);
        SetHeadAnimation(lookingAnimation);
        PositionHeadHorizontally(Direction.Right);
        head.Y = this.Height * 0.15;
        Add(head);

        headAnimationTimer.Interval = RandomGen.NextDouble(1, 5);
        headAnimationTimer.Timeout += AnimateHead;
        headAnimationTimer.Start();

        DirectionChanged += PositionHeadHorizontally;
    }

    void AttackAnimation_Played()
    {
        Projectile nut = new Projectile( 0.8, 1, Color.Brown );
        nut.Tag = "ammo1";
        nut.Position = new Vector( this.Right, this.Y );
        nut.MaximumLifetime = TimeSpan.FromSeconds( 2 );
        nut.Hit( nut.Mass * 200 * Vector.UnitX );
        Game.Add( nut );

        EndAttack( FacingDirection );
    }

    void AttackAnimationLeft_Played()
    {
        Projectile nut = new Projectile( 0.8, 1, Color.Brown );
        nut.Tag = "ammo1";
        nut.Position = new Vector( this.Left, this.Y );
        nut.MaximumLifetime = TimeSpan.FromSeconds( 2 );
        nut.Hit( nut.Mass * -200 * Vector.UnitX );
        Game.Add( nut );

        EndAttack( FacingDirection );
    }

    void EndAttack( Direction newDirection )
    {
        DirectionChanged -= EndAttack;
        SetHeadAnimation( lookingAnimation );
        headAnimationTimer.Start();
        attacking = false;
    }

    void AnimateHead()
    {
        int random = RandomGen.NextInt(100);
        if (random < 66)
        {
            SetHeadAnimation(closeEyesAnimation);
        }
        else
        {
            SetHeadAnimation(lookingAnimation);
        }

        headAnimationTimer.Interval = RandomGen.NextDouble(3, 10);
        head.Animation.Start(1);
    }

    void SetHeadAnimation(Animation rightFacingAnimation)
    {
        if (FacingDirection == Direction.Right)
        {
            head.Animation = rightFacingAnimation;
        }
        else
        {
            if (rightFacingAnimation == lookingAnimation)
                head.Animation = lookingAnimationLeft;
            else if (rightFacingAnimation == closeEyesAnimation)
                head.Animation = closeEyesLeft;
            else if (rightFacingAnimation == attackAnimation)
                head.Animation = attackAnimationLeft;
        }
    }

    void PositionHeadHorizontally(Direction direction)
    {
        double unit = (direction == Direction.Left) ? -1.0 : 1.0;
        head.X = unit * this.Width * 0.40;
        SetHeadAnimation(lookingAnimation);
    }

    public void Attack()
    {
        if ( attacking ) return;

        headAnimationTimer.Stop();
        SetHeadAnimation( attackAnimation );
        head.Animation.Start( 1 );
        attacking = true;
        DirectionChanged += EndAttack;
    }
}
