﻿using System;
using Jypeli;
using Jypeli.Widgets;
using Jypeli.Effects;


enum Outcome
{
    Menu,
    NewGame,
    LevelFinished,
    ExitGame,
}


class PlatformGame : PhysicsGame
{
    int levelNumber = 1;

    const double nopeus = 60;
    const double hyppyNopeus = 280;

    ExplosionSystem rajahdys = new ExplosionSystem( LoadImage( "jytky" ), 500 )
    {
        MinScale = 0.6, MaxScale = 0.7,
        MinVelocity = 10.0, MaxVelocity = 15.0,
        //MinAcceleration = 200.0, MaxAcceleration = 300.0
        MinAcceleration = 2000.0, MaxAcceleration = 3000.0
    };

    Image backgroundImage = Game.LoadImage("background");
    Image tahdenKuva = Game.LoadImage("tahti");
    Image groundImage = Game.LoadImage("stone_ground");
    Image groundWithGrassImage = Game.LoadImage("stone_ground_with_grass");
    Image groundWithRoof = Game.LoadImage( "stone_roof" );
    Image rightGround = Game.LoadImage( "stone_ground_right" );
    Image rightGroundWithGrass = Game.LoadImage( "stone_ground_right_with_grass" );
    Image leftGroundWithGrass = Game.LoadImage( "stone_ground_left_with_grass" );
    Image treeImage = Game.LoadImage("tree");
    Image tombstoneImage;

    IntMeter pisteLaskuri = new IntMeter(0);
    Label pisteNaytto;

    Elephant pelaaja;


    public override void Begin()
    {
        tombstoneImage = Image.DrawTextOnImage( Image.FromGradient( 40, 80, Color.Black, Color.DarkGray ), "RIP", Font.Default, Color.Yellow );
        ShowMainMenu();
    }

    void ShowMainMenu()
    {
        ClearAll();

        Level.Background.CreateGradient(Color.Gray, Color.DarkBlue);

        Menu menu = new Menu(200, 400);
        menu.AddItem("New Game", BeginNewGame);
        menu.AddItem("Exit", Exit);
        Add(menu);

        Keyboard.Listen(Key.Up, ButtonState.Pressed, menu.MoveUp, null);
        Keyboard.Listen(Key.Down, ButtonState.Pressed, menu.MoveDown, null);
        Keyboard.Listen(Key.Enter, ButtonState.Pressed, menu.Select, null);
    }

    void BeginNewGame()
    {
        levelNumber = 1;
        BeginNextLevel();
    }

    void BeginNextLevel()
    {
        ClearAll();

        string levelfile = SelectLevel(levelNumber);
        if (levelfile == null)
        {
            Exit();
            return;
        }
        LataaKentta(levelfile);
        LisaaNaytot();
        lisaaNappaimet();
        Gravity = new Vector(0, -500);
        Camera.ZoomFactor = 4.0;
        Camera.StayInLevel = true;
        Camera.Follow(pelaaja);
        Add( rajahdys );
    }

    void LisaaNaytot()
    {
        pisteNaytto = new Label();
        pisteNaytto.X = Game.Screen.RightSafe - 250;
        pisteNaytto.Y = Game.Screen.TopSafe - 50;
        pisteNaytto.TextColor = Color.LightYellow;
        pisteNaytto.BindTo(pisteLaskuri);
        Add(pisteNaytto);
    }

    void LataaKentta(string levelFile)
    {
        GetLayer(-1).RelativeTransition = new Vector(0.9, 0.9);
        GetLayer(-2).RelativeTransition = new Vector(0.8, 0.8);

        Level.Background.CreateGradient(Color.Gray, Color.DarkBlue);

        TileMap ruudut = TileMap.FromFile(levelFile);
        ruudut.SetTileMethod('P', LuoPelaaja);
        ruudut.SetTileMethod('m', LuoHiiri);
        ruudut.SetTileMethod('*', LuoTahti);
        ruudut.SetTileMethod('#', LuoPalikka);
        ruudut.SetTileMethod('T', CreateTree);
        ruudut.SetTileMethod('_', CreateTopGround);
        ruudut.SetTileMethod('v', CreateRoof);
        ruudut.SetTileMethod(',', CreateLeftTopGround);
        ruudut.SetTileMethod('.', CreateRightTopGround);
        ruudut.SetTileMethod('R', CreateRight);
        ruudut.SetTileMethod('M', LuoMaali);
        ruudut.Execute(20, 20);

        GameObject background = new GameObject(Level.Width, Level.Width / 4);
        background.X = 0;
        background.Y = Level.Bottom + background.Height / 2;
        background.Image = backgroundImage;
        Add(background, -2);

        Level.CreateBorders();
    }

    void LuoPelaaja(Vector paikka, double leveys, double korkeus)
    {
        pelaaja = new Elephant(10, 10);
        pelaaja.Position = paikka;
        pelaaja.Mass = 10;
        pelaaja.Tag = "norsu";

        AddCollisionHandler(pelaaja, OsuiMaaliin);
        Add(pelaaja);
    }

    void LuoHiiri( Vector paikka, double leveys, double korkeus )
    {
        EvilMouse hiiri = new EvilMouse( 16, 12 );
        //EvilMouse hiiri = new EvilMouse( leveys, korkeus );
        hiiri.Position = paikka;
        hiiri.Mass = 2;

        AddCollisionHandler( hiiri, VihollinenOsui );
        Add( hiiri );
    }

    void LuoMaali(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject maali = PhysicsObject.CreateStaticObject(leveys, korkeus, Shape.Circle);
        maali.Position = paikka;
        maali.Tag = "maali";
        maali.IgnoresCollisionResponse = true;
        maali.Image = tahdenKuva;
        Add(maali);
    }

    void CreateTree(Vector position, double width, double height)
    {
        GameObject tree = new GameObject(width * 4, height * 4);
        tree.Position = position + new Vector(0, tree.Height / 2 - height / 2);
        tree.Image = treeImage;
        Add(tree, -1);
    }

    void CreateTopGround(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus, Shape.Rectangle);
        taso.Position = paikka;
        taso.Image = groundWithGrassImage;
        Add(taso);
    }

    void CreateRoof( Vector paikka, double leveys, double korkeus )
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject( leveys, korkeus, Shape.Rectangle );
        taso.Position = paikka;
        taso.Image = groundWithRoof;
        Add( taso );
    }

    void CreateRight( Vector paikka, double leveys, double korkeus )
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject( leveys, korkeus, Shape.Rectangle );
        taso.Position = paikka;
        taso.Image = rightGround;
        Add( taso );
    }

    void CreateLeftTopGround( Vector paikka, double leveys, double korkeus )
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject( leveys, korkeus, Shape.Rectangle );
        taso.Position = paikka;
        taso.Image = leftGroundWithGrass;
        Add( taso );
    }

    void CreateRightTopGround( Vector paikka, double leveys, double korkeus )
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject( leveys, korkeus, Shape.Rectangle );
        taso.Position = paikka;
        taso.Image = rightGroundWithGrass;
        Add( taso );
    }

    void LuoPalikka(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus, Shape.Rectangle);
        taso.Position = paikka;
        taso.Image = groundImage;
        Add(taso);
    }

    void LuoTahti(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject tahti = new PhysicsObject(5, 5);
        tahti.Position = paikka;
        tahti.Mass = 0.005;
        tahti.Shape = Shape.Circle;
        tahti.Restitution = 1.0;
        tahti.Image = tahdenKuva;
        AddCollisionHandler(tahti, Keraa);
        Add(tahti);
    }

    void LuoHautakivi(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject hautakivi = new PhysicsObject( leveys, korkeus, Shape.Rectangle );
        hautakivi.MomentOfInertia = Double.PositiveInfinity;
        hautakivi.Image = tombstoneImage;
        hautakivi.Position = paikka;
        hautakivi.Tag = "ammo0";
        Add( hautakivi );
    }

    void lisaaNappaimet()
    {
        //Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");
        Keyboard.Listen(Key.Left, ButtonState.Down, Liikuta, null, -nopeus);
        Keyboard.Listen(Key.Right, ButtonState.Down, Liikuta, null, nopeus);
        Keyboard.Listen(Key.Up, ButtonState.Pressed, Hyppaa, "Hyppää");
        Keyboard.Listen(Key.Space, ButtonState.Down, Hyokkaa, "Hyökkää");

        Keyboard.AddHelpText<double>(Liikuta, "Liiku");
    }

    void Liikuta(double nopeus)
    {
        pelaaja.Walk(nopeus);
    }

    void Hyppaa()
    {
        pelaaja.Jump(hyppyNopeus);
    }

    void Hyokkaa()
    {
        pelaaja.Attack();
    }

    void OsuiMaaliin(PhysicsObject collidingObject, PhysicsObject otherObject)
    {
        if ( otherObject.Tag.ToString() == "maali" )
        {
            int edellisenKentanPisteet = pisteLaskuri.Value;
            levelNumber++;
            BeginNextLevel();
        }
    }

    void VihollinenOsui( PhysicsObject collidingObject, PhysicsObject otherObject )
    {
        if ( !pelaaja.IsDestroyed && otherObject == pelaaja )
        {
            rajahdys.AddEffect( pelaaja.Position, 100 );
            LuoHautakivi( pelaaja.Position + 40 * Vector.UnitY, 8, 16 );
            pelaaja.Destroy();
            pisteLaskuri.Reset();
            Timer.SingleShot( 3.0, BeginNextLevel );
            return;
        }

        string tagStr = otherObject.Tag.ToString();
        if ( string.IsNullOrEmpty( tagStr ) ) return;

        if ( tagStr.Length == 5 && tagStr.Substring( 0, 4 ) == "ammo" && otherObject.Velocity.Magnitude > 3 )
        {
            rajahdys.AddEffect( collidingObject.Position, 70 );
            collidingObject.Destroy();

            int points = int.Parse( tagStr[4].ToString() );
            pisteLaskuri.Value += points;

            if ( points > 0 && points < 9 )
            {
                // Lisää pisteitä komboista
                points++;
                otherObject.Tag = "ammo" + points.ToString();
            }
        }
    }

    void Keraa(PhysicsObject collidingObject, PhysicsObject otherObject)
    {
        if (otherObject == pelaaja)
        {
            PhysicsObject tahti = collidingObject;
            tahti.Destroy();
            pisteLaskuri.Value += 5;
        }
    }

    //private void NextState(Outcome outcome)
    //{
    //    GameState state = null;

    //    switch (outcome)
    //    {
    //        case Outcome.Menu:
    //            state = new Menu(this);
    //            break;
    //        case Outcome.NewGame:
    //            levelNumber = 1;
    //            state = SelectLevel(levelNumber);
    //            break;
    //        case Outcome.LevelFinished:
    //            levelNumber++;
    //            state = SelectLevel(levelNumber);
    //            break;
    //        default:
    //            Exit();
    //            break;
    //    }

    //    if (state != null)
    //    {
    //        state.Begin();
    //    }
    //}

    string SelectLevel(int levelNumber)
    {
        switch (levelNumber)
        {
            case 1:
                return "kentta1.txt";
            case 2:
                return "kentta2.txt";
            case 3:
                return "kentta3.txt";
            default:
                return null;
        }
    }
}
