﻿using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Assets;

namespace Peli
{
    public class Peli : PhysicsGame
    {
        PhysicsObject player;
        Image ghostImage;

        const double movementForce = 10;

        public override void Begin()
        {
            ghostImage = LoadImage( "ghost" );

            Camera.ZoomToLevel();
            Level.CreateBorders();

            player = new PhysicsObject( 20, 20, Shape.Circle );
            player.Color = Color.Green;
            player.LinearDamping = 0.98;
            Add( player );

            AddEnemies();

            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, null );
            Keyboard.Listen( Key.Left, ButtonState.Down, Move, null, new Vector( -1, 0 ) * movementForce );
            Keyboard.Listen( Key.Right, ButtonState.Down, Move, null, new Vector( 1, 0 ) * movementForce );
            Keyboard.Listen( Key.Up, ButtonState.Down, Move, null, new Vector( 0, 1 ) * movementForce );
            Keyboard.Listen( Key.Down, ButtonState.Down, Move, null, new Vector( 0, -1 ) * movementForce );
        }

        void AddEnemies()
        {
            var ghost1 = CreateGhost( -100, 100 );
            ghost1.Brain = new RandomMoverBrain( 100 );

            //var ghost2 = CreateGhost( -200, 200 );
            //WandererBrain aivot = new WandererBrain();
            //aivot.Add( new Vector( -200, 250 ), new Vector( 200, 250 ) );
            //ghost2.Brain = aivot;

            Vector[] polku = new Vector[]
            {
                new Vector( -200, 250 ),
                new Vector( 200, 250 ),
                new Vector( 150, 50 ),
                new Vector( 0, 230 ),
            };

            var pathGhost = CreateGhost( -200, 200 );
            PathFollowerBrain polkuAivot = new PathFollowerBrain( polku );
            polkuAivot.Speed = 200;
            polkuAivot.Loop = true;
            pathGhost.Brain = polkuAivot;

            var stalker = CreateGhost( 200, 100 );
            FollowerBrain followBrain = new FollowerBrain( player );
            followBrain.Speed = 10;
            stalker.Brain = followBrain;
            Add( stalker );
        }

        PhysicsObject CreateGhost( double x, double y )
        {
            var ghost = new PhysicsObject( 40, 40, Shape.Circle );
            ghost.IgnoresGravity = true;
            ghost.Mass = 50;
            ghost.CanRotate = false;
            ghost.Position = new Vector( x, y );
            ghost.Image = ghostImage;
            Add( ghost );
            return ghost;
        }

        void Move( Vector force )
        {
            player.Hit( force );
        }
    }
}
