﻿using System;
using System.Text;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Assets;

namespace CollisionTest
{
    public class Peli : PhysicsGame
    {
        const double tiilenLeveys = 30;
        const double tiilenKorkeus = 30;

        // fields for testing vertex distance and gridspacing:

        const double minGridSpacing = 1;
        const double maxGridSpacing = 30;

        const double minVertexDistance = 1;
        const double maxVertexDistance = 30;

        const int gridSpacingStepCount = 7;
        const int vertexDistanceStepCount = 7;

        int gridSpacingIndex = 4;
        int vertexDistanceIndex = 4;

        // fields for quality test:

        const double maxQuality = 1.0;
        const double minQuality = 0.0;

        const int qualityStepCount = 10;
        int qualityIndex = 3;


        public override void Begin()
        {
            Keyboard.Listen( Key.F2, ButtonState.Pressed, IndividualParameterTestCase, "Test the vertexDistance and gridSpacing parameters" );
            Keyboard.Listen( Key.F3, ButtonState.Pressed, QualityTestCase, "Test the quality parameter" );
            Keyboard.Listen(Key.F4, ButtonState.Pressed, ThinBlockTestCase, "Test collisions with thin blocks");
            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, "Exit" );

            MessageDisplay.Add( "Test program for finding optimal parameters for collision shapes." );
            MessageDisplay.Add( "" );
            ShowControlHelp();
        }

        void ThinBlockTestCase()
        {
            ClearAll();

            MessageDisplay.Add("Testing performance with thin blocks");

            MessageDisplay.Add("");
            MessageDisplay.Add("Press Enter");

            Keyboard.Listen(Key.Enter, ButtonState.Pressed, CreateLevelWithThinBlocs, null);
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, null);
        }

        void IndividualParameterTestCase()
        {
            ClearAll();

            MessageDisplay.Add( "Testing vertexDistance and gridSpacing parameters." );
            MessageDisplay.Add( "Use the arrow keys to change test case." );
            MessageDisplay.Add( "Space restarts current test case." );
            MessageDisplay.Add( "" );
            MessageDisplay.Add( "Test cases:" );
            MessageDisplay.Add( "" );

            for ( int y = 0; y <= vertexDistanceStepCount; y++ )
            {
                StringBuilder row = new StringBuilder();

                for ( int x = 0; x <= gridSpacingStepCount; x++ )
                {
                    row.Append(
                        string.Format( "({0,5}, {1,5}) ",
                        GetGridSpacing( x ).ToString( "F2" ),
                        GetGridSpacing( y ).ToString( "F2" ) ) );
                }
                MessageDisplay.Add( row.ToString() );
            }

            MessageDisplay.Add( "" );
            MessageDisplay.Add( "Press Enter" );

            Keyboard.Listen( Key.Enter, ButtonState.Pressed, CreateParameterLevel, null );
            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, null );
        }

        void QualityTestCase()
        {
            ClearAll();

            MessageDisplay.Add( "Testing the quality parameter" );

            MessageDisplay.Add( "" );
            MessageDisplay.Add( "Press Enter" );

            Keyboard.Listen( Key.Enter, ButtonState.Pressed, CreateQualityLevel, null );
            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, null );
        }

        CollisionShapeQuality GetQuality( int index )
        {
            double stepSize = ( maxQuality - minQuality ) / qualityStepCount;
            return CollisionShapeQuality.FromValue( minQuality + index * stepSize );
        }

        double GetGridSpacing( int index )
        {
            double stepSize = ( maxGridSpacing - minGridSpacing ) / gridSpacingStepCount;
            return minGridSpacing + index * stepSize;
        }

        double GetVertexDistance( int index )
        {
            double stepSize = ( maxVertexDistance - minVertexDistance ) / vertexDistanceStepCount;
            return minVertexDistance + index * stepSize;
        }

        void CreateEmptyLevel()
        {
            ClearAll();
            Gravity = new Vector(0, -100);
            Level.CreateBorders();
            Camera.ZoomToLevel();
        }

        void CreateLevelWithThinBlocs()
        {
            CreateEmptyLevel();

            BlockCreator createFunction = delegate(double width, double height)
            {
                return new PhysicsObject( width, height, Shape.Rectangle );
            };

            TeeTiilet(1, 10, tiilenLeveys * 10, tiilenKorkeus, createFunction);
        }

        void CreateQualityLevel()
        {
            if ( qualityIndex < 0 || qualityIndex > qualityStepCount )
                return;

            CreateEmptyLevel();

            CollisionShapeQuality quality = GetQuality( qualityIndex );    

            BlockCreator createFunction = delegate( double width, double height )
            {
                return new PhysicsObject( width, height, Shape.Rectangle, quality );
            };

            TeeTiilet( 10, 10, tiilenLeveys, tiilenKorkeus, createFunction );
            AddTriangles();

            // TODO: print vertexDistance and gridSpacing.
            MessageDisplay.Add( "Quality: " + quality.Value.ToString( "F2" ) );

            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, "Exit" );
            Keyboard.Listen( Key.F1, ButtonState.Pressed, ShowControlHelp, "Help" );
            Keyboard.Listen( Key.Space, ButtonState.Pressed, CreateQualityLevel, "Restart" );
            Keyboard.Listen( Key.Up, ButtonState.Pressed, ChangeQuality, "Better quality", 1 );
            Keyboard.Listen( Key.Down, ButtonState.Pressed, ChangeQuality, "Worse quality", -1 );
        }

        void CreateParameterLevel()
        {
            if ( gridSpacingIndex < 0 || gridSpacingIndex > gridSpacingStepCount )
                return;
            if ( vertexDistanceIndex < 0 || vertexDistanceIndex > vertexDistanceStepCount )
                return;

            CreateEmptyLevel();

            double gridSpacing = GetGridSpacing( gridSpacingIndex );
            double vertexDistance = GetVertexDistance( vertexDistanceIndex );

            BlockCreator createFunction = delegate( double width, double height )
            {
                return new PhysicsObject( width, height, Shape.Rectangle, vertexDistance, gridSpacing );
            };

            TeeTiilet( 10, 10, tiilenLeveys, tiilenKorkeus, createFunction );
            AddTriangles();

            MessageDisplay.Add( "GridSpacing: " + gridSpacing.ToString( "F2" ) );
            MessageDisplay.Add( "VertexDistance: " + vertexDistance.ToString( "F2" ) );

            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, "Exit" );
            Keyboard.Listen( Key.F1, ButtonState.Pressed, ShowControlHelp, "Help" );
            Keyboard.Listen( Key.Space, ButtonState.Pressed, CreateParameterLevel, "Restart" );
            Keyboard.Listen( Key.Left, ButtonState.Pressed, ChangeGridSpacing, "Smaller grid spacing", -1 );
            Keyboard.Listen( Key.Right, ButtonState.Pressed, ChangeGridSpacing, "Higher grid spacing", 1 );
            Keyboard.Listen( Key.Up, ButtonState.Pressed, ChangeVertexDistance, "Smaller vertex distance", -1 );
            Keyboard.Listen( Key.Down, ButtonState.Pressed, ChangeVertexDistance, "Higher vertex distance", 1 );
        }

        void ChangeGridSpacing( int change )
        {
            gridSpacingIndex += change;
            CreateParameterLevel();
        }

        void ChangeVertexDistance( int change )
        {
            vertexDistanceIndex += change;
            CreateParameterLevel();
        }

        void ChangeQuality( int change )
        {
            qualityIndex += change;
            CreateQualityLevel();
        }

        void AddTriangles()
        {
            var tr = new PhysicsObject( 50, 50, Shape.Triangle );
            tr.Position = Level.Center + new Vector( 0, 100 );
            tr.Color = Color.Pink;
            tr.Mass = 4;
            Add( tr );
        }

        delegate PhysicsObject BlockCreator( double width, double height );

        void TeeTiilet( int rows, int columns, double width, double height, BlockCreator create )
        {
            double alkuX = -300;
            double alkuY = -350;
            double tiilenPaino = 3;

            for (int column = 0; column < columns; column++)
            {
                for (int row = 0; row < rows; row++)
                {
                    PhysicsObject tiili = create(width, height);
                    tiili.Mass = tiilenPaino;
                    tiili.X = alkuX + width * row;
                    tiili.Y = alkuY + height * column;
                    tiili.Color = RandomGen.NextColor();
                    Add(tiili);
                }
            }
        }

        void Move( PhysicsObject o, Vector force )
        {
            o.Push( force );
        }
    }
}
