﻿using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Assets;
using Jypeli.Widgets;

namespace ControlTest
{
    public class Peli : Game
    {
        GameObject mouseCursor;
        GameObject movementMarker;

        public override void Begin()
        {
            DrawPerimeter = true;
            Camera.ZoomToLevel();

            ListenController();
            ListenKeyboard();
            ListenMouse();

            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, null );

            ShowControlHelp();
        }

        void ListenController()
        {
            var leftStickMarker = CreateMarker();
            leftStickMarker.Color = Color.Red;
            var rightStickMarker = CreateMarker();
            rightStickMarker.Color = Color.Blue;

            var leftTriggerMarker = CreateMarker();
            leftTriggerMarker.Size = new Vector( 200, 200 );
            leftTriggerMarker.X = -Level.Width / 2.0;
            leftTriggerMarker.Color = Color.DarkRed;
            var rightTriggerMarker = CreateMarker();
            rightTriggerMarker.X = Level.Width / 2.0;
            rightTriggerMarker.Color = Color.DarkBlue;

            ControllerOne.Listen( Button.LeftTrigger, ButtonState.Pressed, Show, null, leftTriggerMarker );
            ControllerOne.Listen( Button.LeftTrigger, ButtonState.Released, Hide, null, leftTriggerMarker );

            ControllerOne.ListenAnalog( AnalogControl.LeftStick, 0.01, AnalogHandler, "Liikuta palloa", leftStickMarker );
            ControllerOne.ListenAnalog( AnalogControl.RightStick, 0.01, AnalogHandler, "Liikuta palloa", rightStickMarker );

            ControllerOne.ListenAnalog( AnalogControl.LeftTrigger, 0.01, TriggerHandler, null, leftTriggerMarker );
            ControllerOne.ListenAnalog( AnalogControl.RightTrigger, 0.01, TriggerHandler, null, rightTriggerMarker );
        }

        GameObject CreateKey()
        {
            GameObject key = new GameObject(20, 20, Shape.Rectangle);
            key.Color = Color.Gray;
            Add(key);
            return key;
        }

        void ListenKeyboard()
        {
            GameObject left, right, up, down;
            left = CreateKey();
            left.X = -25;
            left.Y = 100;
            right = CreateKey();
            right.X = 25;
            right.Y = 100;
            down = CreateKey();
            down.X = 0;
            down.Y = 100;
            up = CreateKey();
            up.X = 0;
            up.Y = 125;

            Keyboard.Listen(Key.Left, ButtonState.Pressed, ColorPressed, null, left);
            Keyboard.Listen(Key.Left, ButtonState.Released, ColorReleased, null, left);
            Keyboard.Listen(Key.Right, ButtonState.Pressed, ColorPressed, null, right);
            Keyboard.Listen(Key.Right, ButtonState.Released, ColorReleased, null, right);
            Keyboard.Listen(Key.Up, ButtonState.Pressed, ColorPressed, null, up);
            Keyboard.Listen(Key.Up, ButtonState.Released, ColorReleased, null, up);
            Keyboard.Listen(Key.Down, ButtonState.Pressed, ColorPressed, null, down);
            Keyboard.Listen(Key.Down, ButtonState.Released, ColorReleased, null, down);

            Keyboard.Listen(Key.F5, ButtonState.Pressed, DisableControls, null);
            Keyboard.Listen(Key.F6, ButtonState.Pressed, EnableControls, null);
        }

        void DisableControls()
        {
            Keyboard.Disable(Key.Left);
            Keyboard.Disable(Key.Right);
            Keyboard.Disable(Key.Up);
            Keyboard.Disable(Key.Down);

            Mouse.Disable(MouseButton.Left);
        }

        void EnableControls()
        {
            Keyboard.Enable(Key.Left);
            Keyboard.Enable(Key.Right);
            Keyboard.Enable(Key.Up);
            Keyboard.Enable(Key.Down);

            Mouse.Enable(MouseButton.Left);
        }

        void ColorPressed(GameObject key)
        {
            key.Color = Color.Red;
        }

        void ColorReleased(GameObject key)
        {
            key.Color = Color.Gray;
        }

        void ListenMouse()
        {
            PushButton buttonOnScreen = new PushButton( 50, 50, "Press!" );
            buttonOnScreen.X = Screen.Center.X + Screen.Width / 4;
            buttonOnScreen.Y = Screen.Center.Y + Screen.Height / 4;
            Add( buttonOnScreen );

            GameObject buttonOnWorld = new GameObject( 120, 100 );
            buttonOnWorld.Position = Level.Center + new Vector( -Level.Width / 4, Level.Height / 4 );
            buttonOnWorld.Color = Color.GreenYellow;
            Add( buttonOnWorld );

            mouseCursor = new GameObject( 15, 20, Shape.Triangle );
            mouseCursor.Angle = Angle.FromDegrees( 45 );
            mouseCursor.Color = Color.DarkGreen;
            Add( mouseCursor );

            movementMarker = new GameObject( 10, 10, Shape.Circle );
            movementMarker.Color = Color.Blue;
            Add( movementMarker );

            Mouse.ListenMovement( 0.1, MoveCursor, "moving cursor..." );
            Mouse.Listen( MouseButton.Left, ButtonState.Pressed, HighlightCursor, null );
            Mouse.Listen( MouseButton.Left, ButtonState.Released, UnHighlight, null );
            Mouse.ListenOn( buttonOnScreen, MouseButton.Left, ButtonState.Pressed, PressButton, "Klikkaa nappuloita" );
            Mouse.ListenOn( buttonOnWorld, MouseButton.Left, ButtonState.Pressed, PressButton, "Klikkaa nappuloita" );
        }

        void HighlightCursor()
        {
            mouseCursor.Color = Color.Red;
        }

        void UnHighlight()
        {
            mouseCursor.Color = Color.DarkGreen;
        }

        void PressButton()
        {
            MessageDisplay.Add( "Painoit nappia" );
        }

        void MoveCursor( AnalogState state )
        {
            Vector position = Mouse.PositionOnWorld;
            mouseCursor.Position = position;
            movementMarker.Position = position + state.MouseMovement * 5;
        }

        GameObject CreateMarker()
        {
            GameObject m = new GameObject( 20, 20, Shape.Circle );
            Add( m );
            return m;
        }

        void Show( GameObject o )
        {
            MessageDisplay.Add( "Pressed" );
            o.IsVisible = true;
        }

        void Hide( GameObject o )
        {
            MessageDisplay.Add( "Released" );
            o.IsVisible = false;
        }

        void AnalogHandler( AnalogState analogState, GameObject marker )
        {
            marker.Position = new Vector( analogState.StateVector.X * Level.Width / 2.0, analogState.StateVector.Y * Level.Height / 2.0 );
        }

        void TriggerHandler( AnalogState analogState, GameObject marker )
        {
            marker.Y = Level.Bottom + analogState.State * Level.Height;
        }
    }
}
