﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;

public class Peli : Game
{
    static readonly string[] randomStrings = new string[]
    {
        "foo",
        "kahvia mustana, kiitos",
        "Ny rillataan!",
        "NOOOOOOOOOOOOOOOO!!!!!",
        "Taivas varrrrjele, mitä sieltä tulee!!!!",
        "bar",
        "merkkijono",
        "23323232232323322323989849597348787347126371632",
        "Luke, I am your father.",
        "Sieltä tulee maaaaaaali!!!"
    };

    Widget parent;
    ILayout layout;
    Widget help;
    Stack<Widget> thingsAddedToLayout = new Stack<Widget>();

    public override void Begin()
    {
        layout = new VerticalLayout();
        parent = new Widget(layout) { Color = Color.Transparent, BorderColor = Color.Black };
        parent.X = 200;
        parent.Y = 0;
        parent.Size = new Vector(Screen.Width / 2, Screen.Height / 2);
        Add(parent);

        AddLabel(parent);

        AddControls();
    }

    void AddControls()
    {
        help = new Widget(new VerticalLayout() { TopPadding = 10, BottomPadding = 10, LeftPadding = 10, RightPadding = 10 })
        {
            Color = new Color(100, 100, 100, 100),
            BorderColor = Color.Black,
            X = Screen.Left + 300,
            Y = Screen.Top - 250,
        };
        Add(help);

        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, null);
        Keyboard.Listen(Key.Tab, ButtonState.Pressed, SwitchLayout, null);
        Help(Key.Tab, "Switch between horizontal and vertical layout");
        Keyboard.Listen(Key.D1, ButtonState.Pressed, AddLabel, null, parent);
        Keyboard.Listen(Key.D2, ButtonState.Pressed, AddFixedSizeBox, null);
        Keyboard.Listen(Key.D3, ButtonState.Pressed, AddExpandingBox, null);
        Keyboard.Listen(Key.D4, ButtonState.Pressed, AddHorizontalLayout, null);
        Keyboard.Listen(Key.D5, ButtonState.Pressed, AddVerticalLayout, null);
        Keyboard.Listen(Key.Back, ButtonState.Pressed, RemoveFromLayout, null);
        Help(Key.D1, "Add Label");
        Help(Key.D2, "Add fixed-size box");
        Help(Key.D3, "Add expanding-size box");
        Help(Key.D4, "Add layout");
        Help(Key.Back, "Remove an item from layout");
        Keyboard.Listen(Key.W, ButtonState.Pressed, IncreaseSpacing, null, 5.0);
        Keyboard.Listen(Key.S, ButtonState.Pressed, IncreaseSpacing, null, -5.0);
        Help(Key.W, Key.S, "Spacing");
        Keyboard.Listen(Key.E, ButtonState.Pressed, delegate() { layout.LeftPadding += 5; }, null);
        Keyboard.Listen(Key.D, ButtonState.Pressed, delegate() { layout.LeftPadding -= 5; }, null);
        Help(Key.E, Key.D, "Left padding");
        Keyboard.Listen(Key.R, ButtonState.Pressed, delegate() { layout.RightPadding += 5; }, null);
        Keyboard.Listen(Key.F, ButtonState.Pressed, delegate() { layout.RightPadding -= 5; }, null);
        Help(Key.R, Key.F, "Right padding");
        Keyboard.Listen(Key.T, ButtonState.Pressed, delegate() { layout.TopPadding += 5; }, null);
        Keyboard.Listen(Key.G, ButtonState.Pressed, delegate() { layout.TopPadding -= 5; }, null);
        Help(Key.T, Key.G, "Top padding");
        Keyboard.Listen(Key.Y, ButtonState.Pressed, delegate() { layout.BottomPadding += 5; }, null);
        Keyboard.Listen(Key.H, ButtonState.Pressed, delegate() { layout.BottomPadding -= 5; }, null);
        Help(Key.Y, Key.H, "Bottom padding");
        Keyboard.Listen(Key.U, ButtonState.Pressed, delegate() { parent.Width += 30; }, null);
        Keyboard.Listen(Key.J, ButtonState.Pressed, delegate() { parent.Width -= 30; }, null);
        Help(Key.U, Key.J, "Width");
        Keyboard.Listen(Key.I, ButtonState.Pressed, delegate() { parent.Height += 30; }, null);
        Keyboard.Listen(Key.K, ButtonState.Pressed, delegate() { parent.Height -= 30; }, null);
        Help(Key.I, Key.K, "Height");
    }

    void Help(Key k, string text)
    {
        help.Add(new Label(k.ToString() + " - " + text));
    }

    void Help(Key k, Key l, string text)
    {
        help.Add(new Label(k.ToString() + ", " + l + " - " + text));
    }

    void SwitchLayout()
    {
        if (layout is VerticalLayout)
        {
            layout = new HorizontalLayout();
            parent.Layout = layout;
        }
        else
        {
            layout = new VerticalLayout();
            parent.Layout = layout;
        }
    }

    void IncreaseSpacing(double amount)
    {
        if (layout is VerticalLayout)
        {
            ((VerticalLayout)layout).Spacing += amount;
        }
        else if (layout is HorizontalLayout)
        {
            ((HorizontalLayout)layout).Spacing += amount;
        }
    }

    void RemoveFromLayout()
    {
        if (thingsAddedToLayout.Count > 0)
        {
            Widget w = thingsAddedToLayout.Pop();
            w.Destroy();
        }
    }

    void AddLabel(Widget parent)
    {
        string text = randomStrings[RandomGen.NextInt(randomStrings.Length)];
        Widget w = new Label(text);
        parent.Add(w);
        thingsAddedToLayout.Push(w);
    }

    void AddHorizontalLayout()
    {
        AddLayout(CreateHorizontalLayout());
    }

    void AddVerticalLayout()
    {
        AddLayout(CreateVerticalLayout());
    }

    void AddLayout(Widget w)
    {
        parent.Add(w);
        thingsAddedToLayout.Push(w);
    }

    Widget CreateHorizontalLayout()
    {
        return CreateLayout(new HorizontalLayout { Spacing = 5, LeftPadding = 5, RightPadding = 5, TopPadding = 5, BottomPadding = 5 });
    }

    Widget CreateVerticalLayout()
    {
        return CreateLayout(new VerticalLayout { Spacing = 5, LeftPadding = 5, RightPadding = 5, TopPadding = 5, BottomPadding = 5 });
    }

    Widget CreateLayout(ILayout layout)
    {
        var w = new Widget(layout) { BorderColor = Color.Red, Color = new Color(50, 50, 50, 50) };

        int childCount = RandomGen.NextInt(1, 8);
        for (int i = 0; i < childCount; i++)
        {
            Widget child;
            switch (RandomGen.NextInt(0, 10))
            {
                case 0:
                case 1:
                case 2:
                case 3:
                    child = CreateFixedSizeBox();
                    break;
                case 4:
                    child = CreateHorizontalLayout();
                    break;
                case 5:
                    child = CreateVerticalLayout();
                    break;
                default:
                    child = CreateExpandingBox();
                    break;
            }
            w.Add(child);
        }

        return w;
    }

    void AddExpandingBox()
    {
        var box = CreateExpandingBox();
        parent.Add(box);
        thingsAddedToLayout.Push(box);
    }

    Widget CreateExpandingBox()
    {
        Widget box = new Widget(RandomGen.NextDouble(50, 200), RandomGen.NextDouble(50, 200))
        {
            HorizontalSizing = Sizing.Expanding,
            VerticalSizing = Sizing.Expanding,
            Color = RandomGen.NextColor(Color.Green, Color.LightGreen)
        };
        box.PreferredSize = box.Size;
        return box;
    }

    void AddFixedSizeBox()
    {
        var box = CreateFixedSizeBox();
        parent.Add(box);
        thingsAddedToLayout.Push(box);
    }

    Widget CreateFixedSizeBox()
    {
        Widget box = new Widget(RandomGen.NextDouble(50, 200), RandomGen.NextDouble(50, 200)) { HorizontalSizing = Sizing.FixedSize, Color = RandomGen.NextColor(Color.Red, Color.Orange) };
        box.PreferredSize = box.Size;
        return box;
    }
}
