﻿using System;
using Jypeli;
using Jypeli.Controls;

class Elephant2 : PlatformCharacter2
{
    static Image[] bodyImagesRight;
    static Image[] bodyImagesLeft;

    static Image[] headImagesRight;
    static Image[] headImagesLeft;
    static Image[] closeEyesImagesRight;
    static Image[] closeEyesImagesLeft;

    GameObject head;
    Animation lookingAnimation;
    Animation lookingAnimationLeft;
    Animation closeEyesAnimation;
    Animation closeEyesLeft;
    Timer headAnimationTimer = new Timer();

    public Elephant2(double width, double height)
        : base(width, height)
    {
        if (bodyImagesRight == null)
        {
            bodyImagesRight = Game.LoadImages("elephant\\body_1", "elephant\\body_2", "elephant\\body_3");
            bodyImagesLeft = Image.Mirror(bodyImagesRight);
            headImagesRight = Game.LoadImages("elephant\\head_1", "elephant\\head_2", "elephant\\head_3");
            headImagesLeft = Image.Mirror(headImagesRight);
            closeEyesImagesRight = Game.LoadImages("elephant\\head_1", "elephant\\head_eyes_closed");
            closeEyesImagesLeft = Image.Mirror(closeEyesImagesRight);
        }

        const double WalkFps = 15;

        RightWalkingAnimation = new Animation(bodyImagesRight) { FPS = WalkFps };
        LeftWalkingAnimation = new Animation(bodyImagesLeft) { FPS = WalkFps };

        lookingAnimation = new Animation(headImagesRight) { FPS = 3 };
        lookingAnimationLeft = new Animation(headImagesLeft) { FPS = 3 };
        closeEyesAnimation = new Animation(closeEyesImagesRight) { FPS = 1 };
        closeEyesLeft = new Animation(closeEyesImagesLeft) { FPS = 1 };

        head = new GameObject(width * 0.9, height * 0.9);
        SetHeadAnimation(lookingAnimation);
        PositionHeadHorizontally(Direction.Right);
        head.Y = this.Height * 0.15;
        Add(head);

        headAnimationTimer.Interval = RandomGen.NextDouble(1, 5);
        headAnimationTimer.Trigger += AnimateHead;
        headAnimationTimer.Start();

        DirectionChanged += PositionHeadHorizontally;
    }

    void AnimateHead(Timer timer)
    {
        int random = RandomGen.NextInt(100);
        if (random < 66)
        {
            SetHeadAnimation(closeEyesAnimation);
        }
        else
        {
            SetHeadAnimation(lookingAnimation);
        }

        timer.Interval = RandomGen.NextDouble(3, 10);
        head.Animation.Start(1);
    }

    void SetHeadAnimation(Animation rightFacingAnimation)
    {
        if (FacingDirection == Direction.Right)
        {
            head.Animation = rightFacingAnimation;
        }
        else
        {
            if (rightFacingAnimation == lookingAnimation)
                head.Animation = lookingAnimationLeft;
            else if (rightFacingAnimation == closeEyesAnimation)
                head.Animation = closeEyesLeft;
        }
    }

    void PositionHeadHorizontally(Direction direction)
    {
        double unit = (direction == Direction.Left) ? -1.0 : 1.0;
        head.X = unit * this.Width * 0.40;
        SetHeadAnimation(lookingAnimation);
    }
}
