﻿#region MIT License
/*
 * Copyright (c) 2009 University of Jyväskylä, Department of Mathematical
 * Information Technology.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#endregion

/*
 * Author: Antti-Jussi Lakanen, Jyväskylän yliopisto, 2010.
 */

using System;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.Assets;
using Jypeli.Effects;

/// <summary>
/// Tällä ohjelmalla voi kokeilla miltä räjähdysluokan
/// (ExplosionSystem) ilmentymä näyttää erilaisilla
/// parametreilla. Parametreja muutetaan näppäimillä.
/// Paina ohjelman käynnistyessä F1, niin saat
/// kontrolliohjeet.
/// Jypelin efektit: https://trac.cc.jyu.fi/projects/npo/wiki/Efektit
/// Nuorten peliohjelmointi: https://trac.cc.jyu.fi/projects/npo
/// </summary>
public class RajahdyksenParametritTesti : Game
{
    double minLifeTime = 0.5;
    double maxLifeTime = 1.0;
    double minVelocity = 80.0;
    double maxVelocity = 300.0;
    double minScale = 12;
    double maxScale = 64;
    double minRotationSpeed = 0.01;
    double maxRotationSpeed = 2.0;
    int particleAmount = 3;
    ExplosionSystem rajahdys;
    Timer rajaytysAjastin;

    public override void Begin()
    {
        Level.BackgroundColor = new Color(50, 50, 50);
        MessageDisplay.TextColor = Color.White;
        
        Image rajahdyskuva = LoadImage("bum2");
        rajahdys = new ExplosionSystem(rajahdyskuva, 5000);
        rajahdys.MinLifetime = minLifeTime;
        rajahdys.MaxLifetime = maxLifeTime;
        rajahdys.MinScale = minScale;
        rajahdys.MaxScale = maxScale;
        rajahdys.MinVelocity = minVelocity;
        rajahdys.MaxVelocity = maxVelocity;
        rajahdys.MinRotationSpeed = minRotationSpeed;
        rajahdys.MaxRotationSpeed = maxRotationSpeed;
        Add(rajahdys);

        rajaytysAjastin = new Timer();
        rajaytysAjastin.Interval = 2;
        rajaytysAjastin.Trigger += Rajayta;
        rajaytysAjastin.Start();

        Keyboard.Listen(Key.A, ButtonState.Pressed, MinLifeTime, "Pienennä MinLifeTime-arvoa", 0.66);
        Keyboard.Listen(Key.Q, ButtonState.Pressed, MinLifeTime, "Kasvata MinLifeTime-arvoa", 1.5);
        Keyboard.Listen(Key.S, ButtonState.Pressed, MaxLifeTime, "Pienennä MaxLifeTime-arvoa", 0.66);
        Keyboard.Listen(Key.W, ButtonState.Pressed, MaxLifeTime, "Kasvata MaxLifeTime-arvoa", 1.5);
        Keyboard.Listen(Key.D, ButtonState.Pressed, MinVelocity, "Pienennä MinVelocity-arvoa", 0.66);
        Keyboard.Listen(Key.E, ButtonState.Pressed, MinVelocity, "Kasvata MinVelocity-arvoa", 1.5);
        Keyboard.Listen(Key.F, ButtonState.Pressed, MaxVelocity, "Pienennä MaxVelocity-arvoa", 0.66);
        Keyboard.Listen(Key.R, ButtonState.Pressed, MaxVelocity, "Kasvata MaxVelocity-arvoa", 1.5);
        Keyboard.Listen(Key.G, ButtonState.Pressed, MinScale, "Pienennä MinScale-arvoa", 0.66);
        Keyboard.Listen(Key.T, ButtonState.Pressed, MinScale, "Kasvata MinScale-arvoa", 1.5);
        Keyboard.Listen(Key.H, ButtonState.Pressed, MaxScale, "Pienennä MaxScale-arvoa", 0.66);
        Keyboard.Listen(Key.Y, ButtonState.Pressed, MaxScale, "Kasvata MaxScale-arvoa", 1.5);
        Keyboard.Listen(Key.J, ButtonState.Pressed, ParticleAmount, "Pienennä partikkeleiden määrää", 0.66);
        Keyboard.Listen(Key.U, ButtonState.Pressed, ParticleAmount, "Kasvata partikkeleiden määrää", 1.5);
        Keyboard.Listen(Key.K, ButtonState.Pressed, MinRotationSpeed, "Pienennä MinRotationSpeed-arvoa", 0.66);
        Keyboard.Listen(Key.I, ButtonState.Pressed, MinRotationSpeed, "Kasvata MinRotationSpeed-arvoa", 1.5);
        Keyboard.Listen(Key.L, ButtonState.Pressed, MaxRotationSpeed, "Pienennä MaxRotationSpeed-arvoa", 0.66);
        Keyboard.Listen(Key.O, ButtonState.Pressed, MaxRotationSpeed, "Kasvata MaxRotationSpeed-arvoa", 1.5);

        Keyboard.Listen(Key.Up, ButtonState.Pressed, MuutaAjastimenNopeutta, "Nopeuta räjäytysajastimen intervallia", 0.66);
        Keyboard.Listen(Key.Down, ButtonState.Pressed, MuutaAjastimenNopeutta, "Hidasta räjäytysajastimen intervallia", 1.33);

        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
    }

    void Rajayta(Timer t)
    {
        rajahdys.AddEffect(0, 0, particleAmount);
    }

    void MinLifeTime(double kerroin)
    {
        minLifeTime = minLifeTime * kerroin + .01;
        rajahdys.MinLifetime = minLifeTime;
        MessageDisplay.Add("MinLifeTime: " + minLifeTime);
    }

    void MaxLifeTime(double kerroin)
    {
        maxLifeTime = maxLifeTime * kerroin + .01;
        rajahdys.MaxLifetime = maxLifeTime;
        MessageDisplay.Add("MaxLifeTime: " + maxLifeTime);
    }

    void MinVelocity(double kerroin)
    {
        minVelocity = minVelocity * kerroin + .01;
        rajahdys.MinVelocity = minVelocity;
        MessageDisplay.Add("MinVelocity: " + minVelocity);
    }

    void MaxVelocity(double kerroin)
    {
        maxVelocity = maxVelocity * kerroin + .01;
        rajahdys.MaxVelocity = maxVelocity;
        MessageDisplay.Add("MaxVelocity: " + maxVelocity);
    }

    void MinScale(double kerroin)
    {
        minScale = minScale * kerroin + .01;
        rajahdys.MinScale = minScale;
        MessageDisplay.Add("MinScale: " + minScale);
    }

    void MaxScale(double kerroin)
    {
        maxScale = maxScale * kerroin + .01;
        rajahdys.MaxScale = maxScale;
        MessageDisplay.Add("MaxScale: " + maxScale);
    }

    void ParticleAmount(double kerroin)
    {
        particleAmount = (int)Math.Floor(particleAmount * kerroin) + 1;
        MessageDisplay.Add("Partikkeleiden määrä: " + particleAmount);
    }

    void MinRotationSpeed(double kerroin)
    {
        minRotationSpeed = minRotationSpeed * kerroin + .01;
        rajahdys.MinRotationSpeed = minRotationSpeed;
        MessageDisplay.Add("MinRotationSpeed: " + minRotationSpeed);
    }

    void MaxRotationSpeed(double kerroin)
    {
        maxRotationSpeed = maxRotationSpeed * kerroin + .01;
        rajahdys.MaxRotationSpeed = maxRotationSpeed;
        MessageDisplay.Add("MaxRotationSpeed: " + maxRotationSpeed);
    }

    void MuutaAjastimenNopeutta(double kerroin)
    {
        rajaytysAjastin.Interval = rajaytysAjastin.Interval * kerroin + .01;
        MessageDisplay.Add("Räjäytysajastin: " + rajaytysAjastin.Interval + " sekuntia");
    }
}