﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;

public class Peli : PhysicsGame
{
    GameObject npc1, npc2, npc3, npc4;
    List<Vector> waypoints1 = new List<Vector>();
    List<Vector> waypoints2 = new List<Vector>();
    List<Vector> waypoints3 = new List<Vector>();

    public override void Begin()
    {
        TileMap kentta = TileMap.FromFile( "kentta.txt" );
        kentta.SetTileMethod( 'p', LuoPelaaja );
        kentta.SetTileMethod( '1', LuoNPC1 );
        kentta.SetTileMethod( '2', LuoNPC2 );
        kentta.SetTileMethod( '3', LuoNPC3 );
        kentta.SetTileMethod( '4', LuoNPC4 );
        kentta.SetTileMethod( 'a', LisaaWaypoint1 );
        kentta.SetTileMethod( 'b', LisaaWaypoint2 );
        kentta.Execute(40, 40);

        LisaaAivot( npc1, waypoints1 );
        LisaaAivot( npc2, waypoints2 );

        TileMap reitti3 = TileMap.FromFile( "reitti3.txt" );
        reitti3.SetTileMethod( 'c', LisaaWaypoint3 );
        reitti3.Execute( 40, 40 );
        LisaaAivot( npc3, waypoints3 );

        npc4.Brain = new TagFollowerBrain( "follow", 0.5 );

        Camera.ZoomToLevel();
    }

    private void LisaaAivot( GameObject npc, List<Vector> waypoints )
    {
        Vector[] path1 = ParsiReitti( npc, waypoints );
        PathFollowerBrain aivot1 = new PathFollowerBrain( path1 );
        aivot1.Speed = 1000;
        aivot1.Loop = true;
        npc.Brain = aivot1;
    }

    Vector[] ParsiReitti(GameObject npc, List<Vector> waypoints)
    {
        int n = waypoints.Count;
        Vector[] reitti = new Vector[n];
        Vector v = npc.Position;

        for ( int i = 0; i < n; i++ )
        {
            v = FindNearest( waypoints, v );
            waypoints.Remove( v );
            reitti[i] = v;
        }

        return reitti;
    }

    Vector FindNearest(IEnumerable<Vector> list, Vector vector)
    {
        Vector nearest = Vector.Zero;
        double minDistance = double.PositiveInfinity;

        foreach ( Vector candidate in list )
        {
            double d = Vector.Distance( vector, candidate );

            if ( d < minDistance )
            {
                nearest = candidate;
                minDistance = d;
            }
        }

        return nearest;
    }

    void LuoPelaaja(Vector paikka, double leveys, double korkeus)
    {
        GameObject pelaaja = new GameObject(leveys, korkeus, Shape.Circle);
        pelaaja.Position = paikka;
        pelaaja.Tag = "follow";
        Add( pelaaja );

        Keyboard.Listen( Key.Left, ButtonState.Down, pelaaja.Move, null, -3 * Vector.UnitX );
        Keyboard.Listen( Key.Right, ButtonState.Down, pelaaja.Move, null, 3 * Vector.UnitX );
        Keyboard.Listen( Key.Up, ButtonState.Down, pelaaja.Move, null, 3 * Vector.UnitY );
        Keyboard.Listen( Key.Down, ButtonState.Down, pelaaja.Move, null, -3 * Vector.UnitY );
    }

    void LuoNPC1( Vector paikka, double leveys, double korkeus )
    {
        npc1 = new GameObject( leveys, korkeus, Shape.Triangle );
        npc1.Position = paikka;
        npc1.Color = Color.Green;
        npc1.Tag = "follow";
        Add( npc1 );
    }

    void LuoNPC2( Vector paikka, double leveys, double korkeus )
    {
        npc2 = new GameObject( leveys, korkeus, Shape.Star );
        npc2.Position = paikka;
        npc2.Color = Color.Yellow;
        npc2.Tag = "follow";
        Add( npc2 );
    }

    void LuoNPC3( Vector paikka, double leveys, double korkeus )
    {
        npc3 = new GameObject( leveys, korkeus, Shape.Heart );
        npc3.Position = paikka;
        npc3.Color = Color.Red;
        npc3.Tag = "follow";
        Add( npc3 );
    }

    void LuoNPC4( Vector paikka, double leveys, double korkeus )
    {
        npc4 = new GameObject( leveys, korkeus, Shape.Rectangle );
        npc4.Position = paikka;
        npc4.Color = Color.Blue;
        Add( npc4 );
    }

    void LisaaWaypoint1( Vector paikka, double leveys, double korkeus )
    {
        waypoints1.Add( paikka );
    }

    void LisaaWaypoint2( Vector paikka, double leveys, double korkeus )
    {
        waypoints2.Add( paikka );
    }

    void LisaaWaypoint3( Vector paikka, double leveys, double korkeus )
    {
        waypoints3.Add( paikka );
    }
}
