﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;

namespace Jypeli.SimplePhysics
{
    public struct BoundingRectangle
    {
        public double X;
        public double Y;
        public double Width;
        public double Height;

        public double Left { get { return X - Width / 2; } }
        public double Right { get { return X + Width / 2; } }
        public double Bottom { get { return Y - Height / 2; } }
        public double Top { get { return Y + Height / 2; } }

        public Vector TopLeft
        {
            get { return new Vector( X - Width / 2, Y + Height / 2 ); }
        }

        public Vector BottomRight
        {
            get { return new Vector( X + Width / 2, Y - Height / 2 ); }
        }

        public BoundingRectangle(double x, double y, double w, double h)
        {
            X = x;
            Y = y;
            Width = w;
            Height = h;
        }

        public BoundingRectangle( Vector topLeft, Vector bottomRight )
        {
            Width = bottomRight.X - topLeft.X;
            Height = topLeft.Y - bottomRight.Y;
            X = topLeft.X + Width / 2;
            Y = bottomRight.Y + Height / 2;
        }

        public static bool Intersects( BoundingRectangle a, BoundingRectangle b )
        {
            bool xcond = a.Right >= b.Left && a.Left <= b.Right || b.Right >= a.Left && b.Left <= a.Right;
            bool ycond = a.Top >= b.Bottom && a.Bottom <= b.Top || b.Top >= a.Bottom && b.Bottom <= a.Top;
            return xcond && ycond;
        }

        public static BoundingRectangle GetIntersection( BoundingRectangle a, BoundingRectangle b )
        {
            double minx = Math.Min( a.X, b.X );
            double maxx = Math.Max( a.X, b.X );
            double miny = Math.Min( a.Y, b.Y );
            double maxy = Math.Max( a.Y, b.Y );

            double iw = maxx - minx;
            double ih = maxy - miny;

            return new BoundingRectangle( minx + iw / 2, miny + ih / 2, iw, ih );
        }

        public static Direction GetIntersectionDirection( BoundingRectangle rect, BoundingRectangle intersection )
        {
            double dx = rect.X - intersection.X;
            double dy = rect.Y - intersection.Y;
            double adx = Math.Abs( dx );
            double ady = Math.Abs( dy );

            if ( adx > ady )
            {
                if ( dx < 0 ) return Direction.Left;
                return Direction.Right;
            }

            if ( dy < 0 ) return Direction.Down;
            return Direction.Up;
        }
    }
}
