﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Jypeli;

namespace Jypeli.SimplePhysics
{
    public class SimplePhysicsObject : GameObject
    {
        internal ImpulseCollection Impulses = new ImpulseCollection();
        internal double MassInv;

        private double _mass;
        private BoundingRectangle _boundingRect = new BoundingRectangle();

        public double Mass
        {
            get { return _mass; }
            set
            {
                if ( value <= 0 ) throw new ArgumentException( "Negative or zero mass encountered!" );
                _mass = value;
                MassInv = 1 / value;
            }
        }

        public Vector Velocity { get; internal set; }
        public Vector Acceleration { get; internal set; }

        public BoundingRectangle BoundingRectangle
        {
            get
            {
                _boundingRect.X = X;
                _boundingRect.Y = Y;
                _boundingRect.Width = Width;
                _boundingRect.Height = Height;
                return _boundingRect;
            }
        }

        public SimplePhysicsObject(double width, double height)
            : base(width, height)
        {
            Mass = 1.0;
        }

        public void ApplyImpulse( Impulse impulse )
        {
            Impulses.Add( impulse );
        }
    }
}
