﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.Assets;

namespace WidgetTest
{
    public class Peli : Game
    {
        Font customFont = LoadFont("CustomFont");

        public override void Begin()
        {
            ClearAll();

            Label helppi = new Label( "Testaillaan widgettejä" );
            helppi.Color = Color.Yellow;
            Add( helppi );

            Mouse.IsCursorVisible = true;
            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Exit, "Poistu" );
            Keyboard.Listen( Key.D1, ButtonState.Pressed, TestLabels, "Testaa labeleita" );
            Keyboard.Listen( Key.D2, ButtonState.Pressed, TestBorders, "Testaa reunoja" );
            ShowControlHelp();
        }

        void TestBorders()
        {
            ClearAll();

            Shape[] shapes =
            {
                Shape.Circle,
                Shape.Triangle,
                Shape.Rectangle,
                Shape.Heart,
                Shape.Hexagon,
                Shape.Octagon,
                Shape.Pentagon,
                Shape.Star,
            };

            var layout = new RowLayout { TargetWidth = 400, HorizontalSpacing = 50, VerticalSpacing = 50 };
            Widget parent = new Widget( layout ) { Color = Color.Transparent };
            Add( parent );

            for ( int i = 0; i < shapes.Length; i++ )
            {
                Widget w = new Widget( 100, 100, shapes[i] ) { Color = Color.LightGreen, BorderColor = Color.Black };
                parent.Add( w );
            }

            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Begin, "Back" );
        }

        void TestLabels()
        {
            ClearAll();

            Label[,] labels = new Label[3, 3];
            double margin = 100;
            double padding = 200;

            for ( int y = 0; y < labels.GetLength( 0 ); y++ )
            {
                for ( int x = 0; x < labels.GetLength( 1 ); x++ )
                {
                    HorizontalAlignment horizontal;
                    VerticalAlignment vertical;
                    switch ( x )
                    {
                        case 0:
                            horizontal = HorizontalAlignment.Left;
                            break;
                        case 1:
                            horizontal = HorizontalAlignment.Center;
                            break;
                        default:
                            horizontal = HorizontalAlignment.Right;
                            break;
                    }
                    switch ( y )
                    {
                        case 0:
                            vertical = VerticalAlignment.Top;
                            break;
                        case 1:
                            vertical = VerticalAlignment.Center;
                            break;
                        default:
                            vertical = VerticalAlignment.Bottom;
                            break;
                    }

                    Label l = labels[y, x] = new Label( 200, 100, "TEST" );
                    l.BorderColor = Color.Black;
                    l.HorizontalAlignment = horizontal;
                    l.VerticalAlignment = vertical;
                    l.X = Screen.Left + margin + l.Width / 2 + x * ( l.Width + padding );
                    l.Y = Screen.Top - margin - l.Height / 2 - y * ( l.Height + padding );
                    Add( l );
                }
            }

            string blaaBlaa = "Tässä labelissa on niin paljon tekstiä, että se ei meinaakaan " +
                "mahtua kerralla yhdelle riville. Niinpä sitä täytyy rivittää useammalle riville, " +
                "jotta teksti näyttäisi hyvältä ja olisi luettavaa.";
            Label paljonTekstiä = new Label( 200, 100 );
            paljonTekstiä.SizeMode = TextSizeMode.Wrapped;
            paljonTekstiä.Text = blaaBlaa;
            paljonTekstiä.X = Screen.Right - paljonTekstiä.Width / 2;
            paljonTekstiä.Y = Screen.Top - paljonTekstiä.Height / 2;
            Add( paljonTekstiä );

            Angle angle = Angle.FromDegrees( 10 );

            Keyboard.Listen( Key.Escape, ButtonState.Pressed, Begin, "Back" );
            Keyboard.Listen( Key.Left, ButtonState.Pressed, Rotate, "Rotate counterclockwise", labels, angle );
            Keyboard.Listen( Key.Right, ButtonState.Pressed, Rotate, "Rotate clockwise", labels, -angle );
            Keyboard.Listen(Key.D1, ButtonState.Pressed, SetSmallFont, "Small font", labels);
            Keyboard.Listen(Key.D2, ButtonState.Pressed, SetDefaultFont, "Default font", labels);
            Keyboard.Listen(Key.D3, ButtonState.Pressed, SetLargeFont, "Large font", labels);
            Keyboard.Listen(Key.D4, ButtonState.Pressed, SetCustomFont, "Custom font", labels);
            ShowControlHelp();
        }

        static void ForAll(Label[,] labels, Action<Label> f)
        {
            for (int y = 0; y < labels.GetLength(0); y++)
            {
                for (int x = 0; x < labels.GetLength(1); x++)
                {
                    f(labels[y, x]);
                }
            }
        }

        void SetDefaultFont(Label[,] labels)
        {
            ForAll(labels, l => l.Font = Font.Default);
        }

        void SetSmallFont(Label[,] labels)
        {
            ForAll(labels, l => l.Font = Font.DefaultSmall);
        }

        void SetLargeFont(Label[,] labels)
        {
            ForAll(labels, l => l.Font = Font.DefaultLarge);
        }

        void SetCustomFont(Label[,] labels)
        {
            ForAll(labels, l => l.Font = customFont);
        }

        void Rotate( Widget w, Angle delta )
        {
            w.Angle += delta;
        }

        void Rotate( Label[,] labelit, Angle delta )
        {
            ForAll(labelit, l => Rotate(l, delta));
        }
    }
}
